/* Copyright (c) 2019 Asustor Inc. All rights reserved. */

/**
 * @class AS.ARC.apps.datasynccenter.syncList
 * @extends Ext.view.View
 */
Ext.define('AS.ARC.apps.datasynccenter.syncList', {
    extend: 'Ext.view.View',
    alias: ['widget.syncList'],
    cls: 'sync-list-grid',
    flex: 1,

    itemSelector: 'div.sync-list-item',
    tpl: new Ext.XTemplate(
        '<tpl for=".">',
            '<div class="sync-list-item">',
                '<div class="sync-list-item-row">',
                    '<div class="sync-type type-{service_type}">',
                    '</div>',
                    '<div class="sync-title" data-qtip="{[this.getTitle(values)]}">',
                        '{[this.getTitle(values)]}',
                    '</div>',
                    '<div class="sync-status {status}">',
                    '</div>',
                '</div>',
            '</div>',
        '</tpl>', {
            getTitle: function(values) {
                return values.connection_name;
            }
        }),

    pollingID: 0,
    ajaxRequest: {},
    lastSelected: 0,
    reauthorizeList: [],
    reauthorizeErrorList: {
        5603: []
    },

    initComponent: function() {
        this.store = this.createStore();
        this.callParent(arguments);
    },

    createStore: function() {
        var view = this;
        return Ext.create('Ext.data.Store', {
            fields: [
                'connection_id',
                'connection_name',
                'service_type',
                'user_name',
                'status',
                'running',
                'has_task_error',
                'unsync_files',
                'failed_list',
                'schedule_status',
                'quota',
                'normal',
                'available',
                'trans_status',
            ],
            listeners: {
                load: function(store) {
                    view.unmaskView();
                    if (store.getCount() > 0) {
                        if (view.lastSelected > store.getCount() - 1) view.lastSelected = 0;
                        view.getSelectionModel().select(view.lastSelected);
                    } else {
                        if (view.fn.syncTabPanel.activeTab.getItemId() === 'status-panel') {
                            view.fn.syncTabPanel.activeTab.loadData();
                        } else {
                            view.fn.syncTabPanel.setActiveTab(0);
                        }
                        view.fn.openIntroWindow();
                    }

                    Ps.initialize(view.el.dom);
                }
            }
        });
    },

    stopPolling: function() {
        var view = this;
        if (!Ext.isEmpty(view.ajaxRequest)) {
            Ext.Ajax.abort(view.ajaxRequest);
        }
        clearTimeout(view.pollingID);
    },

    startPolling: function() {
        var view = this;
        view.stopPolling();
        view.pollingID = Ext.defer(function() {
            if (view.fn.win.isDestroyed) return false;
            var connection_id = view.getData().connection_id,
                params = {
                    act: 'list_connection_status',
                    list_trans_status: connection_id
                };

            view.ajaxRequest = AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(view.fn.apiUrl, params),
                method: 'post',
                success: function(json) {
                    if (view.fn.win.isDestroyed) return false;
                    view.store.each(function(record, index) {
                        Ext.iterate(record.data, function(key, value) {
                            if (JSON.stringify(value) !== JSON.stringify(json.list[index][key])) {
                                record.set(key, json.list[index][key]);
                            }
                            if (key === 'trans_status' && record.data.connection_id === connection_id) {
                                record.set(key, json.trans_status);
                            }
                            if (key === 'connection_id' && view.reauthorizeList.indexOf(value) !== -1) {
                                view.reauthorize(record);
                            }
                        });
                        if (record.dirty) {
                            record.commit();
                            if (record === view.getSelectionModel().getLastSelected()) {
                                view.fireEvent('connectionupdate', 'connection-update');
                            }
                        }
                    });
                    view.fireEvent('updateuploadingtasklist');
                    view.startPolling();
                },
                failure: function() { }
            });
        }, 1000);
    },

    loadSyncList: function() {
        var view = this;
        view.stopPolling();
        view.maskView();
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(view.fn.apiUrl, {act: 'list_connection_status'}),
            method: 'post',
            success: function (json) {
                view.loadSyncListStore(json.list);
            },
            failure: function (json) {
                AS.ARC.util.showMsgWindow({}, json, view.fn.win, '', function() {
                    view.loadSyncListStore([]);
                });
            }
        });
    },

    loadSyncListStore: function(list) {
        var view = this;
        view.store.loadRawData(list);
        if (list.length !== 0) view.startPolling();
    },

    getData: function() {
        var view = this,
            record = view.getSelectionModel().getLastSelected();
        return record ? record.data : {};
    },

    getConnectionId: function() {
        var view = this;
        return view.getData().connection_id;
    },

    maskView: function() {
        this.fn.westPanel.el.mask(_S('COMMON', 'LOADING'));
        this.fn.syncTabPanel.el.mask(_S('COMMON', 'LOADING'));
    },

    unmaskView: function() {
        this.fn.westPanel.el.unmask();
        this.fn.syncTabPanel.el.unmask();
    },

    reauthorize: function(record) {
        var view = this,
            list = view.reauthorizeList,
            index = list.indexOf(record.data.connection_id);

        var removeReauthorizeList = function(index) {
            if (index < 0) return false;
            list.splice(index, 1);
        };

        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(view.fn.apiUrl, { act: 'reset_authorize' }),
            params: {
                connection_id: record.data.connection_id
            },
            method: 'post',
            success: function(json) {
                if (json.status === 'finish') {
                    removeReauthorizeList(index);
                }
            },
            failure: function(json) {
                removeReauthorizeList(index);
                view.reauthorizeErrorList[json.error_code].push(record.data.connection_id);
                if (record === view.getSelectionModel().getLastSelected()) {
                    view.fireEvent('connectionupdate', 'connection-update');
                }
            }
        });
    },

    listeners: {
        selectionchange: function(model, selections) {
            var view = this;
            if (selections.length === 0) return false;

            view.fn.syncTabPanel.activeTab.loadData('sync-list-change');
            view.lastSelected = view.store.indexOf(model.getLastSelected());
        },
        connectionupdate: function(type) {
            var view = this;
            view.fn.syncTabPanel.activeTab.loadData(type);
        },
        updateuploadingtasklist: function() {
            var view = this,
                uploadingTaskListSubWindow = view.fn.win.down('#status-panel').uploadingTaskListSubWindow;

            if (uploadingTaskListSubWindow && !uploadingTaskListSubWindow.isDestroyed) {
                uploadingTaskListSubWindow.loadData(view.getData().trans_status);
            }
        },
        beforecontainerclick: function() {
            return false;
        },
        resize: function(view) {
            Ps.update(view.el.dom);
        }
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.syncListToolbar.wizard.serviceList
 * @extends Ext.view.View
 */
Ext.define('AS.ARC.apps.datasynccenter.syncListToolbar.wizard.serviceList', {
    extend: 'Ext.view.View',
    alias: ['widget.serviceList'],
    cls: 'service-list-grid',
    height: 310,

    itemSelector: 'div.service-list-item',
    tpl: new Ext.XTemplate(
        '<tpl for=".">',
            '<div class="service-list-item">',
                '<div class="service-icon service-{service}"></div>',
                '<div class="service-name" data-qtip="{name}">',
                    '{name}',
                '</div>',
                '<div class="service-check x-grid-checkheader"></div>',
            '</div>',
        '</tpl>'),

    SERVICE_LIST: [],

    initComponent: function() {
        this.initServiceList();
        this.store = this.createStore();
        this.callParent(arguments);
    },

    initServiceList: function() {
        this.SERVICE_LIST = [
            ['ezsync', _S('DATASYNC_CENTER', 'ASUSTOR_NAS')],
            ['baidu', _S('DATASYNC_CENTER', 'BAIDU')],
            ['dropbox', _S('DATASYNC_CENTER', 'DROPBOX')],
            ['googledrive', _S('DATASYNC_CENTER', 'GOOGLE_DRIVE')],
            ['onedrive', _S('DATASYNC_CENTER', 'ONE_DRIVE')],
            ['yandex', _S('DATASYNC_CENTER', 'YANDEX')]
        ];
    },

    createStore: function() {
        var view = this;
        return Ext.create('Ext.data.ArrayStore', {
            fields: [
                'service',
                'name'
            ],
            data: view.SERVICE_LIST,
            listeners: {
                load: function(store) {
                    Ext.defer(function() {
                        Ps.initialize(view.el.dom);
                    }, 100);
                }
            }
        });
    },

    getService: function() {
        var view = this;
        return view.getSelectionModel().getLastSelected();
    },

    listeners: {
        viewready: function(view) {
            view.getSelectionModel().select(0);
        },
        selectionchange: function(model, selections) {
            var view = this;
            view.wizard.onSelectionChange();
        },
        beforeselect: function(model, node) {
            var view = this,
                selectedNode = model.getLastSelected();
            if (!Ext.isEmpty(selectedNode)) {
                view.getNode(selectedNode).querySelector('.service-check').toggleClassName('x-grid-checkheader-checked');
            }
            view.getNode(node).querySelector('.service-check').toggleClassName('x-grid-checkheader-checked');
        },
        beforecontainerclick: function() {
            return false;
        }
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.syncTree
 * @extends Ext.container.Container
 */
Ext.define('AS.ARC.apps.datasynccenter.syncTree', {
    extend: 'Ext.container.Container',
    alias: ['widget.syncTree'],

    cancelAjax: true,
    type: '',
    connectionId: 0,
    localPath: '',
    remotePath: '',
    localFolder: '',
    remoteFolder: '',
    includePath: [],
    excludePath: [],

    FOLDER_ICON_CLS: {
        two_way: 'two-way-folder',
        upload: 'local-folder',
        download: 'remote-folder'
    },
    FILE_ICON_CLS: {
        two_way: 'two-way-file',
        upload: 'local-file',
        download: 'remote-file'
    },
    TREE_ROOT: {
        download: '<span class="root-wrap"><span class="root-icon root-remote-icon"></span>{1}</span>',
        upload: '<span class="root-wrap"><span class="root-icon root-local-icon"></span>{0}</span>',
        two_way: '<span class="root-wrap"><span class="root-icon root-local-icon"></span>{0}<span class="root-icon root-two-way-icon"></span><span class="root-icon root-remote-icon"></span>{1}</span>'
    },
    TRAVERSAL_TREE_ROOT: {
        download: '<span class="root-wrap"><span class="root-icon root-remote-icon"></span>{1}<span class="root-icon root-to-icon"></span><span class="root-icon root-local-icon"></span>{0}</span>',
        upload: '<span class="root-wrap"><span class="root-icon root-local-icon"></span>{0}<span class="root-icon root-to-icon"></span><span class="root-icon root-remote-icon"></span>{1}</span>',
        two_way: '<span class="root-wrap"><span class="root-icon root-local-icon"></span>{0}<span class="root-icon root-two-way-icon"></span><span class="root-icon root-remote-icon"></span>{1}</span>'
    },
    API_URL: AS.ARC.util.getUserAppsPath() + 'datasync-center/datasync_center.cgi',

    EXPAND_LIST: [],
    MSG_TYPE_ERROR: 'error',
    MSG_TYPE_WARNING: 'warning',
    tri: {
        NONE: 0,
        HALF: 1,
        ALL: 2
    },

    initComponent: function() {
        this.items = [this.getItems()];
        this.init();
        this.callParent(arguments);
    },

    init: function() {
        this.EXPAND_LIST = [];
    },

    getItems: function() {
        return [{
            xtype: 'asFlexbox',
            itemId: 'sync-direction-mark',
            cls: 'sync-direction-mark',
            hidden: (this.type === 'two_way' || this.markTitle) ? false : true,
            items: [{
                xtype: 'displayfield',
                value: _S('DATASYNC_CENTER', 'SELECT_FILES_FOLDERS'),
                width: 350,
                hidden: this.markTitle ? false : true
            }, {
                xtype: 'asFlexbox',
                itemId: 'local',
                items: [{
                    xtype: 'box',
                    cls: 'local-mark'
                }, {
                    xtype: 'box',
                    html: _S('DATASYNC_CENTER', 'LOCAL')
                }]
            }, {
                xtype: 'asFlexbox',
                itemId: 'remote',
                items: [{
                    xtype: 'box',
                    cls: 'remote-mark'
                }, {
                    xtype: 'box',
                    html: _S('DATASYNC_CENTER', 'REMOTE')
                }]
            }, {
                xtype: 'asFlexbox',
                itemId: 'two-way',
                items: [{
                    xtype: 'box',
                    cls: 'two-way-mark'
                }, {
                    xtype: 'box',
                    html: _S('DATASYNC_CENTER', 'TWO_WAY')
                }]
            }]
        }, this.getTree()];
    },

    checkParent: function(node) {
        var me = this;

        node.bubble(function(parent) {
            var count = 0,
                half = false;
            if (parent.data.id == node.data.id)
                return true;

            parent.eachChild(function(child) {
                if (child.data.tri !== me.tri.NONE) {
                    if (child.data.tri === me.tri.HALF) {
                        half = true;
                    }
                    count++;
                }
            });

            if (count === parent.childNodes.length) {
                if (half) {
                    parent.set('checked', false);
                    parent.set('tri', me.tri.HALF);
                    parent.set('cls', 'x-tree-checked-down');
                } else {
                    parent.set('checked', true);
                    parent.set('tri', me.tri.ALL);
                    parent.set('cls', '');
                }
            } else if (count > 0) {
                parent.set('checked', false);
                parent.set('tri', me.tri.HALF);
                parent.set('cls', 'x-tree-checked-down');
            } else {
                parent.set('checked', false);
                parent.set('tri', me.tri.NONE);
                parent.set('cls', '');
            }
        });
    },

    checkChild: function(node) {
        var me = this;
        node.cascadeBy(function(child) {
            if (child.data.id == node.data.id)
                return true;

            child.set('checked', node.data.checked);
            child.set('tri', node.data.checked ? me.tri.ALL : me.tri.NONE);
            child.set('cls', '');
        });
    },

    abortRequest: function() {
        var me = this;
        if (Ext.isEmpty(me.request)) {
            return;
        }
        Ext.Ajax.abort(me.request);
    },

    maskTreePanel: function() {
        var me = this,
            mask;

        if (me.cancelAjax) {
            mask = me.tree.el.mask(Ext.String.format(_S('DATASYNC_CENTER', 'PLEASE_WAIT_OR_CANCEL'), '<a class="hyperlink-string" href="#">', '</a>'));
            mask.next().on('click', function() {
                me.abortRequest();
                me.selectedNode.collapse();
                me.selectedNode.set('loading', false);
                me.selectedNode.set('expanded', false);
                me.selectedNode.set('loaded', false);
                var index = me.EXPAND_LIST.indexOf(me.selectedNode);
                if (index >= 0) {
                    me.EXPAND_LIST.splice(index, 1);
                }
                me.tree.el.unmask();
            }, this, {
                delegate: '.hyperlink-string'
            });
        }
    },

    unmaskTreePanel: function() {
        var me = this;

        if (me.cancelAjax) {
            me.tree.el.unmask();
        }
    },

    renderTree: function(node, callback) {
        var me = this,
            urlOpts = {
                act: 'list_folder',
                connection_id: me.connectionId
            },
            isRoot = node.data.path ? false : true;

        if (me.type === 'two_way') {
            if (Ext.isDefined(node.data.local_id)) {
                Ext.apply(urlOpts, { local_id: node.data.local_id });
            }
            if (Ext.isDefined(node.data.remote_id)) {
                Ext.apply(urlOpts, { remote_id: node.data.remote_id });
            }
            
        } else if (me.type === 'upload') {
            Ext.apply(urlOpts, { local_id: node.data.local_id });
        } else if (me.type === 'download') {
            Ext.apply(urlOpts, { remote_id: node.data.remote_id });
        }

        node.removeAll();
        node.set('loading', true);
        me.selectedNode = node;
        me.maskTreePanel();
        me.request = AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(me.API_URL, urlOpts),
            method: 'post',
            success: function(json, opts) {
                var lists = json.list,
                    fileIconCls = me.FILE_ICON_CLS[me.type],
                    fileText = isRoot ? _S('DATASYNC_CENTER', 'ONLY_FILES_IN_THIS_FOLDER') : Ext.String.format(_S('DATASYNC_CENTER', 'FILES_IN'), node.data.text),
                    leafNode = {
                        text: fileText,
                        leaf: true,
                        qtip: Ext.htmlEncode(fileText),
                        checked: false,
                        iconCls: fileIconCls,

                        tri: me.tri.NONE,
                        path: node.data.path + '/*'
                    };

                if (me.type === 'two_way' && !isRoot) {
                    if (Ext.isDefined(node.data.local_id) && Ext.isDefined(node.data.remote_id)) {
                        Ext.apply(leafNode, { iconCls: me.FILE_ICON_CLS[me.type] });
                    } else if (Ext.isDefined(node.data.local_id)) {
                        Ext.apply(leafNode, { iconCls: me.FILE_ICON_CLS['upload'] });
                    } else if (Ext.isDefined(node.data.remote_id)) {
                        Ext.apply(leafNode, { iconCls: me.FILE_ICON_CLS['download'] });
                    }
                }

                node.appendChild(leafNode);
                Ext.each(lists, function(data) {
                    var nodeConfig = {
                            text: Ext.htmlEncode(data.path),
                            leaf: false,
                            id: data.path,
                            qtip: Ext.htmlEncode(Ext.htmlEncode(data.path)),
                            checked: false,
                            iconCls: me.FOLDER_ICON_CLS[me.type],

                            tri: me.tri.NONE,
                            path: node.data.path + '/' + data.path
                        };
                    if (me.type === 'two_way') {
                        if (Ext.isDefined(data.local_id) && Ext.isDefined(data.remote_id)) {
                            Ext.apply(nodeConfig, {
                                iconCls: me.FOLDER_ICON_CLS[me.type],
                                remote_id: data.remote_id,
                                local_id: data.local_id
                            });
                        } else if (Ext.isDefined(data.local_id)) {
                            Ext.apply(nodeConfig, {
                                iconCls: me.FOLDER_ICON_CLS['upload'],
                                local_id: data.local_id
                            });
                        } else if (Ext.isDefined(data.remote_id)) {
                            Ext.apply(nodeConfig, {
                                iconCls: me.FOLDER_ICON_CLS['download'],
                                remote_id: data.remote_id
                            });
                        }
                    } else if (me.type === 'upload') {
                        Ext.apply(nodeConfig, {
                            local_id: data.local_id
                        });
                    } else if (me.type === 'download') {
                        Ext.apply(nodeConfig, {
                            remote_id: data.remote_id
                        });
                    }
                    node.appendChild(nodeConfig);
                });

                node.set('loading', false);
                me.checkChild(node);
                if (Ext.isFunction(callback)) {
                    callback();
                    Ps.update(me.tree.body.dom);
                    me.unmaskTreePanel();
                } else {
                    setTimeout(function() {
                        Ps.update(me.tree.body.dom);
                        me.unmaskTreePanel();
                    }, 200);
                }
            },
            failure: function(json, opts) {
                node.set('loading', false);
                node.set('loaded', false);
                node.set('expanded', false);
                if (me.EXPAND_LIST.indexOf(node) !== -1) {
                    me.EXPAND_LIST.splice(me.EXPAND_LIST.indexOf(node), 1);
                }
                me.unmaskTreePanel();
                AS.ARC.util.showMsgWindow({
                    5001: _S('DATASYNC_CENTER', 'ACCOUNT_REVOKED_EXPIRED')
                }, json, me.parentWin);
            }
        });
    },

    getTree: function() {
        var me = this;

        me.tree = Ext.create('Ext.tree.TreePanel', {
            cls: 'sync-tree',
            viewConfig: {
                rootVisible: true,
            },
            useArrows: true,
            autoScroll: true,
            height: me.treeHeight || 260,
            flex: 1,
            resizable: false,
            scope: this,
            listeners: {
                afterrender: function(com, eOpts) {
                    if (Ext.isEmpty(me.type)) return false;
                    var rootNode = {
                        text: Ext.String.format(me.TREE_ROOT[me.type], Ext.htmlEncode(me.localFolder), Ext.htmlEncode(me.remoteFolder)),
                        expanded: false,
                        checked: true,
                        icon: Ext.BLANK_IMAGE_URL,

                        remote_id: me.remotePath,
                        local_id: me.localPath,
                        path: '',
                        tri: me.tri.NONE
                    };
                    me.tree.setRootNode(rootNode);
                    me.EXPAND_LIST.push(me.tree.getRootNode());
                    me.renderTree(me.tree.getRootNode());
                },
                itemexpand: function(node, eOpts) {
                    if (me.EXPAND_LIST.indexOf(node) === -1) {
                        me.EXPAND_LIST.push(node);
                        me.renderTree(node);
                    }
                },
                checkchange: function(node, checked) {
                    node.set('tri', checked ? me.tri.ALL : me.tri.NONE);
                    node.set('cls', '');
                    me.checkParent(node);
                    me.checkChild(node);
                },
                beforeselect: function(mode, record) {
                    record.set('checked', !record.data.checked);
                    record.commit();
                    me.tree.fireEvent('checkchange', record, record.data.checked);
                    return false;
                }
            }
        });

        return me.tree;
    },

    setLocalPath: function(objectPath) {
        this.localPath = objectPath.getValue();
        this.setLocalFolder(objectPath.getValue().split('/').pop());
    },

    setLocalFolder: function(value) {
        this.localFolder = value;
    },

    setRemotePath: function(objectPath) {
        this.remotePath = objectPath.getRemoteId();
        this.setRemoteFolder(objectPath.getValue().split('/').pop());
    },

    setRemoteFolder: function(value) {
        this.remoteFolder = value;
    },

    refreshTree: function(mode) {
        var me = this,
            rootFormat = me.TREE_ROOT;
        
        if (mode === 'edit') {
            rootFormat = me.TRAVERSAL_TREE_ROOT;
        }

        if (Ext.isEmpty(me.tree)) return false;
        var rootNode = {
            text: Ext.String.format(rootFormat[me.type], Ext.htmlEncode(me.localFolder), Ext.htmlEncode(me.remoteFolder)),
            expanded: false,
            checked: true,
            icon: Ext.BLANK_IMAGE_URL,

            remote_id: me.remotePath,
            local_id: me.localPath,
            path: '',
            tri: me.tri.NONE
        };
        me.tree.setRootNode(rootNode);
        me.EXPAND_LIST = [];
        me.EXPAND_LIST.push(me.tree.getRootNode());
        if (mode === 'edit') {
            me.renderTree(me.tree.getRootNode(), function() {
                me.walkTree();
            });
        } else {
            me.renderTree(me.tree.getRootNode());
        }
    },

    getPath: function() {
        var me = this,
            path = {
                include_path: [],
                exclude_path: []
            };
        me.tree.getRootNode().cascadeBy(function(node) {
            if (me.tree.getRootNode() === node) {
                path[node.firstChild.data.checked ? 'include_path' : 'exclude_path'].push('/');
                return true;
            }

            if (node.data.leaf) { return true; }

            if (node.hasChildNodes()) {
                if (node.firstChild.data.checked !== node.parentNode.firstChild.data.checked) {
                    path[node.firstChild.data.checked ? 'include_path' : 'exclude_path'].push(node.data.path);
                }
            } else {
                if (node.data.checked !== node.parentNode.firstChild.data.checked) {
                    path[node.data.checked ? 'include_path' : 'exclude_path'].push(node.data.path);
                }
            }
        });

        return path;
    },

    setIncludePath: function(list) {
        this.includePath = list;
    },

    setExcluePath: function(list) {
        this.excludePath = list;
    },

    walkTree: function() {
        var me = this;
        var rootNode = me.tree.getRootNode(),
            pathList = me.includePath.concat(me.excludePath),
            getParentChecked = function(node) {
                return node.parentNode.firstChild.data.checked;
            },
            checkNodes = function(node) {
                node.eachChild(function(child) {
                    if (child.data.leaf) return true;
                    var isMatch = pathList.some(function(path) {
                        return path === child.data.path;
                    }),
                        isNeedExpand = pathList.some(function(path) {
                            return path.indexOf(child.data.path + '/') === 0;
                        });

                    if (isNeedExpand) {
                        me.EXPAND_LIST.push(child);
                        child.expand();
                        me.renderTree(child, function() {
                            child.firstChild.set('checked', isMatch ? !getParentChecked(child) : getParentChecked(child));
                            me.tree.fireEvent('checkchange', child.firstChild, child.firstChild.data.checked);
                            checkNodes(child);
                        });
                    } else {
                        child.set('checked', isMatch ? !getParentChecked(child) : getParentChecked(child));
                        me.tree.fireEvent('checkchange', child, child.data.checked);
                    }
                });
            };

        rootNode.firstChild.set('checked', me.includePath.include('/'));
        me.tree.fireEvent('checkchange', rootNode.firstChild, rootNode.firstChild.data.checked);
        checkNodes(rootNode);
    },
});

/**
 * @class AS.ARC.apps.datasynccenter.FileExtensionFilter
 * @extends Ext.container.Container
 */
Ext.define('AS.ARC.apps.datasynccenter.FileExtensionFilter', {
	extend: 'Ext.container.Container',
    alias: ['widget.fileextensionfilter'],
    baseCls: 'as-file-extension-filter',

    EXTENSION_LIST: [],
    ADD_RULE_TIPS: [],
    tri: {
        NONE: 0,
        HALF: 1,
        ALL: 2
    },
    
    initComponent: function() {
        this.items = [this.getItems()];
        this.init();
        this.callParent(arguments);
    },

    init: function() {
        this.EXTENSION_LIST = [
            [_S('COMMON', 'DOCUMENTS'), ['*.azw', '*.csv', '*.doc', '*.docx', '*.dps', '*.epub', '*.et', '*.mobi', '*.mpp', '*.odp', '*.ods', '*.odt', '*.pdf', '*.pps', '*.ppt', '*.pptx', '*.txt', '*.vsd', '*.wps', '*.xlr', '*.xls', '*.xlsx']],
            [_S('COMMON', 'VIDEO'), ['*.avi', '*.flv', '*.m2t', '*.m2ts', '*.m4v', '*.mkv', '*.mov', '*.mp4', '*.mpeg', '*.mpg', '*.rmvb', '*.swf', '*.vob', '*.webm', '*.wmv']],
            [_S('COMMON', 'AUDIO'), ['*.ac3', '*.acc', '*.amr', '*.ape', '*.awb', '*.dss', '*.dvf', '*.flac', '*.m4a', '*.m4p', '*.mp3', '*.ogg', '*.wav', '*.wma']],
            [_S('COMMON', 'IMAGES'), ['*.ai', '*.bmp', '*.eps', '*.gif', '*.jp2', '*.jpg', '*.obj', '*.png', '*.psd', '*.svg', '*.tga', '*.tif']],
            [_S('COMMON', 'APPLICATIONS'), ['*.app', '*.clg', '*.com', '*.exe', '*.gadgt', '*.jar', '*.pif', '*.vb', '*.wsf']],
            [_S('DATASYNC_CENTER', 'TEMPORARY_FILES'), ['*.aso', '*.box', '*.cache', '*.dat', '*.download', '*.exd', '*.temp']],
            [_S('DATASYNC_CENTER', 'GOOGLE_DOC'), ['*.gdoc', '*.gdraw', '*.gform', '*.gscript', '*.gsheet', '*.gslide', '*.gtable']],
            [_S('DATASYNC_CENTER', 'BOX_DOC'), ['*.boxnote', '*.webdoc']],
            [_S('DEFENDER', 'CUSTOMIZED'), []]
        ];
        this.ADD_RULE_TIPS = [
            _S('DATASYNC_CENTER', 'FILTER_RULES_1'),
            _S('DATASYNC_CENTER', 'FILTER_RULES_2'),
            _S('DATASYNC_CENTER', 'FILTER_RULES_3'),
            _S('DATASYNC_CENTER', 'FILTER_RULES_4'),
            _S('DATASYNC_CENTER', 'FILTER_RULES_5')
        ];
    },

    getItems: function() {
        var me = this;
        return [
            this.getTree(), {
            xtype: 'asFlexbox',
            margin: '10 0 0 0',
            height: 28,
            layout: {
                type: 'hbox',
                align: 'stretch'
            },
            items: [{
                xtype: 'textfield',
                itemId: 'customized-file-extension',
                flex: 1,
                msgTarget: AS.ARC.config.msgTarget,
                margin: '0 0 0 0',
                emptyText: _S('DATASYNC_CENTER', 'BY_NAME_FILE_EXTENSION_PLACEHOLDER'),
                validator: function(value) {
                    var rootNode = me.tree.getRootNode(),
                        isExisted = false;

                    rootNode.cascadeBy(function(child) {
                        if (child.data.leaf && child.data.id === value) {
                            isExisted = child.data.id === value;
                            return false;
                        }
                    });

                    if (value === '') {
                        return true;
                    } else if (value.length > 32) {
                        return Ext.String.format(_S('MESSAGE', 'HF00016'), '32');
                    } else if (isExisted) {
                        return _S('ERROR', 'NEW_FILE_NAME_EXISTED');
                    } else 
                    return true;
                },
                listeners: {
                    render: function(tf) {
                        var str = '';
                        me.ADD_RULE_TIPS.map(function(tip, index) {
                            str += (index + 1) + '. ' + tip + '<br/>';
                        });
                        tf.el.dom.setAttribute('data-qtip', str);
                    },
                    validitychange: function(tf, isValid) {
                        tf.next().setDisabled(!isValid);
                    }
                }
            }, {
                xtype: 'button',
                itemId: 'add',
                cls: 'image-btn',
                iconCls: 'add-btn',
                margin: '0 0 0 5',
                handler: function(btn) {
                    var value = btn.prev().getValue();
                    btn.prev().reset();
                    me.addFileExtension(value);
                },
            }]
        }];
    },

    addFileExtension: function(value, manualAdd) {
        var me = this,
            manualAdd = (manualAdd === false) ? false : true,
            customizedNode = me.tree.getRootNode().lastChild;

        if (value === '') return false;

        if (manualAdd) me.tree.el.mask(_S('COMMON', 'LOADING'));
        customizedNode.expand();
        if (manualAdd) {
            Ext.defer(function() {
                var node = customizedNode.appendChild({
                    text: '<span class="node-wrap"><span class="node-wrap__text">' + value + '</span><span class="node-wrap__remove-icon status-icon-16 clear"></span></span>',
                    leaf: true,
                    qtip: value,
                    checked: true,
                    icon: Ext.BLANK_IMAGE_URL,
                    id: value,
        
                    tri: me.tri.ALL
                });
                me.tree.el.unmask();
                me.tree.fireEvent('checkchange', node, node.data.checked);
                me.tree.body.dom.scrollTop = me.tree.view.el.getHeight() - me.tree.body.getHeight();
            }, 1000);
        } else {
            var node = customizedNode.appendChild({
                text: '<span class="node-wrap"><span class="node-wrap__text">' + value + '</span><span class="node-wrap__remove-icon status-icon-16 clear"></span></span>',
                leaf: true,
                qtip: value,
                checked: false,
                icon: Ext.BLANK_IMAGE_URL,
                id: value,
    
                tri: me.tri.ALL
            });
            me.tree.fireEvent('checkchange', node, node.data.checked);
            me.tree.body.dom.scrollTop = me.tree.view.el.getHeight() - me.tree.body.getHeight();
        }
        
    },

    checkParent: function(node) {
        var me = this;

        node.bubble(function(parent) {
            var count = 0,
                half = false;
            if (parent.data.text == node.data.text)
                return true;

            parent.eachChild(function(child) {
                if (child.data.tri !== me.tri.NONE) {
                    if (child.data.tri === me.tri.HALF) {
                        half = true;
                    }
                    count++;
                }
            });

            if (count === parent.childNodes.length) {
                if (half) {
                    parent.set('checked', false);
                    parent.set('tri', me.tri.HALF);
                    parent.set('cls', 'x-tree-checked-down');
                } else {
                    parent.set('checked', true);
                    parent.set('tri', me.tri.ALL);
                    parent.set('cls', '');
                }
            } else if (count > 0) {
                parent.set('checked', false);
                parent.set('tri', me.tri.HALF);
                parent.set('cls', 'x-tree-checked-down');
            } else {
                parent.set('checked', false);
                parent.set('tri', me.tri.NONE);
                parent.set('cls', '');
            }
        });
    },

    checkChild: function(node) {
        var me = this;
        node.cascadeBy(function(child) {
            if (child.data.text == node.data.text)
                return true;

            child.set('checked', node.data.checked);
            child.set('tri', node.data.checked ? me.tri.ALL : me.tri.NONE);
            child.set('cls', '');
        });
    },

    renderTree: function() {
        var me = this;
        var rootNode = me.tree.setRootNode({
            text: 'root',
            expaned: true,
            checked: false,

            tri: me.tri.NONE
        });
        
        Ext.each(this.EXTENSION_LIST, function(list, index) {
            var node = rootNode.appendChild({
                text: list.first(),
                expaned: true,
                checked: false,
                icon: Ext.BLANK_IMAGE_URL,
                id: index + 1,

                tri: me.tri.NONE
            });

            Ext.each(list.last(), function(extension) {
                node.appendChild({
                    text: extension,
                    leaf: true,
                    qtip: extension,
                    checked: false,
                    icon: Ext.BLANK_IMAGE_URL,
                    id: extension,

                    tri: me.tri.NONE
                });
            });
        });

        Ps.update(me.tree.body.dom);
    },

    getTree: function() {
        var me = this;

        me.tree = Ext.create('Ext.tree.TreePanel', {
            viewConfig: {
                rootVisible: false
            },
            useArrows: true,
            height: me.treeHeight || 240,
            flex: 1,
            resizable: false,
            scope: this,
            margin: '20 0 0 0',
            listeners: {
                afterrender: function() {
                    me.renderTree();
                },
                checkchange: function(node, checked) {
                    node.set('tri', checked ? me.tri.ALL : me.tri.NONE);
                    node.set('cls', '');
                    me.checkParent(node);
                    me.checkChild(node);
                },
                beforeselect: function() {
                    return false;
                },
                itemclick: function(view, record, item, index, e) {
                    if (e.target.tagName === 'INPUT') return false;
                    if (e.target.classList.contains('node-wrap__remove-icon')) {
                        var parentNode = record.parentNode;
                        record.remove();
                        if (parentNode.hasChildNodes()) {
                            me.checkParent(parentNode.firstChild);
                        } else {
                            parentNode.set('checked', false);
                            parentNode.set('tri', me.tri.NONE);
                            parentNode.set('cls', '');
                        }
                        
                        Ps.update(me.tree.body.dom);
                    } else {
                        record.set('checked', !record.data.checked);
                        record.commit();
                        me.tree.fireEvent('checkchange', record, record.data.checked);
                    }
                }
            }
        });

        return me.tree;
    },

    getList: function() {
        var me = this;
            checked = me.tree.getChecked(),
            customizedNode = me.tree.getRootNode().lastChild,
            list = {
                checkedList: [],
                customizedList: []
            };

        checked.forEach(function(node) {
            if (node.data.leaf) {
                list.checkedList.push(node.data.id);
            }
        });

        customizedNode.childNodes.forEach(function(node) {
            list.customizedList.push(node.data.id);
        });

        return list;
    },

    setCheckedList: function(list) {
        var me = this;
        list.forEach(function(id) {
            var node = me.tree.store.getNodeById(id);
            if (!Ext.isEmpty(node)) {
                node.set('checked', true);
                me.tree.fireEvent('checkchange', node, node.data.checked);
            }
        });
    },

    setCustomizedList: function(list) {
        var me = this;
        list.forEach(function(id) {
            me.addFileExtension(id, false);
        });
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.syncListToolbar.wizard
 * @extends AS.ARC.wizard.modalWindow
 */
Ext.define('AS.ARC.apps.datasynccenter.syncListToolbar.wizard', {
    extend: 'AS.ARC.wizard.modalWindow',

    loginInfo: '',
    imported: false,
    pollingID: 0,
    ajaxRequest: null,

    LABEL_WIDTH: 200,
    WIDTH: 500,

    CREATE_CONNECTION: 'create_connection',
    CREATE_TASK: 'create_task',

    SERVICE_LIST: {
        EZ_SYNC: 'ezsync',
        BAIDU: 'baidu',
        DROPBOX: 'dropbox',
        GOOGLE_DRIVE: 'googledrive',
        ONE_DRIVE: 'onedrive',
        YANDEX: 'yandex'
    },

    constructor: function(config) {
        var wizard = this;
        config.steps = this.getSteps(config.type);
        wizard.callParent(arguments);
    },

    getSteps: function(type) {
        var wizard = this,
            steps = [];
        if (type === this.CREATE_CONNECTION) {
            steps = [
                wizard.getPanelSelectService(),
                wizard.getPanelTaskSetting(),

                wizard.getPanelUploadSetting(),
                wizard.getPanelTwoWaySetting(),
                wizard.getPanelDownloadSetting(),

                wizard.getPanelSchedule(),
                wizard.getPanelFilter(),
                wizard.getPanelConfirm()
            ];
        } else if (type === this.CREATE_TASK) {
            steps = [
                wizard.getPanelTaskSetting(),

                wizard.getPanelUploadSetting(),
                wizard.getPanelTwoWaySetting(),
                wizard.getPanelDownloadSetting(),

                wizard.getPanelSchedule(),
                wizard.getPanelFilter(),
                wizard.getPanelConfirm()
            ];
        }
        return steps;
    },

    goNextStep: function() {
        var wizard = this;
        Ext.defer(function() {
            wizard.onNext();
            wizard.removePrevPage();
            wizard.el.unmask();
        }, 1000);
    },

    onSelectionChange: function() {
        var wizard = this,
            service = wizard.down('serviceList').getService().data.service;
        if (Ext.isEmpty(wizard.loginInfo)) return false;

        wizard.down('#authorize').setDisabled(Ext.isEmpty(wizard.loginInfo[service]) && service !== wizard.SERVICE_LIST.EZ_SYNC);
        wizard.down('#import').setVisible(!Ext.isEmpty(wizard.loginInfo[service]) && wizard.loginInfo[service].import);
    },

    handleAuthorize: function() {
        var wizard = this,
            service = wizard.down('serviceList').getService().data.service,
            url = '';

        if (Ext.isEmpty(service)) return false;
        wizard.startCheckLoginInfo();
        if (service === wizard.SERVICE_LIST.EZ_SYNC) {
            var loginWin = Ext.create('AS.ARC.apps.datasynccenter.ezsyncLoginWin', {
                parentWin: wizard
            });
            loginWin.show();
        } else {
            url = wizard.loginInfo[service].login_url + encodeURIComponent('&adm_url=' + encodeURIComponent(window.location.href.split('portal').shift()));
            window.open(url, "newwindow", "height=700, width=600, toolbar=no, menubar=no, scrollbars=no, resizable=no, location=no, status=no");
        }
    },

    handleImport: function() {
        var wizard = this,
            service = wizard.down('serviceList').getService().data.service;
        wizard.el.mask(_S('DATASYNC_CENTER', 'BEING_IMPORT_PLEASE_WAIT'));
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(wizard.fn.apiUrl, { act: 'import' }),
            params: {
                service_type: service
            },
            method: 'post',
            success: function() {
                wizard.imported = true;
                wizard.close();
                wizard.fn.syncList.loadSyncList();
            },
            failure: function(json) {
                AS.ARC.util.showMsgWindow({
                    5001: _S('DATASYNC_CENTER', 'IMPORT_ACCOUNT_AUTH_IS_FAILED'),
                    5008: _S('DATASYNC_CENTER', 'ACCOUNT_EXISTS_WARNING')
                }, json, wizard);
            }
        });
    },

    startCheckLoginInfo: function() {
        var wizard = this;
        wizard.stopCheckLoginInfo();
        wizard.ajaxRequest = AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(wizard.fn.apiUrl, { act: 'get_login_info' }),
            method: 'post',
            success: function(json) {
                if (Ext.isEmpty(wizard) || wizard.isDestroyed) return false;

                if (Ext.isEmpty(wizard.loginInfo)) {
                    wizard.loginInfo = json.login_info
                    wizard.onSelectionChange();
                } else {
                    wizard.loginInfo = json.login_info;
                }

                if (json.login) {
                    wizard.connectionId = json.login_status.connection_setting.connection_id;
                    wizard.serviceType = json.login_status.service_type;
                    wizard.connectionName = json.login_status.connection_setting.connection_name;

                    wizard.el.mask(_S('COMMON', 'LOADING'));
                    wizard.goNextStep();
                } else {
                    wizard.pollingID = Ext.defer(function() {
                        wizard.startCheckLoginInfo();
                    }, 1000);
                }
            },
            failure: function(json) {
                wizard.el.mask(_S('COMMON', 'LOADING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(wizard.fn.apiUrl, { act: 'cancel_new_connection' }),
                    method: 'post',
                    success: function() {
                        AS.ARC.util.showMsgWindow({
                            5008: _S('DATASYNC_CENTER', 'ACCOUNT_EXISTS_WARNING')
                        }, json, wizard);
                    },
                    failure: function() {}
                });
                
            }
        });
    },

    stopCheckLoginInfo: function() {
        var wizard = this;
        if (!Ext.isEmpty(wizard.ajaxRequest)) {
            Ext.Ajax.abort(wizard.ajaxRequest);
        }
        clearTimeout(wizard.pollingID);
    },

    getPanelSelectService: function() {
        var wizard = this,
            IMPORT_APP_NAME_LIST = [
                ['dropbox', _UN_TS('DataSync for Dropbox')],
                ['googledrive', _UN_TS('DataSync for GoogleDrive')],
                ['onedrive', _UN_TS('DataSync for OneDriveBusiness')],
                ['yandex', _UN_TS('DataSync for Yandex')]
            ];

        return Ext.create('AS.ARC.wizard.formPanel', {
            itemId: 'panel-select-service',
            cls: 'app-datasync-center-wizard',
            id: '',
            defaults: {
                labelWidth: AS.ARC.config.layout.labelWidthMedium,
                msgTarget: AS.ARC.config.msgTarget,
                blankText: _S('MESSAGE', 'HF00011'),
                maxLengthText: _S('MESSAGE', 'HF00016')
            },
            items: [{
                xtype: 'displayfield',
                value: _S('DATASYNC_CENTER', 'CHOOSE_CLOUD_SERVICE')
            }, {
                xtype: 'displayfield',
                value: '&nbsp;'
            }, {
                xtype: 'serviceList',
                itemId: 'service-list',
                wizard: wizard
            }, {
                xtype: 'asFlexbox',
                itemId: 'buttons',
                style: 'justify-content: center;margin-top: 30px',
                items: [{
                    xtype: 'button',
                    itemId: 'authorize',
                    cls: 'authorize',
                    text: _S('DATASYNC_CENTER', 'AUTHORIZE'),
                    width: 280,
                    height: 30,
                    handler: wizard.handleAuthorize,
                    scope: this
                }, {
                    xtype: 'button',
                    itemId: 'import',
                    iconCls: 'status-icon-16 warning',
                    iconAlign: 'right',
                    cls: 'authorize x-btn-text-icon',
                    text: _S('DATASYNC_CENTER', 'IMPORT_DATA'),
                    width: 280,
                    height: 30,
                    hidden: true,
                    handler: wizard.handleImport,
                    scope: this,
                    listeners: {
                        'mouseover': function(button) {
                            var service = wizard.down('serviceList').getService().data.service,
                                appName = IMPORT_APP_NAME_LIST.find(function(item) {
                                    return item.first() === service;
                                }).last(),
                                tip = button.up().tip;
                            
                            tip.down('#tip-info').setValue(
                                Ext.String.format(_S('DATASYNC_CENTER', 'IMPORT_DATA_CONFIRM_1'), appName)
                            );
                            tip.show();
                        },
                        'mouseout': function(button) {
                            button.up().tip.hide();
                        }
                    }
                }],
                listeners: {
                    afterrender: function(flexbox) {
                        flexbox.tip = Ext.create('Ext.container.Container', {
                            cls: 'import-tip',
                            floating: true,
                            width: 600,
                            height: 232,
                            padding: '35 50 0 50',
                            renderTo: flexbox.el,
                            items: [{
                                xtype: 'displayfield',
                                cls: 'tip-title displayfield',
                                value: _S('DATASYNC_CENTER', 'IMPORT_DATA')
                            }, {
                                xtype: 'displayfield',
                                cls: 'displayfield',
                                itemId: 'tip-info',
                                margin: '18 0 0 0'
                            }],
                            listeners: {
                                afterrender: function(me) {
                                    me.el.alignTo(flexbox.el, 'b-t', [10, -38]);
                                    me.hide();
                                }
                            }
                        });
                    }
                }
            }],
            nextId: 'panel-task-setting',
            loadData: function() {
                if (wizard.loaded) return false;
                wizard.loaded = true;
                wizard.footerBar.next.disable();
                wizard.el.mask(_S('COMMON', 'LOADING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(wizard.fn.apiUrl, { act: 'cancel_new_connection' }),
                    method: 'post',
                    success: function() {
                        wizard.el.unmask();
                        wizard.startCheckLoginInfo();
                    },
                    failure: function() {
                        wizard.el.unmask();
                        wizard.startCheckLoginInfo();
                    }
                });
            }
        });
    },

    getPanelTaskSetting: function() {
        var wizard = this,
        SERVICE_LIST = [
            ['ezsync', _S('DATASYNC_CENTER', 'ASUSTOR_NAS')],
            ['baidu', _S('DATASYNC_CENTER', 'BAIDU')],
            ['dropbox', _S('DATASYNC_CENTER', 'DROPBOX')],
            ['googledrive', _S('DATASYNC_CENTER', 'GOOGLE_DRIVE')],
            ['onedrive', _S('DATASYNC_CENTER', 'ONE_DRIVE')],
            ['yandex', _S('DATASYNC_CENTER', 'YANDEX')]
        ];

        var handleSyncTypeChange = function(combo) {
            var serviceType = wizard.serviceType;

            wizard.down('#remain-local-files').setVisible(
                combo.getValue() === 'download'
            );
            wizard.down('#remain-remote-files').setVisible(
                combo.getValue() === 'upload'
            );
            wizard.down('#convert-google-office-files').setVisible(
                serviceType === 'googledrive' && combo.getValue() !== 'upload'
            );
            wizard.down('#force-sync-malicious-files').setVisible(
                serviceType === 'googledrive' && combo.getValue() !== 'upload'
            );
        };

        var getSyncTypeData = function(serviceName) {
            if (wizard.serviceType === 'ezsync') {
                return [{
                    'display': _S('DATASYNC_CENTER', 'UPLOAD_LOCAL_MODIFICATIONS'),
                    'value': 'upload'
                }, {
                    'display': _S('DATASYNC_CENTER', 'TWO_WAY'),
                    'value': 'two_way'
                }, {
                    'display': _S('DATASYNC_CENTER', 'DOWNLOAD_REMOTE_MODIFICATIONS'),
                    'value': 'download'
                }];
            } else {
                return [{
                    'display': Ext.String.format(_S('DATASYNC_CENTER', 'NAS_TO_REMOTE'), serviceName),
                    'value': 'upload'
                }, {
                    'display': _S('DATASYNC_CENTER', 'TWO_WAY'),
                    'value': 'two_way'
                }, {
                    'display': Ext.String.format(_S('DATASYNC_CENTER', 'REMOTE_TO_NAS'), serviceName),
                    'value': 'download'
                }];
            }
        };

        return Ext.create('AS.ARC.wizard.formPanel', {
            itemId: 'panel-task-setting',
            cls: 'app-datasync-center-wizard',
            adjustFooterbar: true,
            defaults: {
                labelWidth: wizard.LABEL_WIDTH,
                msgTarget: AS.ARC.config.msgTarget,
                blankText: _S('MESSAGE', 'HF00011'),
                maxLengthText: _S('MESSAGE', 'HF00016')
            },
            items: [{
                xtype: 'textfield',
                itemId: 'task-name',
                fieldLabel: _S('DATASYNC_CENTER', 'NAME'),
                maxLength: 255,
                allowBlank: false,
                width: wizard.WIDTH,
                validator: function(value) {
                    var me = this,
                        store;

                    if (me.isDirty()) {
                        store = wizard.fn.syncList.store;
                        if (store.findExact('connection_name', value) >= 0) {
                            return _S('DATASYNC_CENTER', 'NAME_EXISTS_WARNING');
                        }
                    }
                    return true;
                }
            }, {
                xtype: 'combo',
                itemId: 'sync-type',
                fieldLabel: _S('DATASYNC_CENTER', 'SYNCHRONIZATION_TYPE'),
                width: wizard.WIDTH,
                store: new Ext.data.Store({
                    fields: ['display', 'value'],
                    data: [{
                        'display': _S('DATASYNC_CENTER', 'UPLOAD_LOCAL_MODIFICATIONS'),
                        'value': 'upload'
                    }, {
                        'display': _S('DATASYNC_CENTER', 'TWO_WAY'),
                        'value': 'two_way'
                    }, {
                        'display': _S('DATASYNC_CENTER', 'DOWNLOAD_REMOTE_MODIFICATIONS'),
                        'value': 'download'
                    }]
                }),
                displayField: 'display',
                valueField: 'value',
                queryMode: 'local',
                editable: false,
                lastQuery: '',
                value: 'two_way',
                listeners: {
                    change: handleSyncTypeChange
                }
            }, {
                xtype: 'checkbox',
                itemId: 'remain-local-files',
                boxLabel: _S('DATASYNC_CENTER', 'DELETE_REMOTE_FILES_NOT_DELETE_LOCAL_FILES'),
                style: 'margin-top: 32px;',
                hidden: true
            }, {
                xtype: 'checkbox',
                itemId: 'remain-remote-files',
                boxLabel: _S('DATASYNC_CENTER', 'DELETE_LOCAL_FILES_NOT_DELETE_REMOTE_FILES'),
                style: 'margin-top: 32px;',
                hidden: true
            }, {
                xtype: 'checkbox',
                itemId: 'convert-google-office-files',
                boxLabel: _S('DATASYNC_CENTER', 'DOWNLOAD_AND_CONVERT_GOOGLE_OFFICE_FILES'),
                style: 'margin-top: 32px;',
                hidden: true
            }, {
                xtype: 'checkbox',
                itemId: 'force-sync-malicious-files',
                style: 'margin-top: 32px;',
                cls: 'displayfield',
                boxLabel: AS.ARC.datas.defaultValue
            }],
            getNextId: function() {
                var syncType = wizard.down('#sync-type');
                if (syncType.getValue() === 'upload') {
                    return 'panel-upload-setting';
                } else if (syncType.getValue() === 'two_way') {
                    return 'panel-two-way-setting';
                } else if (syncType.getValue() === 'download') {
                    return 'panel-download-setting';
                }
            },
            loadData: function() {
                var selectedServiceName = SERVICE_LIST.find(function(service) {
                        return service.first() === wizard.serviceType
                    }).last(),
                    taskSettingPanel = wizard.down('#panel-task-setting'),
                    comboboxSyncType = wizard.down('#sync-type'),
                    data = getSyncTypeData(selectedServiceName);

                comboboxSyncType.store.loadData(data);
                comboboxSyncType.fireEvent('change', comboboxSyncType);

                taskSettingPanel.down('#task-name').setVisible(wizard.type === wizard.CREATE_CONNECTION)
                .setValue(wizard.connectionName)
                .setDisabled(wizard.type === wizard.CREATE_TASK);
                taskSettingPanel.down('#force-sync-malicious-files').boxLabel = Ext.String.format(_S('DATASYNC_CENTER', 'FORCE_SUSPECTED_MALICIOUS_FILES'), selectedServiceName);
                taskSettingPanel.down('#force-sync-malicious-files').boxLabelEl.update(
                    Ext.String.format(_S('DATASYNC_CENTER', 'FORCE_SUSPECTED_MALICIOUS_FILES'), selectedServiceName)
                );
            },
            handleFooterbar: function() {
                wizard.footerBar.back.hide();
            }
        });
    },

    getPanelUploadSetting: function() {
        var wizard = this;
        return Ext.create('AS.ARC.wizard.formPanel', {
            itemId: 'panel-upload-setting',
            cls: 'app-datasync-center-wizard',
            defaults: {
                labelWidth: AS.ARC.config.layout.labelWidthMedium,
                msgTarget: AS.ARC.config.msgTarget,
                blankText: _S('MESSAGE', 'HF00011'),
                maxLengthText: _S('MESSAGE', 'HF00016')
            },
            items: [
                wizard.getLocalPathSelector('upload'), {
                    xtype: 'checkbox',
                    itemId: 'select-files-folders',
                    boxLabel: _S('DATASYNC_CENTER', 'SELECT_FILES_FOLDERS'),
                    disabled: true,
                    listeners: {
                        change: function(me) {
                            var panel = wizard.down('#panel-upload-setting');
                            if (Ext.isEmpty(panel.down('syncTree')) && me.getValue()) {
                                panel.insert(2, {
                                    xtype: 'syncTree',
                                    itemId: 'sync-tree',
                                    parentWin: wizard,
                                    type: 'upload',
                                    localPath: panel.down('#local-path-selector').down('#path').getValue(),
                                    localFolder: panel.down('#local-path-selector').down('#path').getValue().split('/').pop(),
                                    connectionId: wizard.connectionId
                                });
                            } else {
                                panel.down('syncTree').setVisible(me.getValue());
                            }
                        }
                    }
                },
                wizard.getRemotePathSelector('upload')
            ],
            onNext: function() {
                var tree = wizard.down('#panel-upload-setting').down('syncTree');
                if (Ext.isEmpty(tree)) {
                    return true;
                } else if (Ext.isEmpty(tree.tree.getChecked())) {
                    wizard.getNoSyncSelectionWarning(wizard).show()
                    return false;
                } else {
                    return true;
                }
            },
            getNextId: function() {
                if (wizard.type === wizard.CREATE_CONNECTION) {
                    return 'panel-schedule';
                } else if (wizard.type === wizard.CREATE_TASK) {
                    return 'panel-filter';
                }
            },
            loadData: function() {}
        });
    },

    getPanelTwoWaySetting: function() {
        var wizard = this;
        return Ext.create('AS.ARC.wizard.formPanel', {
            itemId: 'panel-two-way-setting',
            cls: 'app-datasync-center-wizard',
            defaults: {
                labelWidth: wizard.LABEL_WIDTH,
                msgTarget: AS.ARC.config.msgTarget,
                blankText: _S('MESSAGE', 'HF00011'),
                maxLengthText: _S('MESSAGE', 'HF00016')
            },
            items: [
                wizard.getLocalPathSelector('two_way'),
                wizard.getRemotePathSelector('two_way'), {
                    xtype: 'checkbox',
                    itemId: 'select-files-folders',
                    boxLabel: _S('DATASYNC_CENTER', 'SELECT_FILES_FOLDERS'),
                    disabled: true,
                    listeners: {
                        change: function(me) {
                            var panel = wizard.down('#panel-two-way-setting');
                            if (Ext.isEmpty(panel.down('syncTree')) && me.getValue()) {
                                panel.insert(3, {
                                    xtype: 'syncTree',
                                    itemId: 'sync-tree',
                                    parentWin: wizard,
                                    type: 'two_way',
                                    localPath: panel.down('#local-path-selector').down('#path').getValue(),
                                    localFolder: panel.down('#local-path-selector').down('#path').getValue().split('/').pop(),
                                    remotePath: panel.down('#remote-path-selector').down('#path').getRemoteId(),
                                    remoteFolder: panel.down('#remote-path-selector').down('#path').getValue().split('/').pop(),
                                    connectionId: wizard.connectionId
                                });
                            } else {
                                panel.down('syncTree').setVisible(me.getValue());
                            }
                        }
                    }
                }
            ],
            onNext: function() {
                var tree = wizard.down('#panel-two-way-setting').down('syncTree');
                if (Ext.isEmpty(tree)) {
                    return true;
                } else if (Ext.isEmpty(tree.tree.getChecked())) {
                    wizard.getNoSyncSelectionWarning(wizard).show();
                    return false;
                } else {
                    return true;
                }
            },
            getNextId: function() {
                if (wizard.type === wizard.CREATE_CONNECTION) {
                    return 'panel-schedule';
                } else if (wizard.type === wizard.CREATE_TASK) {
                    return 'panel-filter';
                }
            },
            loadData: function() {}
        });
    },

    getPanelDownloadSetting: function() {
        var wizard = this;
        return Ext.create('AS.ARC.wizard.formPanel', {
            itemId: 'panel-download-setting',
            cls: 'app-datasync-center-wizard',
            defaults: {
                labelWidth: AS.ARC.config.layout.labelWidthMedium,
                msgTarget: AS.ARC.config.msgTarget,
                blankText: _S('MESSAGE', 'HF00011'),
                maxLengthText: _S('MESSAGE', 'HF00016')
            },
            items: [
                wizard.getRemotePathSelector('download'), {
                    xtype: 'checkbox',
                    itemId: 'select-files-folders',
                    boxLabel: _S('DATASYNC_CENTER', 'SELECT_FILES_FOLDERS'),
                    disabled: true,
                    listeners: {
                        change: function(me) {
                            var panel = wizard.down('#panel-download-setting');
                            if (Ext.isEmpty(panel.down('syncTree')) && me.getValue()) {
                                panel.insert(2, {
                                    xtype: 'syncTree',
                                    itemId: 'sync-tree',
                                    parentWin: wizard,
                                    type: 'download',
                                    remotePath: panel.down('#remote-path-selector').down('#path').getRemoteId(),
                                    remoteFolder: panel.down('#remote-path-selector').down('#path').getValue().split('/').pop(),
                                    connectionId: wizard.connectionId
                                });
                            } else {
                                panel.down('syncTree').setVisible(me.getValue());
                            }
                        }
                    }
                },
                wizard.getLocalPathSelector('download')
            ],
            onNext: function() {
                var tree = wizard.down('#panel-download-setting').down('syncTree');
                if (Ext.isEmpty(tree)) {
                    return true;
                } else if (Ext.isEmpty(tree.tree.getChecked())) {
                    wizard.getNoSyncSelectionWarning(wizard).show()
                    return false;
                } else {
                    return true;
                }
            },
            getNextId: function() {
                if (wizard.type === wizard.CREATE_CONNECTION) {
                    return 'panel-schedule';
                } else if (wizard.type === wizard.CREATE_TASK) {
                    return 'panel-filter';
                }
            },
            loadData: function() {}
        });
    },

    getPanelSchedule: function() {
        var wizard = this;
        return Ext.create('AS.ARC.wizard.formPanel', {
            itemId: 'panel-schedule',
            cls: 'app-datasync-center-wizard',
            defaults: {
                labelWidth: AS.ARC.config.layout.labelWidthMedium,
                msgTarget: AS.ARC.config.msgTarget,
                blankText: _S('MESSAGE', 'HF00011'),
                maxLengthText: _S('MESSAGE', 'HF00016')
            },
            items: [{
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'PLEASE_SET_TIME')
            }, {
                xtype: 'scheduletable',
                style: AS.layout.getIndentCss(1),
                suspendString: _S('DATASYNC_CENTER', 'PAUSE'),
                syncString: _S('FILE_EXPLORER', 'DATA_SYNCHRONIZE')
            }, {
                xtype: 'displayfield',
                cls: 'warning-string',
                value: _S('DATASYNC_CENTER', 'SYNCHRONIZATION_END_IF_PAUSED'),
                style: AS.layout.getIndentCss(1) + ';margin-top: 20px'
            }],
            nextId: 'panel-filter',
            loadData: function() {
                wizard.down('scheduletable').setSchedule(wizard.down('scheduletable').getSchedule());
            }
        });
    },

    getPanelFilter: function() {
        var wizard = this;
        return Ext.create('AS.ARC.wizard.formPanel', {
            itemId: 'panel-filter',
            cls: 'sync-wizard-filter-set',
            items: [{
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'FILTER_RULES')
            }, {
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'FILE_SIZE')
            }, {
                xtype: 'asFlexbox',
                style: AS.layout.getIndentCss(1),
                defaults: {
                    labelWidth: AS.ARC.config.layout.labelWidthMedium,
                    msgTarget: AS.ARC.config.msgTarget,
                    blankText: _S('MESSAGE', 'HF00011'),
                    maxLengthText: _S('MESSAGE', 'HF00016'),
                },
                items: [{
                    xtype: 'numberfield',
                    itemId: 'size-limit',
                    labelStyle: 'min-width: 240',
                    minValue: 0,
                    maxValue: 10239,
                    maxLength: 6,
                    enforceMaxLength: true,
                    allowDecimals: false,
                    allowBlank: false,
                    fieldLabel: _S('DATASYNC_CENTER', 'DO_NOT_SYNC_IF'),
                    value: 0,
                    width: 400
                }, {
                    xtype: 'displayfield',
                    style: 'margin-left: 5px;',
                    value: _UN_TS('MB')
                }]
            }, {
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'BY_NAME_FILE_EXTENSION')
            }, {
                xtype: 'displayfield',
                style: AS.layout.getIndentCss(1),
                cls: 'displayfield',
                labelCls: 'displayfield',
                fieldLabel: _S('DATASYNC_CENTER', 'FILTER_BY_NAME_OR_TYPE')
            }, {
                xtype: 'fileextensionfilter'
            }],
            nextId: 'panel-confirm',
            loadData: function() {
                Ps.update(wizard.down('fileextensionfilter').tree.el.dom);
            }
        });
    },

    getPanelConfirm: function() {
        var wizard = this;
        var gridConfirm = Ext.create('Ext.grid.Panel', {
            itemId: 'confirm-grid',
            cls: 'app-datasync-center-wizard',
            viewConfig: {
                loadMask: false,
                getRowClass: function() {
                    return 'datasync-center-multiline-row';
                }
            },
            store: Ext.create('Ext.data.Store', {
                fields: [
                    'name',
                    'value'
                ],
                data: []
            }),
            bodyStyle: 'max-height: 350px',
            margin: '30 0 0 0',
            columns: [{
                text: _S('COMMON', 'NAME'),
                flex: 1,
                sortable: false,
                dataIndex: 'name',
                renderer: function(value, metadata, record) {
                    metadata.tdAttr = 'data-qtip="' + value + '"';
                    return value;
                }
            }, {
                text: _S('DATASYNC_CENTER', 'VALUE'),
                flex: 1,
                sortable: false,
                dataIndex: 'value'
            }]
        });

        return Ext.create('AS.ARC.wizard.formPanel', {
            itemId: 'panel-confirm',
            defaults: {
                labelWidth: AS.ARC.config.layout.labelWidthMedium,
                msgTarget: AS.ARC.config.msgTarget,
                blankText: _S('MESSAGE', 'HF00011'),
                maxLengthText: _S('MESSAGE', 'HF00016')
            },
            items: [{
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'CONFIRM_FOLLOWING_SETTINGS')
            }, gridConfirm],
            loadData: function() {
                var form = [
                    wizard.down('#task-name'),
                    wizard.down('#sync-type'),
                    wizard.down('#remain-remote-files'),
                    wizard.down('#remain-local-files'),
                    wizard.down('#convert-google-office-files'),
                    wizard.down('#force-sync-malicious-files')
                ],
                    type = wizard.down('#sync-type').getValue(),
                    panel;

                if (type === 'two_way') {
                    panel = wizard.down('#panel-two-way-setting');
                } else if (type === 'download') {
                    panel = wizard.down('#panel-download-setting');
                } else if (type === 'upload') {
                    panel = wizard.down('#panel-upload-setting');
                }

                form.splice(2, 0,
                    panel.down('#local-path-selector').down('#path'),
                    panel.down('#remote-path-selector').down('#path')
                );

                var datas = [];

                form.forEach(function(item) {
                    var name = '';
                    var value = '';

                    if (!item.isVisible()) return false;

                    if (item.boxLabel) {
                        name = item.boxLabel;
                    } else {
                        name = item.getFieldLabel();
                    }

                    if (item.getDisplayValue) {
                        value = item.getDisplayValue();
                    } else if (item.getValue() === true) {
                        value = _S('COMMON', 'ENABLE');
                    } else if (item.getValue() === false) {
                        value = _S('COMMON', 'DISABLE');
                    } else {
                        value = item.getValue() || AS.ARC.datas.defaultValue;
                    }

                    datas.push({
                        name: name,
                        value: value
                    });
                });

                gridConfirm.store.loadRawData(datas);
            },
            onApply: function() {
                var urlOpts = {},
                    params = {};
                if (wizard.type === wizard.CREATE_CONNECTION) {
                    urlOpts = { "act": 'add_new_connection' };
                    params = {
                        "connection_setting": JSON.stringify(wizard.getConnectionSetting()),
                        "task_setting": JSON.stringify(wizard.getTaskSetting())
                    };
                } else if (wizard.type === wizard.CREATE_TASK) {
                    urlOpts = { "act": 'set_task_setting' };
                    params = {
                        "connection_id": wizard.connectionId,
                        "task_setting": JSON.stringify(wizard.getTaskSetting())
                    };
                }
                wizard.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(wizard.fn.apiUrl, urlOpts),
                    params: params,
                    method: 'get',
                    success: function (json) {
                        wizard.close();
                        if (wizard.type === wizard.CREATE_CONNECTION) {
                            wizard.fn.syncList.loadSyncList();
                        } else if (wizard.type === wizard.CREATE_TASK) {
                            wizard.fn.syncTabPanel.activeTab.loadData('set-task-setting');
                        }
                    },
                    failure: function (json) {
                        AS.ARC.util.showMsgWindow({
                            5008: _S('DATASYNC_CENTER', 'ACCOUNT_EXISTS_WARNING')
                        }, json, wizard);
                    }
                });
            }
        });
    },

    getLocalPathSelector: function(direction) {
        var wizard = this;
        var path = Ext.create('Ext.form.Text', {
            itemId: 'path',
            labelWidth: wizard.LABEL_WIDTH,
            width: wizard.WIDTH,
            readOnly: true,
            style: 'margin-right: 12px;',
            fieldLabel: _S('DATASYNC_CENTER', 'LOCAL_PATH'),
            allowBlank: false,
            msgTarget: AS.ARC.config.msgTarget,
            blankText: _S('MESSAGE', 'HF00011'),
            validator: function(value) {
                var isPathUsed = false;
                if (Ext.isEmpty(value)) return true;

                if (wizard.type !== wizard.CREATE_CONNECTION) {
                    wizard.fn.syncTabPanel.taskListPanel.getGridTaskList().store.each(function(record) {
                        var newPath = value + '/',
                            existedPath = record.data.local_path + '/';
                        if (existedPath.indexOf(newPath) === 0 || newPath.indexOf(existedPath) === 0) {
                            isPathUsed = true;
                            return false;
                        }
                    });
                }

                if (isPathUsed) {
                    return _S('DATASYNC_CENTER', 'PATH_USED_WARNING');
                } else if (value.split('/').some(function(folder) {
                    return folder === '#Recycle' || folder === '.datasync-center';
                })) {
                    return _S('DATASYNC_CENTER', 'PATH_UNABLE_WARNING');
                } else {
                    return true;
                }
            },
            listeners: {
                change: function(me) {
                    if (direction === 'download') return true;
                    var remote = me.ownerCt.ownerCt.down('#remote-path-selector').down('#path'),
                        checkbox = me.ownerCt.ownerCt.down('#select-files-folders'),
                        tree = me.ownerCt.ownerCt.down('syncTree');
                    if (direction === 'two_way') {
                        checkbox.setDisabled(
                            Ext.isEmpty(me.getValue()) || Ext.isEmpty(remote.getValue())
                        );
                    } else {
                        checkbox.setDisabled(
                            Ext.isEmpty(me.getValue())
                        );
                    }

                    if (!Ext.isEmpty(tree)) {
                        tree.setLocalPath(me);
                        tree.refreshTree();
                    }
                }
            }
        });

        var browse = Ext.create('Ext.button.Button', {
            cls: 'image-btn',
            iconCls: 'local-btn',
            handler: function() {
                var fs = Ext.create('AS.ARC.apps.datasynccenter.localTreeSelector', {
                    cls: 'app-datasync-center local-selector',
                    parentWin: wizard,
                    objectPath: path,
                    hasRefresh: true,
                    cancelAjax: true,
                    isShowHome: true,
                    hideUserName: true
                });
                fs.win.show();
            }
        });

        return Ext.create('AS.ARC.flexbox', {
            cls: direction === 'upload' ? '' : 'common-layout-horizontal-3-0',
            itemId: 'local-path-selector',
            items: [path, browse]
        });
    },

    getRemotePathSelector: function(direction) {
        var wizard = this;
        var path = Ext.create('Ext.form.Text', {
            itemId: 'path',
            labelWidth: wizard.LABEL_WIDTH,
            width: wizard.WIDTH,
            readOnly: true,
            style: 'margin-right: 12px;',
            fieldLabel: _S('DATASYNC_CENTER', 'REMOTE_PATH'),
            allowBlank: false,
            msgTarget: AS.ARC.config.msgTarget,
            blankText: _S('MESSAGE', 'HF00011'),
            getRemoteId: function() {
                return this.remoteId || '';
            },
            validator: function(value) {
                var isPathUsed = false;
                if (Ext.isEmpty(value) || wizard.type === wizard.CREATE_CONNECTION) return true;
                wizard.fn.syncTabPanel.taskListPanel.getGridTaskList().store.each(function(record) {
                    var newPath = value + '/',
                        existedPath = record.data.remote_path + '/';
                    if (existedPath.indexOf(newPath) === 0 || newPath.indexOf(existedPath) === 0) {
                        isPathUsed = true;
                        return false;
                    }
                });

                return isPathUsed ? _S('DATASYNC_CENTER', 'PATH_USED_WARNING') : true;
            },
            listeners: {
                change: function(me) {
                    if (direction === 'upload') return true;
                    var local = me.ownerCt.ownerCt.down('#local-path-selector').down('#path'),
                        checkbox = me.ownerCt.ownerCt.down('#select-files-folders'),
                        tree = me.ownerCt.ownerCt.down('syncTree');
                    if (direction === 'two_way') {
                        checkbox.setDisabled(
                            Ext.isEmpty(me.getValue()) || Ext.isEmpty(local.getValue())
                        );
                    } else {
                        checkbox.setDisabled(
                            Ext.isEmpty(me.getValue())
                        );
                    }

                    if (!Ext.isEmpty(tree)) {
                        tree.setRemotePath(me);
                        tree.refreshTree();
                    }
                }
            }
        });

        var browse = Ext.create('Ext.button.Button', {
            cls: 'image-btn',
            iconCls: 'remote-btn',
            handler: function() {
                var fs = Ext.create('AS.ARC.apps.datasynccenter.remoteTreeSelector', {
                    cls: 'app-datasync-center remote-selector',
                    parentWin: wizard,
                    objectPath: path,
                    connectionId: wizard.connectionId
                });
                fs.win.show();
            }
        });

        return Ext.create('AS.ARC.flexbox', {
            cls: direction === 'download' ? '' : 'common-layout-horizontal-3-0',
            itemId: 'remote-path-selector',
            items: [
                path,
                browse, {
                    xtype: 'box',
                    width: 16,
                    height: 16,
                    autoEl: {
                        tag: 'img',
                        src: AS.ARC.util.fixDc('resources/images/icons/common_16x16/common-icon_16X16_info.png'),
                        style: 'margin-left: 4px; width: 16px; height: 16px; vertical-align: middle',
                        'data-qtip': _S('DATASYNC_CENTER', 'SELECTING_ROOT_INFO')
                    }
                }]
        });
    },

    getConnectionSetting: function() {
        var wizard = this,
            str = wizard.down('scheduletable').getSchedule(),
            connectionSetting = {
                "connection_id": wizard.connectionId,
                "connection_name": wizard.down('#task-name').getValue(),
                "schedule": {
                    "weekday": []
                },

                "upload_bw": 0,
                "download_bw": 0,
                "log_size": 1000,
                "concurrent_transfer": 6
            };

        for (var i = 0; i < 7; i++) {
            connectionSetting.schedule.weekday.push(str.substr(i * 24, 24));
        }

        return connectionSetting;
    },

    getTaskSetting: function() {
        var wizard = this;
        var pathPanel,
            syncType = wizard.down('#sync-type').getValue(),
            list = wizard.down('fileextensionfilter').getList();

        if (syncType === 'two_way') {
            pathPanel = wizard.down('#panel-two-way-setting');
        } else if (syncType === 'upload') {
            pathPanel = wizard.down('#panel-upload-setting');
        } else if (syncType === 'download') {
            pathPanel = wizard.down('#panel-download-setting');
        }

        var tree = pathPanel.down('syncTree'),
            remainRemote = wizard.down('#remain-remote-files'),
            remainLocal = wizard.down('#remain-local-files'),
            taskSetting = {
                "task_id": 0,
                "local_root": pathPanel.down('#local-path-selector').down('#path').getValue(),
                "remote_root": pathPanel.down('#remote-path-selector').down('#path').getRemoteId(),
                "local_root_path": pathPanel.down('#local-path-selector').down('#path').getValue(),
                "remote_root_path": pathPanel.down('#remote-path-selector').down('#path').getValue(),
                "delete_remote": remainRemote.isHidden() ? true : !remainRemote.getValue(),
                "delete_local": remainLocal.isHidden() ? true : !remainLocal.getValue(),
                "sync_abuse": wizard.down('#force-sync-malicious-files').getValue(),
                "trans_gdoc": wizard.down('#convert-google-office-files').getValue(),
                "size_limit": wizard.down('#size-limit').getValue(),
                "sync_type": syncType,
                "filter": {
                    "name": list.checkedList,
                    "include_path": [],
                    "exclude_path": [],
                    "user_define": list.customizedList
                }
            };

        if (Ext.isEmpty(tree)) {
            taskSetting.filter.include_path.push('/');
        } else {
            Ext.apply(taskSetting.filter, tree.getPath());
        }

        return taskSetting;
    },

    getNoSyncSelectionWarning: function(parentWin) {
        var win = Ext.create('AS.ARC.msgWindow', {
            parentWin: parentWin,
            title: _S('COMMON', 'WARNING'),
            width: 400,
            height: 180,
            iconType: 'warning',
            asItems: [{
                xtype: 'displayfield',
                value: _S('DATASYNC_CENTER', 'PLEASE_SELECT_AT_LEAST_ONE')
            }],
            fbar: [{
                text: _S('COMMON', 'OK'),
                handler: function() {
                    win.close();
                },
                scope: this
            }]
        });
        return win;
    },
    /**
     * @override onCancel
     */
    onCancel: function() {
        var wizard = this,
            cancleWizard = function() {
                wizard.fireEvent('beforeWizardClose', wizard, wizard.content.layout.activeItem);
                wizard.close();
                wizard.fireEvent('wizardClose', wizard, wizard.content.layout.activeItem);
            };
        if (wizard.fn.syncList.getStore().getCount() !== 0) {
            cancleWizard();
        } else if (wizard.imported) {
            wizard.imported = false;
            cancleWizard();
        } else {
            var buttonOk = Ext.create('Ext.button.Button', {
                text: _S('COMMON', 'OK'),
                handler: function() {
                    cancleWizard();
                    winConfirm.close();
                    wizard.fn.win.close();
                },
                scope: this
            });

            var buttonCancel = Ext.create('Ext.button.Button', {
                text: _S('COMMON', 'CANCEL'),
                handler: function() { winConfirm.close(); },
                scope: this
            });

            var winConfirm = new AS.ARC.msgWindow({
                parentWin: wizard,
                title: _S('COMMON', 'CONFIRMATION'),
                width: 400,
                height: 180,
                iconType: 'question',
                asItems: [{
                    xtype: 'displayfield',
                    value: _S('DATASYNC_CENTER', 'CONFIRM_TO_EXIT')
                }],
                fbar: [
                    buttonOk,
                    buttonCancel
                ]
            });

            winConfirm.show();
        }
    },
});

/**
 * @class AS.ARC.apps.datasynccenter.syncListToolbar
 * @extends AS.ARC.flexbox
 */
Ext.define('AS.ARC.apps.datasynccenter.syncListToolbar', {
    extend: 'AS.ARC.flexbox',
    alias: ['widget.syncListToolbar'],
    cls: 'sync-list-toolbar',

    height: 40,

    initComponent: function() {
        var toolbar = this;
        this.items = [{
            xtype: 'button',
            cls: 'image-btn',
            iconCls: 'add-btn',
            listeners: {
                click: toolbar.handleAdd.bind(toolbar)
            }
        }, {
            xtype: 'button',
            cls: 'image-btn',
            iconCls: 'remove-btn',
            listeners: {
                click: toolbar.handleRemove.bind(toolbar)
            }
        }];

        this.callParent(arguments);
    },

    handleAdd: function() {
        var toolbar = this;
        toolbar.wizard = Ext.create('AS.ARC.apps.datasynccenter.syncListToolbar.wizard', {
            fn: toolbar.fn,
            parentWin: toolbar.fn.win,
            title: _S('DATASYNC_CENTER', 'CREATE_NEW_TASK'),
            type: 'create_connection'
        });

        toolbar.wizard.show();
    },

    handleRemove: function() {
        var toolbar = this;
        var buttonOk = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'OK'),
            handler: function() {
                winConfirm.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(toolbar.fn.apiUrl, { act: 'remove_connection' }),
                    params: {
                        connection_id: toolbar.fn.syncList.getConnectionId()
                    },
                    method: 'post',
                    success: function() {
                        winConfirm.close();
                        toolbar.fn.syncList.loadSyncList();
                    },
                    failure: function(json) {
                        winConfirm.close();
                        AS.ARC.util.showMsgWindow({}, json, toolbar.fn.win);
                    }
                });
            },
            scope: this
        });

        var buttonCancel = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'CANCEL'),
            handler: function() {
                winConfirm.close();
            },
            scope: this
        });

        var winConfirm = new AS.ARC.msgWindow({
            parentWin: toolbar.fn.win,
            title: _S('COMMON', 'CONFIRMATION'),
            width: 400,
            height: 180,
            iconType: 'question',
            asItems: [{
                xtype: 'displayfield',
                value: _S('DATASYNC_CENTER', 'CONNECTION_REMOVE_CONFIRM')
            }],
            fbar: [
                buttonOk,
                buttonCancel
            ]
        });

        winConfirm.show();
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.searchField
 * @extends AS.ARC.asSearchField
 */
Ext.define('AS.ARC.apps.datasynccenter.searchField', {
    extend: 'AS.ARC.asSearchField',
    cls: 'as-search-field datasync-center-with-advanced-search',

    initComponent: function(config) {
        this.on('afterrender', function(textField) {

            textField.inputEl.dom.style.paddingRight = '25px';

            textField.clearButtonEl = textField.bodyEl.createChild({
                tag: 'div',
                cls: 'datasync-center-search-clear status-icon-16 clear'
            });

            textField.clearButtonEl.on('click', function() {
                textField.setValue('');
            });

            textField.clearButtonEl.setDisplayed(false);
        });

        this.on('change', function(textField, value) {
            if (value === '') {
                textField.clearButtonEl.setDisplayed(false);
            } else {
                textField.clearButtonEl.setDisplayed(true);
            }

        });

        this.callParent(arguments);
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.syncTabPanel.statusPanel
 * @extends Ext.tab.Panel
 */
Ext.define('AS.ARC.apps.datasynccenter.syncTabPanel.statusPanel', {
    extend: 'Ext.form.Panel',
    border: false,
    cls: 'as-page-panel',
    itemId: 'status-panel',

    STATUS_STRING: {},
    STATUS: {
        DONE: 'done',
        SYNCING: 'syncing',
        PAUSE: 'pause',
        SCHEDULED: 'scheduled',
        AUTH_FAILED: 'auth_failed',
        LOST_LOCAL_FOLDER: 'lost_local_folder',
        LOST_REMOTE_FOLDER: 'lost_remote_folder',
        LOCAL_NO_SPACE: 'local_no_space',
        REMOTE_NO_SPACE: 'remote_no_space',
        DISCONNECTED: 'disconnected',
        EZSYNC_VOLUME_FAILED: 'ezsync_volume_failed'
    },
    STATUS_LIST: [
        'done',
        'syncing',
        'pause',
        'scheduled',
        'auth_failed',
        'lost_local_folder',
        'lost_remote_folder',
        'local_no_space',
        'remote_no_space',
        'disconnected',
        'ezsync_volume_failed'
    ],

    initComponent: function() {
        this.items = [this.getItems()];
        this.init();
        this.callParent(arguments);
    },

    init: function() {
        this.title = _S('DATASYNC_CENTER', 'STATUS');
        this.STATUS_STRING = {
            done: _S('DATASYNC_CENTER', 'UP_TO_DATE'),
            syncing: _S('DATASYNC_CENTER', 'SYNCHRONIZATION_IN_PROGRESS'),
            pause: _S('DATASYNC_CENTER', 'PAUSED'),
            scheduled: _S('DATASYNC_CENTER', 'SCHEDULED'),
            auth_failed: _S('DATASYNC_CENTER', 'FAILED_TO_CONNECT'),
            lost_local_folder: _S('DATASYNC_CENTER', 'LOCAL_FOLDER_MISSING'),
            lost_remote_folder: _S('DATASYNC_CENTER', 'REMOTE_FOLDER_MISSING'),
            local_no_space: _S('DATASYNC_CENTER', 'LOCAL_STORAGE_FULL'),
            remote_no_space: _S('DATASYNC_CENTER', 'REMOTE_STORAGE_FULL'),
            disconnected: _S('DATASYNC_CENTER', 'FAILED_TO_CONNECT'),
            ezsync_volume_failed: _S('DATASYNC_CENTER', 'FAILED_TO_CONNECT')
        }
    },

    getItems: function() {
        var panel = this;
        return [{
            xtype: 'asFlexbox',
            itemId: 'status',
            margin: '15 0 0 20',
            style: 'align-items: flex-start;',
            hidden: true,
            items: [{
                xtype: 'component',
                itemId: 'status-icon',
                autoEl: {
                    cls: 'sync-status-icon pause'
                }
            }, {
                xtype: 'container',
                items: [{
                    xtype: 'container',
                    cls: 'common-layout-vertical-3-0',
                    style: 'min-height: 150px',
                    margin: '0 0 8 0',
                    items: [{
                        xtype: 'displayfield',
                        itemId: 'status-title',
                        cls: 'status-title displayfield',
                        value: AS.ARC.datas.defaultValue
                    }, {
                        xtype: 'displayfield',
                        itemId: 'status-info',
                        style: 'align-items: start;',
                        width: 450,
                        height: 52,
                        value: AS.ARC.datas.defaultValue
                    }, {
                        xtype: 'errorfilelist',
                        itemId: 'error-file-list',
                        cls: 'common-layout-horizontal-3-0',
                        fn: panel.tabPanel.fn,
                        hidden: true
                    }, {
                        xtype: 'tasklistlink',
                        itemId: 'uploading-task',
                        cls: 'info',
                        textItemId: 'uploading-task-text',
                        textValue: '<a class="hyperlink-string" href="#">' + _S('DATASYNC_CENTER', 'TASKS') + '</a>',
                        click: function() {
                            panel.uploadingTaskListSubWindow = Ext.create('AS.ARC.apps.datasynccenter.uploadingTaskListSubWindow', {
                                parentWin: panel.tabPanel.fn.win,
                                clickHyperlink: function() {
                                    panel.tabPanel.setActiveTab(2);
                                    panel.uploadingTaskListSubWindow.hide();
                                },
                            });
                            panel.uploadingTaskListSubWindow.show();
                        }
                    }, {
                        xtype: 'tasklistlink',
                        itemId: 'error-task',
                        cls: 'warning',
                        textItemId: 'link',
                        textValue: Ext.String.format(_S('DATASYNC_CENTER', 'SOME_TASK_SOME_PROBLEM'), '<a class="hyperlink-string" href="#">', '</a>'),
                        click: function() {
                            panel.tabPanel.setActiveTab('task-list-panel');
                        }
                    }]
                }, panel.getActionButtons()]
            }]
        }, {
            xtype: 'fieldset',
            itemId: 'info',
            title: _S('DATASYNC_CENTER', 'SYNCHRONIZATION_INFO'),
            defaults: {
                labelWidth: 345,
                width: 680,
                style: 'margin-top: 20px;'
            },
            hidden: true,
            items: [{
                xtype: 'displayfield',
                fieldLabel: _S('DATASYNC_CENTER', 'NAME'),
                itemId: 'connection-name',
                fieldStyle: 'overflow: hidden; text-overflow: ellipsis; width: 100%; white-space: nowrap;',
                value: AS.ARC.datas.defaultValue
            }, {
                xtype: 'displayfield',
                fieldLabel: _S('COMMON', 'USERNAME'),
                itemId: 'user-name',
                value: AS.ARC.datas.defaultValue
            }, {
                xtype: 'displayfield',
                fieldLabel: _S('DATASYNC_CENTER', 'USAGE'),
                itemId: 'storage-usage',
                value: AS.ARC.datas.defaultValue
            }, {
                xtype: 'progressbar',
                cls: 'progressbar-size',
                style: 'margin-top: 30px;',
                value: 0,
                listeners: {
                    update: function(pb, value) {
                        if (value >= 1) {
                            pb.addCls('x-progress-error');
                        } else {
                            pb.removeCls('x-progress-error');
                        }
                    }
                }
            }]
        }];
    },

    getActionButtons: function() {
        var panel = this;
        return {
            xtype: 'asPanelToolbarTop',
            defaults: {
                scope: panel
            },
            items: [{
                xtype: 'button',
                itemId: 'act-btn-run',
                width: 75,
                cls: 'action-btn',
                iconCls: 'action-run',
                handler: panel.handleRun
            }, {
                xtype: 'button',
                itemId: 'act-btn-pause',
                width: 75,
                cls: 'action-btn',
                iconCls: 'action-pause',
                handler: panel.handlPause,
                hidden: true
            }, {
                xtype: 'button',
                itemId: 'act-btn-reauthorize',
                text: _S('DATASYNC_CENTER', 'REAUTHORIZE'),
                handler: panel.handleReauthorize,
                hidden: true
            }, {
                xtype: 'button',
                itemId: 'act-btn-reselect-lost-folder',
                text: _S('DATASYNC_CENTER', 'NEW_SYNC_FOLDER'),
                handler: panel.handleReselectLostFolder,
                hidden: true
            }, {
                xtype: 'button',
                itemId: 'act-btn-setting',
                text: _S('COMMON', 'SETTING'),
                handler: panel.handleSetting
            }]
        };
    },

    handleRunAndPause: function(act) {
        var panel = this;
        panel.tabPanel.fn.syncList.maskView();
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: act }),
            params: {
                connection_id: panel.tabPanel.fn.syncList.getConnectionId()
            },
            method: 'post',
            success: function() {
                panel.tabPanel.fn.syncList.loadSyncList();
            },
            failure: function(json) {
                AS.ARC.util.showMsgWindow({}, json, panel.tabPanel.fn.win);
            }
        });
    },

    handleRun: function() {
        this.handleRunAndPause('start_connection');
    },

    handlPause: function() {
        this.handleRunAndPause('stop_connection');
    },

    handleReauthorize: function() {
        var panel = this,
            connection = panel.tabPanel.fn.syncList.getData(),
            pollingList = panel.tabPanel.fn.syncList.reauthorizeList,
            SERVICE_LIST = {
                EZ_SYNC: 'ezsync',
                BAIDU: 'baidu',
                DROPBOX: 'dropbox',
                GOOGLE_DRIVE: 'googledrive',
                ONE_DRIVE: 'onedrive',
                YANDEX: 'yandex'
            };

        var openAuthorizeWin = function() {
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'get_login_info' }),
                method: 'post',
                success: function(json) {
                    if (!Ext.isEmpty(json.login_info)) {
                        if (connection.service_type === SERVICE_LIST.EZ_SYNC) {
                            var loginWin = Ext.create('AS.ARC.apps.datasynccenter.ezsyncLoginWin', {
                                parentWin: panel.tabPanel.fn.win,
                                isReauthorize: true,
                                connectionId: connection.connection_id
                            });
                            loginWin.show();
                        } else {
                            url = json.login_info[connection.service_type].login_url + encodeURIComponent('&adm_url=' + encodeURIComponent(window.location.href.split('portal').shift()));
                            window.open(url, "newwindow", "height=700, width=600, toolbar=no, menubar=no, scrollbars=no, resizable=no, location=no, status=no");
                        }
                        if (pollingList.indexOf(connection.connection_id) === -1) {
                            pollingList.push(connection.connection_id);
                        }
                    }
                },
                failure: function(json) {}
            });
        };

        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'cancel_new_connection' }),
            method: 'post',
            success: function() { openAuthorizeWin(); },
            failure: function() { openAuthorizeWin(); }
        });
    },

    handleSetting: function() {
        var panel = this;

        var panelSchedule = Ext.create('AS.ARC.panelTop20', {
            title: _S('EXT_BACKUP', 'SCHEDULE'),
            itemId: 'schedule',
            items: [{
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'PLEASE_SET_TIME')
            }, {
                xtype: 'scheduletable',
                style: AS.layout.getIndentCss(1),
                suspendString: _S('DATASYNC_CENTER', 'PAUSE'),
                syncString: _S('FILE_EXPLORER', 'DATA_SYNCHRONIZE')
            }, {
                xtype: 'displayfield',
                cls: 'warning-string',
                value: _S('DATASYNC_CENTER', 'SYNCHRONIZATION_END_IF_PAUSED'),
                style: AS.layout.getIndentCss(1) + ';margin-top: 20px'
            }]
        });

        var panelAdvanced = Ext.create('AS.ARC.panelTop20', {
            title: _S('DATASYNC_CENTER', 'ADVANCED'),
            itemId: 'advanced',
            items: [{
                xtype: 'fieldset',
                itemId: 'general',
                title: _S('COMMON', 'GENERAL'),
                defaults: {
                    labelWidth: 230,
                    allowBlank: false,
                    blankText: _S('MESSAGE', 'HF00011'),
                    maxLengthText: _S('MESSAGE', 'HF00016'),
                    msgTarget: AS.ARC.config.msgTarget,
                },
                items: [{
                    xtype: 'textfield',
                    fieldLabel: _S('DATASYNC_CENTER', 'NAME'),
                    itemId: 'connection-name',
                    maxLength: 255,
                    width: 500,
                    validator: function(value) {
                        var me = this,
                            store;

                        if (me.isDirty()) {
                            store = panel.tabPanel.fn.syncList.store;
                            if (store.findExact('connection_name', value) >= 0) {
                                return _S('DATASYNC_CENTER', 'NAME_EXISTS_WARNING');
                            }
                        }
                        return true;
                    }
                }, {
                    xtype: 'combo',
                    itemId: 'max-logs',
                    fieldLabel: _S('DATASYNC_CENTER', 'MAX_LOGS'),
                    store: new Ext.data.ArrayStore({
                        fields: ['display', 'value'],
                        data: [
                            ['1000', 1000],
                            ['3000', 3000],
                            ['5000', 5000],
                            ['10000', 10000],
                            ['30000', 30000],
                            ['50000', 50000],
                            ['100000', 100000],
                        ]
                    }),
                    displayField: 'display',
                    valueField: 'value',
                    queryMode: 'local',
                    editable: false,
                    lastQuery: '',
                    value: 1000,
                    width: 335
                }, {
                    xtype: 'combo',
                    itemId: 'max-simultaneous-transfers',
                    fieldLabel: _S('DATASYNC_CENTER', 'MAXIMUM_SIMULTANEOUS_TRANSFERS'),
                    store: new Ext.data.ArrayStore({
                        fields: ['display', 'value'],
                        data: (function(start, end) {
                            var data = [];
                            for (var i = start; i <= end; i++) {
                                data.push([i.toString(), i]);
                            }
                            return data;
                        })(2, 8)
                    }),
                    displayField: 'display',
                    valueField: 'value',
                    queryMode: 'local',
                    editable: false,
                    lastQuery: '',
                    value: 6,
                    width: 335
                }]
            }, {
                xtype: 'fieldset',
                itemId: 'bandwidth',
                title: _S('DATASYNC_CENTER', 'TRAFFIC_CONTROL'),
                items: [{
                    xtype: 'asFlexbox',
                    defaults: {
                        labelWidth: 230,
                        allowBlank: false,
                        blankText: _S('MESSAGE', 'HF00011'),
                        msgTarget: AS.ARC.config.msgTarget,
                    },
                    items: [{
                        xtype: 'textfield',
                        fieldLabel: _S('DATASYNC_CENTER', 'MAX_UPLOAD_SPEED'),
                        itemId: 'max-upload-speed',
                        value: '0',
                        width: 365
                    }, {
                        xtype: 'component',
                        html: _S('DATASYNC_CENTER', 'KB_PER_SECOND'),
                        style: 'margin-left: 10px'
                    }]
                }, {
                    xtype: 'asFlexbox',
                    cls: 'common-layout-horizontal-3-0',
                    defaults: {
                        labelWidth: 230,
                        allowBlank: false,
                        blankText: _S('MESSAGE', 'HF00011'),
                        msgTarget: AS.ARC.config.msgTarget,
                    },
                    items: [{
                        xtype: 'textfield',
                        fieldLabel: _S('DATASYNC_CENTER', 'MAX_DOWNLOAD_SPEED'),
                        itemId: 'max-download-speed',
                        value: '0',
                        width: 365
                    }, {
                        xtype: 'component',
                        html: _S('DATASYNC_CENTER', 'KB_PER_SECOND'),
                        style: 'margin-left: 10px'
                    }]
                }]
            }]
        });

        var buttonApply = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'APPLY'),
            disabled: true,
            handler: function() {
                win.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'set_connection_setting' }),
                    params: {
                        connection_setting: JSON.stringify(win.getConnectionSetting())
                    },
                    method: 'post',
                    success: function() {
                        if (panelAdvanced.down('#connection-name').isDirty()) {
                            panel.tabPanel.fn.syncList.loadSyncList();
                        }
                        win.close();
                    },
                    failure: function(json) {
                        win.close();
                        AS.ARC.util.showMsgWindow({}, json, panel.tabPanel.fn.win);
                    }
                });
            },
            scope: this
        });

        var tabPanel = Ext.create('AS.ARC.formTabPanel', {
            buttonApply: buttonApply,
            items: [
                panelSchedule,
                panelAdvanced,
            ],
            deferredRender: false
        });

        var win = Ext.create('AS.ARC.subWindow', {
            title: _S('COMMON', 'SETTING'),
            parentWin: panel.tabPanel.fn.win,
            width: 700,
            height: 550,
            layout: 'fit',
            items: [tabPanel],
            fbar: {
                items: [
                    buttonApply, {
                    text: _S('COMMON', 'CANCEL'),
                    scale: AS.ARC.config.defaultApplyButtonSize,
                    handler: function() {
                        win.close();
                    },
                    scope: this
                }]
            },
            updateFields: function() {
                var data = this.fetchedData;
                panelSchedule.down('scheduletable').setSchedule(data.schedule.weekday.join(''));
                panelAdvanced.down('#connection-name').originalValue = data.connection_name;
                panelAdvanced.down('#connection-name').setValue(data.connection_name);
                panelAdvanced.down('#max-logs').setValue(data.log_size);
                panelAdvanced.down('#max-upload-speed').setValue(data.upload_bw);
                panelAdvanced.down('#max-download-speed').setValue(data.download_bw);
                panelAdvanced.down('#max-simultaneous-transfers').setValue(data.concurrent_transfer);
            },
            getConnectionSetting: function() {
                var str = panelSchedule.down('scheduletable').getSchedule(),
                    connectionSetting = {
                        "connection_id": panel.tabPanel.fn.syncList.getConnectionId(),
                        "connection_name": panelAdvanced.down('#connection-name').getValue(),
                        "schedule": {
                            "weekday": []
                        },
                        "upload_bw": panelAdvanced.down('#max-upload-speed').getValue(),
                        "download_bw": panelAdvanced.down('#max-download-speed').getValue(),
                        "log_size": panelAdvanced.down('#max-logs').getValue(),
                        "concurrent_transfer": panelAdvanced.down('#max-simultaneous-transfers').getValue()
                    };

                for (var i = 0; i < 7; i++) {
                    connectionSetting.schedule.weekday.push(str.substr(i * 24, 24));
                }

                return connectionSetting;
            },
            listeners: {
                afterrender: function() {
                    win.el.mask(_S('COMMON','LOADING'));
                    AS.ARC.ajax({
                        url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'get_connection_setting' }),
                        params: {
                            connection_id: panel.tabPanel.fn.syncList.getConnectionId()
                        },
                        method: 'get',
                        success: function(json) {
                            win.fetchedData = json.connection_setting;
                            win.updateFields();
                            win.el.unmask();
                        },
                        failure: function(json) {
                            AS.ARC.util.showMsgWindow({}, json, win);
                        }
                    });
                }
            }
        });

        win.show();
    },

    handleReselectLostFolder: function() {
        var panel = this,
            isLostLocalFolder = (panel.tabPanel.fn.syncList.getData().status === panel.STATUS.LOST_LOCAL_FOLDER);

        var buttonApply = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'APPLY'),
            disabled: true,
            handler: function() {
                win.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'set_task_setting' }),
                    params: {
                        connection_id: panel.tabPanel.fn.syncList.getConnectionId(),
                        task_setting: JSON.stringify(win.getTaskSetting())
                    },
                    method: 'get',
                    success: function() {
                        win.close();
                    },
                    failure: function(json) {
                        AS.ARC.util.showMsgWindow({}, json, win);
                    }
                });
            },
            scope: this
        });

        var localPath = Ext.create('Ext.form.Text', {
            itemId: 'path',
            labelWidth: 200,
            width: 500,
            readOnly: true,
            style: 'margin-right: 12px;',
            fieldLabel: _S('DATASYNC_CENTER', 'LOCAL_PATH'),
            allowBlank: false,
            msgTarget: AS.ARC.config.msgTarget,
            blankText: _S('MESSAGE', 'HF00011'),
            validator: function(value) {
                if (value.split('/').some(function(folder) {
                    return folder === '#Recycle' || folder === '.datasync-center';
                })) {
                    return _S('DATASYNC_CENTER', 'PATH_UNABLE_WARNING');
                } else {
                    return true;
                }
            }
        });

        var panelLocalFolder = Ext.create('AS.ARC.panelTop20', {
            buttonApply: buttonApply,
            itemId: 'filter',
            items: [{
                xtype: 'asFlexbox',
                items: [localPath, {
                    xtype: 'button',
                    cls: 'image-btn',
                    iconCls: 'local-btn',
                    handler: function() {
                        var fs = Ext.create('AS.ARC.apps.datasynccenter.localTreeSelector', {
                            cls: 'app-datasync-center local-selector',
                            parentWin: win,
                            objectPath: localPath,
                            hasRefresh: true,
                            cancelAjax: true,
                            isShowHome: true,
                            hideUserName: true
                        });
                        fs.win.show();
                    }
                }]
            }]
        });

        var remotePath = Ext.create('Ext.form.Text', {
            itemId: 'path',
            labelWidth: 200,
            width: 500,
            readOnly: true,
            style: 'margin-right: 12px;',
            fieldLabel: _S('DATASYNC_CENTER', 'REMOTE_PATH'),
            allowBlank: false,
            msgTarget: AS.ARC.config.msgTarget,
            blankText: _S('MESSAGE', 'HF00011'),
            getRemoteId: function() {
                return this.remoteId || '';
            }
        });

        var panelRemoteFolder = Ext.create('AS.ARC.panelTop20', {
            buttonApply: buttonApply,
            itemId: 'filter',
            items: [{
                xtype: 'asFlexbox',
                items: [remotePath, {
                    xtype: 'button',
                    cls: 'image-btn',
                    iconCls: 'remote-btn',
                    handler: function() {
                        var fs = Ext.create('AS.ARC.apps.datasynccenter.remoteTreeSelector', {
                            cls: 'app-datasync-center remote-selector',
                            parentWin: win,
                            objectPath: remotePath,
                            connectionId: panel.tabPanel.fn.syncList.getConnectionId()
                        });
                        fs.win.show();
                    }
                }]
            }]
        });

        var items = [isLostLocalFolder ? panelLocalFolder : panelRemoteFolder];

        var win = Ext.create('AS.ARC.subWindow', {
            title: _S('DATASYNC_CENTER', 'NEW_SYNC_FOLDER'),
            parentWin: panel.tabPanel.fn.win,
            cls: 'app-datasync-center-wizard',
            width: 600,
            height: 180,
            layout: 'fit',
            items: items,
            fbar: {
                items: [
                    buttonApply, {
                    text: _S('COMMON', 'CANCEL'),
                    scale: AS.ARC.config.defaultApplyButtonSize,
                    handler: function() {
                        win.close();
                    },
                    scope: this
                }]
            },
            getTaskSetting: function() {
                if (isLostLocalFolder) {
                    Ext.apply(win.fetchedData, {
                        local_root: localPath.getValue(),
                        local_root_path: localPath.getValue()
                    });
                } else {
                    Ext.apply(win.fetchedData, {
                        remote_root: remotePath.getRemoteId(),
                        remote_root_path: remotePath.getValue()
                    });
                }
                return win.fetchedData;
            },
            listeners: {
                afterrender: function() {
                    win.el.mask(_S('COMMON', 'LOADING'));
                    AS.ARC.ajax({
                        url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'list_task_status' }),
                        params: {
                            connection_id: panel.tabPanel.fn.syncList.getConnectionId()
                        },
                        method: 'get',
                        success: function(json) {
                            var task = json.list.find(function(task) {
                                return task.status === panel.tabPanel.fn.syncList.getData().status;
                            });
                            win.taskList = json.list;
                            AS.ARC.ajax({
                                url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'get_task_setting' }),
                                params: {
                                    connection_id: panel.tabPanel.fn.syncList.getConnectionId(),
                                    task_id: task.task_id
                                },
                                method: 'get',
                                success: function(json) {
                                    win.fetchedData = json.task_setting;
                                    win.el.unmask();
                                },
                                failure: function(json) {
                                    AS.ARC.util.showMsgWindow({}, json, win);
                                }
                            });
                            
                        },
                        failure: function(json) {
                            AS.ARC.util.showMsgWindow({}, json, win);
                        }
                    });
                }
            }
        });

        win.show();
    },

    checkButtons: function(data) {
        var panel = this,
            btnRun = panel.down('#act-btn-run'),
            btnPause = panel.down('#act-btn-pause'),
            btnReauthorize = panel.down('#act-btn-reauthorize'),
            btnReselectLostFolder = panel.down('#act-btn-reselect-lost-folder');
        
        btnRun.setVisible(!data.running);
        btnPause.setVisible(btnRun.isHidden());
        btnReauthorize.setVisible(data.status === panel.STATUS.AUTH_FAILED);
        btnReselectLostFolder.setVisible(
            data.status === panel.STATUS.LOST_LOCAL_FOLDER ||
            data.status === panel.STATUS.LOST_REMOTE_FOLDER
        );
    },

    getInfo: function(data) {
        var panel = this,
            info = '', stringFormat = '';

        if (data.status === panel.STATUS.SCHEDULED) {
            var timeOffset = AS.ARC.config.dateFormat.LocalTimeGmtOffset,
                dateFormat = AS.ARC.config.dateFormat.DateFormat,
                timeFormat = AS.ARC.config.dateFormat.TimeFormat,
                time = AS.ARC.util.toSystemTime(dateFormat, timeFormat, data.schedule_status.next_time + timeOffset * 60),
                lastStatus = data.schedule_status.last_status;
            stringFormat = '<span style="display: inline-block; width: 210px;">{0}:</span>{1}<br>';
            info += Ext.String.format(stringFormat, _S('DATASYNC_CENTER', 'NEXT_SYNCHRONIZATION'), time);
            info += Ext.String.format(stringFormat, _S('DATASYNC_CENTER', 'LAST_STATUS'), '<span style="font-weight: bold;">' + panel.STATUS_STRING[lastStatus] + '</span>');
        } else if (data.status === panel.STATUS.AUTH_FAILED) {
            info += (data.service_type === 'ezsync') ? Ext.String.format(_S('DATASYNC_CENTER', 'EZ_SYNC_UNLINKED'), data.user_name.split('/').first()) : _S('DATASYNC_CENTER', 'ACCOUNT_REVOKED_EXPIRED');
        } else if (data.status === panel.STATUS.DISCONNECTED) {
            info += _S('DATASYNC_CENTER', 'CANNOT_CONNECT_PLEASE_CHECK_SETTINGS');
        } else if (data.status === panel.STATUS.EZSYNC_VOLUME_FAILED) {
            info += _S('DATASYNC_CENTER', 'UNABLE_NAS_VOLUME');
        }
        
        if (data.unsync_files > 0) {
            info += Ext.String.format(_S('DATASYNC_CENTER', 'N_FILES_NOT_BEEN_SYNCHRONIZED'), data.unsync_files);
        }

        return info;
    },

    updateStatus: function(data) {
        var panel = this,
            statusIcon = panel.down('#status-icon'),
            statusTitle = panel.down('#status-title'),
            statusInfo = panel.down('#status-info'),
            errorFileList = panel.down('#error-file-list'),
            uploadingTask = panel.down('#uploading-task'),
            errorTask = panel.down('#error-task');

        var index = Math.max(panel.STATUS_LIST.indexOf(data.status), 0);
        statusIcon.removeCls(panel.STATUS_LIST);
        statusIcon.addCls(panel.STATUS_LIST[index]);

        statusTitle.setValue(panel.STATUS_STRING[data.status]);
        statusInfo.setValue(panel.getInfo(data));
        statusInfo.setVisible(!Ext.isEmpty(statusInfo.getValue()));

        errorFileList.setData(data.failed_list);
        errorFileList.setVisible(!Ext.isEmpty(data.failed_list));

        uploadingTask.setVisible(!Ext.isEmpty(panel.tabPanel.fn.syncList.getData().trans_status));

        errorTask.setVisible(data.has_task_error);

        if (statusInfo.isHidden() && errorFileList.isHidden() && errorTask.isHidden()) {
            statusTitle.up().el.addCls('status-vertical-center');
        } else {
            statusTitle.up().el.removeCls('status-vertical-center');
        }
    },

    updateInfo: function(data) {
        var panel = this,
            connectionName = panel.down('#connection-name'),
            userName = panel.down('#user-name'),
            storageUsage = panel.down('#storage-usage'),
            progressbar = panel.down('progressbar'),
            NO_LIMITED = -1;

        connectionName.setValue(data.connection_name);
        connectionName.inputEl.set({ 'data-qtip': data.connection_name });
        userName.setValue(data.user_name);

        if (data.quota === NO_LIMITED) {
            storageUsage.setValue(Ext.String.format(
                _S('DATASYNC_CENTER', 'GB_USED'),
                AS.ARC.util.convertSizeUnit(data.normal)
            ));
        } else {
            storageUsage.setValue(Ext.String.format(
                _S('DATASYNC_CENTER', 'GB_OF_GB_USED'),
                AS.ARC.util.convertSizeUnit(data.quota),
                AS.ARC.util.convertSizeUnit(data.normal)
            ));
            progressbar.updateProgress((data.normal / data.quota));
        }

        progressbar.setVisible(data.quota !== NO_LIMITED);
    },

    detectReauthorizeError: function(data) {
        var panel = this,
            errorList = panel.tabPanel.fn.syncList.reauthorizeErrorList,
            isEzsync = (data.service_type === 'ezsync');
        if (errorList[5603].indexOf(data.connection_id) >= 0) {
            AS.ARC.util.showMsgWindow({
                5603: isEzsync ? _S('DATASYNC_CENTER', 'EZ_SYNC_REAUTHORIZE_FAILED') : _S('DATASYNC_CENTER', 'CONNECT_FAILED_WARNING')
            }, { error_code: 5603 }, panel.tabPanel.fn.win);
            errorList[5603].splice(errorList[5603].indexOf(data.connection_id), 1);
        }
    },

    loadData: function(type) {
        var panel = this,
            data = panel.tabPanel.fn.syncList.getData();

        if (Ext.isEmpty(Object.keys(data))) {
            panel.down('#status').hide();
            panel.down('#info').hide();
        } else {
            panel.down('#status').show();
            panel.down('#info').show();
            panel.checkButtons(data);
            panel.updateStatus(data);
            panel.updateInfo(data);
            panel.detectReauthorizeError(data);
        }
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.syncTabPanel.taskListPanel
 * @extends Ext.tab.Panel
 */
Ext.define('AS.ARC.apps.datasynccenter.syncTabPanel.taskListPanel', {
    extend: 'Ext.form.Panel',
    border: false,
    cls: 'as-page-panel',
    itemId: 'task-list-panel',
    layout: {
        type: 'vbox',
        align: 'stretch'
    },

    pollingID: 0,
    ajaxRequest: {},
    lastSelected: null,

    initComponent: function() {
        this.items = [this.getItems()];
        this.init();
        this.callParent(arguments);
    },

    init: function() {
        this.title = _S('DATASYNC_CENTER', 'TASK_LIST');
    },

    getItems: function() {
        var panel = this;
        return [{
            xtype: 'asPanelToolbarTop',
            defaults: {
                minWidth: 75,
                scope: this,
            },
            items: [{
                xtype: 'button',
                itemId: 'add',
                text: _S('COMMON', 'ADD'),
                handler: panel.handleAdd
            }, {
                xtype: 'button',
                itemId: 'open',
                text: _S('APPCENTRAL', 'OPEN_APP'),
                handler: panel.handleOpen,
                disabled: true
            }, {
                xtype: 'button',
                itemId: 'edit',
                text: _S('COMMON', 'EDIT'),
                handler: function() {
                    var STATUS = panel.tabPanel.statusPanel.STATUS,
                        item = panel.getSelectedItem();
                    if (item.data.status === STATUS.LOST_LOCAL_FOLDER ||
                        item.data.status === STATUS.LOST_REMOTE_FOLDER) {
                        panel.handleReselectLostFolder();
                    } else {
                        panel.handleEdit();
                    }
                },
                disabled: true
            }, {
                xtype: 'button',
                itemId: 'delete',
                text: _S('COMMON', 'DELETE'),
                handler: panel.handleDelete,
                disabled: true
            }, {
                xtype: 'button',
                itemId: 'failed-list',
                text: _S('DATASYNC_CENTER', 'INCOMPLETE_SYNCHRONIZATIONS'),
                handler: panel.handleFailedList,
                disabled: true
            }]
        }, panel.getGridTaskList()];
    },

    getSelectedItem: function() {
        return this.getGridTaskList().getSelectionModel().getLastSelected();
    },

    getGridTaskList: function() {
        var panel = this;
        if (!Ext.isEmpty(panel.gridTaskList))
            return panel.gridTaskList;

        panel.gridTaskList = Ext.create('Ext.grid.GridPanel', {
            itemId: 'grid-task-list',
            margin: '0 0 10 0',
            flex: 1,
            viewConfig: {
                loadMask: false
            },
            defaults: {
                draggable: false
            },
            columns: [{
                text: _S('COMMON', 'TYPE'),
                dataIndex: 'type',
                flex: 1.5,
                renderer: function(value, metadata) {
                    return '<div class="status-icon-16 ' + value + '"></div>';
                }
            }, {
                text: _S('DATASYNC_CENTER', 'LOCAL_PATH'),
                dataIndex: 'local_path',
                flex: 3,
                renderer: function(value, metadata) {
                    value = Ext.htmlEncode(value);
                    metadata.tdAttr = 'data-qtip="' + Ext.htmlEncode(value) + '"';
                    return value;
                }
            }, {
                text: _S('DATASYNC_CENTER', 'REMOTE_PATH'),
                dataIndex: 'remote_path',
                flex: 3,
                renderer: function(value, metadata) {
                    value = Ext.htmlEncode(value);
                    metadata.tdAttr = 'data-qtip="' + Ext.htmlEncode(value) + '"';
                    return value;
                }
            }, {
                text: _S('COMMON', 'STATUS'),
                dataIndex: 'status',
                flex: 2,
                renderer: function(value, metadata, record) {
                    var status = panel.tabPanel.statusPanel.STATUS_STRING[value],
                        STATUS = panel.tabPanel.statusPanel.STATUS,
                        result = '',
                        hasFailedList = '';

                    if (value === STATUS.AUTH_FAILED ||
                        value === STATUS.LOCAL_NO_SPACE ||
                        value === STATUS.REMOTE_NO_SPACE) {
                        metadata.tdAttr = 'data-errorqtip="' + Ext.htmlEncode(status) + '"'; 
                    } else if (value === STATUS.LOST_LOCAL_FOLDER) {
                        metadata.tdAttr = 'data-errorqtip="' + _S('DATASYNC_CENTER', 'LOCAL_FOLDER_MISSING_PLEASE_SELECT_AGAIN') + '"';
                    } else if (value === STATUS.LOST_REMOTE_FOLDER) {
                        metadata.tdAttr = 'data-errorqtip="' + _S('DATASYNC_CENTER', 'REMOTE_FOLDER_MISSING_PLEASE_SELECT_AGAIN') + '"';
                    } else {
                        metadata.tdAttr = 'data-qtip="' + Ext.htmlEncode(status) + '"';
                    }

                    if (!Ext.isEmpty(record.data.failed_list)) {
                        hasFailedList = '<div style="height: 24px; width: 24px;" class="sync-status has-failed-list"></div>'
                    }
                    result = '<div style="height: 24px; width: 24px;" class="sync-status ' + value + '">' + hasFailedList + '</div>';
                    return result;
                }
            }],
            listeners: {
                select: function(model, record, index) {
                    panel.lastSelected = index;
                    panel.checkButtons();
                },
                itemdblclick: function() {
                    panel.handleOpen();
                }
            }
        });

        return panel.gridTaskList;
    },

    getGridStore: function(connectionId) {
        var panel = this,
            urlOpts = {
                act: 'list_task_status',
                connection_id: connectionId
            };

        var store = Ext.create('Ext.data.Store', {
            fields: [
                'task_id', 'status', 'failed_list', 'task_setting', {
                    name: 'local_path',
                    mapping: 'task_setting.local_root_path'
                }, {
                    name: 'remote_path',
                    mapping: 'task_setting.remote_root_path'
                }, {
                    name: 'type',
                    mapping: 'task_setting.sync_type'
                }
            ],
            proxy: {
                type: 'ajax',
                url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, urlOpts),
                timeout: 60000,
                reader: {
                    type: 'json',
                    root: 'list'
                },
                listeners: {
                    exception: function(proxy, response, operation, opts) {
                        AS.ARC.util.showMsgWindow({
                            5006: _S('ERROR', 'UNABLE_APPLY_SETTINGS')
                        }, Ext.JSON.decode(response.responseText), panel.tabPanel.fn.win);
                    }
                }
            },
            listeners: {
                beforeload: function(store, operation) {},
                load: function(store) {
                    if (!Ext.isEmpty(panel.lastSelected)) {
                        panel.getGridTaskList().getSelectionModel().select(panel.lastSelected);
                    }
                    panel.getGridTaskList().el.unmask();
                    panel.checkButtons();
                }
            }
        });

        return store;
    },

    checkButtons: function() {
        var panel = this,
            item = panel.getSelectedItem();
        
        panel.down('#open').setDisabled(Ext.isEmpty(item));
        panel.down('#edit').setDisabled(Ext.isEmpty(item));
        panel.down('#delete').setDisabled(Ext.isEmpty(item));
        panel.down('#failed-list').setDisabled(
            Ext.isEmpty(item) ||
            Ext.isEmpty(item.data.failed_list)
        );
    },

    handleAdd: function() {
        var panel = this;
        panel.wizard = Ext.create('AS.ARC.apps.datasynccenter.syncListToolbar.wizard', {
            fn: panel.tabPanel.fn,
            parentWin: panel.tabPanel.fn.win,
            title: _S('DATASYNC_CENTER', 'CREATE_NEW_TASK'),
            type: 'create_task',
            serviceType: panel.tabPanel.fn.syncList.getData().service_type,
            connectionName: panel.tabPanel.fn.syncList.getData().connection_name,
            connectionId: panel.tabPanel.fn.syncList.getConnectionId()
        });

        panel.wizard.show();
    },

    handleOpen: function() {
        var panel = this,
            item = panel.getSelectedItem();
        if (Ext.isEmpty(item)) {
            return false;
        }

        AS.ARC.core.openApp('app-fileExplorer', item.data.local_path);
    },

    handleEdit: function() {
        var panel = this;

        var panelFileAndFolderOptions = Ext.create('AS.ARC.panelTop20', {
            title: _S('DATASYNC_CENTER', 'FILE_AND_FOLDER_OPTIONS'),
            itemId: 'file-and-folder-options',
            defaults: {
                labelWidth: 200,
            },
            items: [{
                xtype: 'combo',
                itemId: 'sync-type',
                fieldLabel: _S('DATASYNC_CENTER', 'SYNCHRONIZATION_TYPE'),
                width: 500,
                store: new Ext.data.Store({
                    fields: ['display', 'value'],
                    data: [{
                        'display': _S('DATASYNC_CENTER', 'UPLOAD_LOCAL_MODIFICATIONS'),
                        'value': 'upload'
                    }, {
                        'display': _S('DATASYNC_CENTER', 'TWO_WAY'),
                        'value': 'two_way'
                    }, {
                        'display': _S('DATASYNC_CENTER', 'DOWNLOAD_REMOTE_MODIFICATIONS'),
                        'value': 'download'
                    }]
                }),
                displayField: 'display',
                valueField: 'value',
                queryMode: 'local',
                editable: false,
                lastQuery: '',
                listeners: {
                    change: function(me, newValue, oldValue) {
                        if (!oldValue) { return false; }
                        me.next().type = newValue;
                        me.next().refreshTree('edit');

                        var serviceType = panel.tabPanel.fn.syncList.getData().service_type;

                        panelFileAndFolderOptions.down('#remain-local-files').setVisible(
                            newValue === 'download'
                        );
                        panelFileAndFolderOptions.down('#remain-remote-files').setVisible(
                            newValue === 'upload'
                        );
                        panelFileAndFolderOptions.down('#convert-google-office-files').setVisible(
                            serviceType === 'googledrive' && newValue !== 'upload'
                        );
                        panelFileAndFolderOptions.down('#force-sync-malicious-files').setVisible(
                            serviceType === 'googledrive' && newValue !== 'upload'
                        );
                    },
                    beforeselect: function(me) {
                        if (me.next().el.isMasked()) {
                            return false;
                        }
                    }
                }
            }, {
                xtype: 'syncTree',
                itemId: 'sync-tree',
                treeHeight: 180,
                markTitle: true
            }, {
                xtype: 'checkbox',
                itemId: 'remain-local-files',
                boxLabel: _S('DATASYNC_CENTER', 'DELETE_REMOTE_FILES_NOT_DELETE_LOCAL_FILES'),
                hidden: true
            }, {
                xtype: 'checkbox',
                itemId: 'remain-remote-files',
                boxLabel: _S('DATASYNC_CENTER', 'DELETE_LOCAL_FILES_NOT_DELETE_REMOTE_FILES'),
                hidden: true
            }, {
                xtype: 'checkbox',
                itemId: 'convert-google-office-files',
                boxLabel: _S('DATASYNC_CENTER', 'DOWNLOAD_AND_CONVERT_GOOGLE_OFFICE_FILES'),
                hidden: true
            }, {
                xtype: 'checkbox',
                itemId: 'force-sync-malicious-files',
                cls: 'displayfield',
                boxLabel: _S('DATASYNC_CENTER', 'FORCE_SUSPECTED_MALICIOUS_FILES'),
                hidden: true
            }]
        });

        var panelFilter = Ext.create('AS.ARC.panelTop20', {
            title: _S('DATASYNC_CENTER', 'FILE_FILTER'),
            itemId: 'filter',
            items: [{
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'FILTER_RULES')
            }, {
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'FILE_SIZE')
            }, {
                xtype: 'asFlexbox',
                style: AS.layout.getIndentCss(1),
                defaults: {
                    labelWidth: AS.ARC.config.layout.labelWidthMedium,
                    msgTarget: AS.ARC.config.msgTarget,
                    allowBlank: false,
                    blankText: _S('MESSAGE', 'HF00011')
                },
                items: [{
                    xtype: 'numberfield',
                    itemId: 'size-limit',
                    labelStyle: 'min-width: 240',
                    minValue: 0,
                    maxValue: 10239,
                    maxLength: 6,
                    enforceMaxLength: true,
                    allowDecimals: false,
                    allowBlank: false,
                    fieldLabel: _S('DATASYNC_CENTER', 'DO_NOT_SYNC_IF'),
                    value: 0,
                    width: 400
                }, {
                    xtype: 'displayfield',
                    style: 'margin-left: 10px;',
                    value: _UN_TS('MB'),
                }]
            }, {
                xtype: 'displayfield',
                fieldStyle: 'font-weight: bold',
                value: _S('DATASYNC_CENTER', 'BY_NAME_FILE_EXTENSION')
            }, {
                xtype: 'displayfield',
                style: AS.layout.getIndentCss(1),
                cls: 'displayfield',
                labelCls: 'displayfield',
                fieldLabel: _S('DATASYNC_CENTER', 'FILTER_BY_NAME_OR_TYPE')
            }, {
                xtype: 'fileextensionfilter'
            }]
        });

        var doApply = function() {
            win.el.mask(_S('COMMON', 'APPLYING'));
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'set_task_setting' }),
                params: {
                    connection_id: panel.tabPanel.fn.syncList.getConnectionId(),
                    task_setting: JSON.stringify(win.getTaskSetting())
                },
                method: 'get',
                success: function() {
                    win.close();
                    panel.loadData('set-task-setting');
                },
                failure: function(json) {
                    AS.ARC.util.showMsgWindow({}, json, win);
                }
            });
        };

        var buttonApply = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'APPLY'),
            disabled: true,
            handler: function() {
                if (Ext.isEmpty(win.down('syncTree').tree.getChecked())) {
                    var winConfirm = new AS.ARC.msgWindow({
                        parentWin: win,
                        title: _S('COMMON', 'WARNING'),
                        width: 400,
                        height: 180,
                        iconType: 'warning',
                        asItems: [{
                            xtype: 'displayfield',
                            value: _S('DATASYNC_CENTER', 'PLEASE_SELECT_AT_LEAST_ONE')
                        }],
                        fbar: [{
                            text: _S('COMMON', 'OK'),
                            handler: function() {
                                winConfirm.close();
                            },
                            scope: this
                        }]
                    });
            
                    winConfirm.show();
                } else {
                    doApply();
                }
            },
            scope: this
        });

        var tabPanel = Ext.create('AS.ARC.formTabPanel', {
            buttonApply: buttonApply,
            cls: 'datasync-center-tab',
            items: [
                panelFileAndFolderOptions,
                panelFilter,
            ],
            deferredRender: false
        });

        var win = Ext.create('AS.ARC.subWindow', {
            title: _S('DATASYNC_CENTER', 'OPTIONS'),
            parentWin: panel.tabPanel.fn.win,
            width: 700,
            height: 600,
            layout: 'fit',
            items: [tabPanel],
            fbar: {
                items: [
                    buttonApply, {
                    text: _S('COMMON', 'CANCEL'),
                    scale: AS.ARC.config.defaultApplyButtonSize,
                    handler: function() {
                        win.close();
                    },
                    scope: this
                }]
            },
            updateTree: function() {
                var data = this.fetchedData,
                    tree = panelFileAndFolderOptions.down('syncTree');

                tree.setRemotePath({
                    getRemoteId: function() { return data.remote_root; },
                    getValue: function() { return data.remote_root_path; }
                });
                tree.setLocalPath({
                    getValue: function() { return data.local_root_path; }
                });
                tree.type = data.sync_type;
                tree.parentWin = win;
                tree.connectionId = panel.tabPanel.fn.syncList.getConnectionId();
                tree.setIncludePath(data.filter.include_path);
                tree.setExcluePath(data.filter.exclude_path);
                tree.refreshTree('edit');
            },
            updateFields: function() {
                var data = this.fetchedData,
                    serviceType = panel.tabPanel.fn.syncList.getData().service_type;
                panelFileAndFolderOptions.down('#sync-type').setValue(data.sync_type);
                panelFileAndFolderOptions.down('#remain-local-files').setVisible(
                    data.sync_type === 'download'
                ).setValue(!data.delete_local);
                panelFileAndFolderOptions.down('#remain-remote-files').setVisible(
                    data.sync_type === 'upload'
                ).setValue(!data.delete_remote);
                panelFileAndFolderOptions.down('#convert-google-office-files').setVisible(
                    serviceType === 'googledrive' && data.sync_type !== 'upload'
                ).setValue(data.trans_gdoc);
                panelFileAndFolderOptions.down('#force-sync-malicious-files').setVisible(
                    serviceType === 'googledrive' && data.sync_type !== 'upload'
                ).setValue(data.sync_abuse);
            },
            updateFilter: function() {
                var data = this.fetchedData;
                panelFilter.down('#size-limit').setValue(data.size_limit);
                panelFilter.down('fileextensionfilter').setCustomizedList(data.filter.user_define);
                panelFilter.down('fileextensionfilter').setCheckedList(data.filter.name);
            },
            getTaskSetting: function() {
                var list = panelFilter.down('fileextensionfilter').getList(),
                    tree = panelFileAndFolderOptions.down('syncTree'),
                    remainRemote = panelFileAndFolderOptions.down('#remain-remote-files'),
                    remainLocal = panelFileAndFolderOptions.down('#remain-local-files'),
                    taskSetting = {
                        "task_id": win.fetchedData.task_id,
                        "local_root": win.fetchedData.local_root,
                        "remote_root": win.fetchedData.remote_root,
                        "local_root_path": win.fetchedData.local_root_path,
                        "remote_root_path": win.fetchedData.remote_root_path,
                        "delete_remote": remainRemote.isHidden() ? true : !remainRemote.getValue(),
                        "delete_local": remainLocal.isHidden() ? true : !remainLocal.getValue(),
                        "sync_abuse": panelFileAndFolderOptions.down('#force-sync-malicious-files').getValue(),
                        "trans_gdoc": panelFileAndFolderOptions.down('#convert-google-office-files').getValue(),
                        "size_limit": panelFilter.down('#size-limit').getValue(),
                        "sync_type": panelFileAndFolderOptions.down('#sync-type').getValue(),
                        "filter": {
                            "name": list.checkedList,
                            "include_path": win.fetchedData.filter.include_path,
                            "exclude_path": win.fetchedData.filter.exclude_path,
                            "user_define": list.customizedList
                        }
                    };

                if (tree.tree.getRootNode().hasChildNodes() && !tree.tree.el.isMasked()) {
                    Ext.apply(taskSetting.filter, tree.getPath());
                }

                return taskSetting;
            },
            listeners: {
                afterrender: function() {
                    win.el.mask(_S('COMMON', 'LOADING'));
                    var serviceType = panel.tabPanel.fn.syncList.getData().service_type,
                        getSyncTypeData = function(serviceName) {
                            if (serviceType === 'ezsync') {
                                return [{
                                    'display': _S('DATASYNC_CENTER', 'UPLOAD_LOCAL_MODIFICATIONS'),
                                    'value': 'upload'
                                }, {
                                    'display': _S('DATASYNC_CENTER', 'TWO_WAY'),
                                    'value': 'two_way'
                                }, {
                                    'display': _S('DATASYNC_CENTER', 'DOWNLOAD_REMOTE_MODIFICATIONS'),
                                    'value': 'download'
                                }];
                            } else {
                                return [{
                                    'display': Ext.String.format(_S('DATASYNC_CENTER', 'NAS_TO_REMOTE'), serviceName),
                                    'value': 'upload'
                                }, {
                                    'display': _S('DATASYNC_CENTER', 'TWO_WAY'),
                                    'value': 'two_way'
                                }, {
                                    'display': Ext.String.format(_S('DATASYNC_CENTER', 'REMOTE_TO_NAS'), serviceName),
                                    'value': 'download'
                                }];
                            }
                        },
                        SERVICE_LIST = [
                            ['ezsync', _S('DATASYNC_CENTER', 'ASUSTOR_NAS')],
                            ['baidu', _S('DATASYNC_CENTER', 'BAIDU')],
                            ['dropbox', _S('DATASYNC_CENTER', 'DROPBOX')],
                            ['googledrive', _S('DATASYNC_CENTER', 'GOOGLE_DRIVE')],
                            ['onedrive', _S('DATASYNC_CENTER', 'ONE_DRIVE')],
                            ['yandex', _S('DATASYNC_CENTER', 'YANDEX')]
                        ],
                        serviceName = SERVICE_LIST.find(function(service) {
                            return service.first() === serviceType;
                        }).last();

                    panelFileAndFolderOptions.down('#sync-type').store.loadData(getSyncTypeData(serviceName));
                    panelFileAndFolderOptions.down('#force-sync-malicious-files').boxLabelEl.update(
                        Ext.String.format(_S('DATASYNC_CENTER', 'FORCE_SUSPECTED_MALICIOUS_FILES'), serviceName)
                    );
                    AS.ARC.ajax({
                        url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'get_task_setting' }),
                        params: {
                            connection_id: panel.tabPanel.fn.syncList.getConnectionId(),
                            task_id: panel.getSelectedItem().data.task_id
                        },
                        method: 'get',
                        success: function(json) {
                            win.fetchedData = json.task_setting;
                            win.updateTree();
                            win.updateFields();
                            win.updateFilter();
                            
                            win.el.unmask();
                        },
                        failure: function(json) {
                            AS.ARC.util.showMsgWindow({}, json, win);
                        }
                    });
                }
            }
        });

        win.show();
    },

    handleReselectLostFolder: function() {
        var panel = this,
            STATUS = panel.tabPanel.statusPanel.STATUS,
            isLostLocalFolder = (panel.getSelectedItem().data.status === STATUS.LOST_LOCAL_FOLDER);

        var buttonApply = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'APPLY'),
            disabled: true,
            handler: function() {
                win.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'set_task_setting' }),
                    params: {
                        connection_id: panel.tabPanel.fn.syncList.getConnectionId(),
                        task_setting: JSON.stringify(win.getTaskSetting())
                    },
                    method: 'get',
                    success: function() {
                        win.close();
                        panel.loadData('set-task-setting');
                    },
                    failure: function(json) {
                        AS.ARC.util.showMsgWindow({}, json, win);
                    }
                });
            },
            scope: this
        });

        var localPath = Ext.create('Ext.form.Text', {
            itemId: 'path',
            labelWidth: 200,
            width: 500,
            readOnly: true,
            style: 'margin-right: 12px;',
            fieldLabel: _S('DATASYNC_CENTER', 'LOCAL_PATH'),
            allowBlank: false,
            msgTarget: AS.ARC.config.msgTarget,
            blankText: _S('MESSAGE', 'HF00011'),
            validator: function(value) {
                var isPathUsed = false;
                if (Ext.isEmpty(value)) return true;
                panel.getGridTaskList().store.each(function(record) {
                    if (panel.getSelectedItem().data.task_id === record.data.task_id) return true;
                    if (record.data.local_path.indexOf(value) === 0 || value.indexOf(record.data.local_path) === 0) {
                        isPathUsed = true;
                        return false;
                    }
                });

                if (isPathUsed) {
                    return _S('DATASYNC_CENTER', 'PATH_USED_WARNING');
                } else if (value.split('/').some(function(folder) {
                    return folder === '#Recycle' || folder === '.datasync-center';
                })) {
                    return _S('DATASYNC_CENTER', 'PATH_UNABLE_WARNING');
                } else {
                    return true;
                }
            }
        });

        var panelLocalFolder = Ext.create('AS.ARC.panelTop20', {
            buttonApply: buttonApply,
            itemId: 'filter',
            items: [{
                xtype: 'asFlexbox',
                items: [localPath, {
                    xtype: 'button',
                    cls: 'image-btn',
                    iconCls: 'local-btn',
                    handler: function() {
                        var fs = Ext.create('AS.ARC.apps.datasynccenter.localTreeSelector', {
                            cls: 'app-datasync-center local-selector',
                            parentWin: win,
                            objectPath: localPath,
                            hasRefresh: true,
                            cancelAjax: true,
                            isShowHome: true,
                            hideUserName: true
                        });
                        fs.win.show();
                    }
                }]
            }]
        });

        var remotePath = Ext.create('Ext.form.Text', {
            itemId: 'path',
            labelWidth: 200,
            width: 500,
            readOnly: true,
            style: 'margin-right: 12px;',
            fieldLabel: _S('DATASYNC_CENTER', 'REMOTE_PATH'),
            allowBlank: false,
            msgTarget: AS.ARC.config.msgTarget,
            blankText: _S('MESSAGE', 'HF00011'),
            getRemoteId: function() {
                return this.remoteId || '';
            },
            validator: function(value) {
                var isPathUsed = false;
                if (Ext.isEmpty(value)) return true;
                panel.getGridTaskList().store.each(function(record) {
                    if (panel.getSelectedItem().data.task_id === record.data.task_id) return true;
                    if (record.data.remote_path.indexOf(value) === 0 || value.indexOf(record.data.remote_path) === 0) {
                        isPathUsed = true;
                        return false;
                    }
                });

                return isPathUsed ? _S('DATASYNC_CENTER', 'PATH_USED_WARNING') : true;
            }
        });

        var panelRemoteFolder = Ext.create('AS.ARC.panelTop20', {
            buttonApply: buttonApply,
            itemId: 'filter',
            items: [{
                xtype: 'asFlexbox',
                items: [remotePath, {
                    xtype: 'button',
                    cls: 'image-btn',
                    iconCls: 'remote-btn',
                    handler: function() {
                        var fs = Ext.create('AS.ARC.apps.datasynccenter.remoteTreeSelector', {
                            cls: 'app-datasync-center remote-selector',
                            parentWin: win,
                            objectPath: remotePath,
                            connectionId: panel.tabPanel.fn.syncList.getConnectionId()
                        });
                        fs.win.show();
                    }
                }]
            }]
        });

        var items = [isLostLocalFolder ? panelLocalFolder : panelRemoteFolder];

        var win = Ext.create('AS.ARC.subWindow', {
            title: _S('DATASYNC_CENTER', 'NEW_SYNC_FOLDER'),
            parentWin: panel.tabPanel.fn.win,
            cls: 'app-datasync-center-wizard',
            width: 600,
            height: 180,
            layout: 'fit',
            items: items,
            fbar: {
                items: [
                    buttonApply, {
                    text: _S('COMMON', 'CANCEL'),
                    scale: AS.ARC.config.defaultApplyButtonSize,
                    handler: function() {
                        win.close();
                    },
                    scope: this
                }]
            },
            getTaskSetting: function() {
                if (isLostLocalFolder) {
                    Ext.apply(win.fetchedData, {
                        local_root: localPath.getValue(),
                        local_root_path: localPath.getValue()
                    });
                } else {
                    Ext.apply(win.fetchedData, {
                        remote_root: remotePath.getRemoteId(),
                        remote_root_path: remotePath.getValue()
                    });
                }
                return win.fetchedData;
            },
            listeners: {
                afterrender: function() {
                    win.el.mask(_S('COMMON', 'LOADING'));
                    AS.ARC.ajax({
                        url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'get_task_setting' }),
                        params: {
                            connection_id: panel.tabPanel.fn.syncList.getConnectionId(),
                            task_id: panel.getSelectedItem().data.task_id
                        },
                        method: 'get',
                        success: function(json) {
                            win.fetchedData = json.task_setting;
                            win.el.unmask();
                        },
                        failure: function(json) {
                            AS.ARC.util.showMsgWindow({}, json, win);
                        }
                    });
                }
            }
        });

        win.show();
    },

    handleDelete: function() {
        var panel = this,
            tabPanel = this.tabPanel,
            isLastTask = panel.getGridTaskList().store.getCount() === 1 ? true : false;

        var buttonOk = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'OK'),
            handler: function() {
                panel.lastSelected = null;
                winConfirm.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(tabPanel.fn.apiUrl, { act: 'remove_task' }),
                    params: {
                        connection_id: tabPanel.fn.syncList.getConnectionId(),
                        task_id: panel.getSelectedItem().data.task_id
                    },
                    method: 'get',
                    success: function() {
                        if (isLastTask) {
                            tabPanel.fn.syncList.loadSyncList();
                        } else {
                            panel.loadData('remove-task');
                        }
                        winConfirm.close();
                    },
                    failure: function(json) {
                        winConfirm.close();
                        AS.ARC.util.showMsgWindow({}, json, tabPanel.fn.win);
                    }
                });
            },
            scope: this
        });

        var buttonCancel = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'CANCEL'),
            handler: function() {
                winConfirm.close();
            },
            scope: this
        });

        var winConfirm = new AS.ARC.msgWindow({
            parentWin: tabPanel.fn.win,
            title: _S('COMMON', 'CONFIRMATION'),
            width: 400,
            height: 180,
            iconType: 'question',
            asItems: [{
                xtype: 'displayfield',
                value: isLastTask ? _S('DATASYNC_CENTER', 'LAST_TASK_REMOVE_CONFIRM') : _S('DATASYNC_CENTER', 'TASK_REMOVE_CONFIRM')
            }],
            fbar: [
                buttonOk,
                buttonCancel
            ]
        });

        winConfirm.show();
    },

    handleFailedList: function() {
        var panel = this,
            failedList = panel.tabPanel.statusPanel.down('#error-file-list'),
            item = panel.getSelectedItem();
        if (Ext.isEmpty(item)) return false;
        failedList.setData(item.data.failed_list);
        failedList.openFileList();
    },

    stopPolling: function() {
        var panel = this;
        if (!Ext.isEmpty(panel.ajaxRequest)) {
            Ext.Ajax.abort(panel.ajaxRequest);
        }
        clearTimeout(panel.pollingID);
    },

    startPolling: function() {
        var panel = this;
        panel.stopPolling();
        panel.pollingID = Ext.defer(function() {
            if (panel.tabPanel.fn.win.isDestroyed) return false;
            if (panel.getItemId() === panel.tabPanel.activeTab.getItemId()) {
                panel.ajaxRequest = AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'list_task_status' }),
                    params: {
                        connection_id: panel.tabPanel.fn.syncList.getConnectionId()
                    },
                    method: 'get',
                    success: function(json) {
                        panel.getGridTaskList().getStore().loadRawData(json);
                        panel.startPolling();
                    },
                    failure: Ext.emptyFn
                });
            }
        }, 1000);
    },

    loadData: function(type) {
        var panel = this,
            connectionId = panel.tabPanel.fn.syncList.getConnectionId();
        if (type === 'connection-update') return false;

        panel.stopPolling();
        if (type === 'sync-list-change' || type === 'set-task-setting' || type === 'remove-task' || type === 'tab-change') {
            panel.getGridTaskList().el.mask(_S('COMMON', 'LOADING'));
        }
        panel.getGridTaskList().bindStore(panel.getGridStore(connectionId));
        panel.getGridTaskList().getStore().load(function() {
            panel.startPolling();
        });
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.syncTabPanel.logPanel
 * @extends Ext.tab.Panel
 */
Ext.define('AS.ARC.apps.datasynccenter.syncTabPanel.logPanel', {
    extend: 'Ext.form.Panel',
    border: false,
    cls: 'as-page-panel',
    itemId: 'log-panel',
    layout: {
        type: 'vbox',
        align: 'stretch'
    },

    initComponent: function() {
        this.items = [this.getItems()];
        this.init();
        this.callParent(arguments);
    },

    init: function() {
        this.title = _S('DATASYNC_CENTER', 'LOGS');
    },

    getItems: function() {
        var panel = this;
        return [{
            xtype: 'asPanelToolbarTop',
            layout: 'hbox',
            items: [{
                xtype: 'button',
                text: _S('COMMON', 'CLEAR'),
                scope: panel,
                handler: panel.handleClear,
                minWidth: 75
            }, {
                xtype: 'tbfill'
            }, panel.getSearchField(), panel.getBtnSearch()]
        }, panel.getGridLogList()];
    },

    createSearchPanel: function() {
        var panel = this;
        var keyword = Ext.create('Ext.form.field.Text', {
            fieldLabel: _S('COMMON', 'KEYWORD'),
            name: 'search_key',
            itemId: 'keyword'
        });

        var status = Ext.create('Ext.form.field.ComboBox', {
            fieldLabel: _S('COMMON', 'STATUS'),
            name: 'status',
            itemId: 'status',
            displayField: 'display',
            valueField: 'value',
            editable: false,
            value: '',
            store: new Ext.data.ArrayStore({
                fields: ['display', 'value'],
                data: [
                    [_S('COMMON', 'ALL'), ''],
                    [_S('COMMON', 'SUCCESS'), 'done'],
                    [_S('COMMON', 'FAILURE'), 'failed']
                ]
            })
        });

        var event = Ext.create('Ext.form.field.ComboBox', {
            fieldLabel: _S('SYSTEMINFORMATION', 'EVENT'),
            name: 'event',
            itemId: 'event',
            displayField: 'display',
            valueField: 'value',
            editable: false,
            value: '',
            store: new Ext.data.ArrayStore({
                fields: ['display', 'value'],
                data: [
                    [_S('COMMON', 'ALL'), ''],
                    [_S('COMMON', 'UPLOAD'), 'upload'],
                    [_S('COMMON', 'DOWNLOAD'), 'download'],
                    [_S('FILE_EXPLORER', 'MENU_MOVE'), 'move'],
                    [_S('COMMON', 'DELETE'), 'delete'],
                    [_S('COMMON', 'RENAME'), 'rename']
                ]
            })
        });

        var type = Ext.create('Ext.form.field.ComboBox', {
            fieldLabel: _S('COMMON', 'TYPE'),
            name: 'file_type',
            itemId: 'type',
            displayField: 'display',
            valueField: 'value',
            editable: false,
            value: '',
            store: new Ext.data.ArrayStore({
                fields: ['display', 'value'],
                data: [
                    [_S('FILE_EXPLORER', 'DATA_ANY'), ''],
                    [_S('FILE_EXPLORER', 'DATA_BY_FILE'), 'file'],
                    [_S('FILE_EXPLORER', 'DATA_BY_FOLDER'), 'folder']
                ]
            })
        });

        var from = Ext.create('Ext.form.field.Date', {
            emptyText: _S('SYSTEMINFORMATION', 'FROM'),
            name: 'time_after',
            itemId: 'from',
            editable: false,
            margin: '0 10 0 0',
            width: 165,
            format: AS.ARC.datas.dateFormat[AS.ARC.config.dateFormat.DateFormat].fieldFormat,
            listeners: {
                change: function(dateField, value) {
                    to.setMinValue(value);
                }
            }
        });

        var to = Ext.create('Ext.form.field.Date', {
            emptyText: _S('SYSTEMINFORMATION', 'UNTIL'),
            name: 'time_before',
            itemId: 'to',
            editable: false,
            width: 165,
            format: AS.ARC.datas.dateFormat[AS.ARC.config.dateFormat.DateFormat].fieldFormat,
            listeners: {
                change: function(dateField, value) {
                    from.setMaxValue(value);
                }
            }
        });

        panel.searchPanel = Ext.create('Ext.form.Panel', {
            cls: 'as-function-search-wrap',
            floating: true,
            width: 400,
            height: 328,
            hidden: true,
            padding: '25 0 15 30',
            border: 0,
            fieldDefaults: {
                labelWidth: AS.ARC.config.layout.defaultLabelWidth,
                width: 340
            },
            items: [keyword, status, event, type, {
                xtype: 'displayfield',
                margin: '20 0 0 0',
                fieldLabel: _S('COMMON', 'DATE')
            }, {
                xtype: 'asFlexbox',
                margin: '10 0 0 0',
                items: [from, to]
            }],
            fbar: [{
                text: _S('COMMON', 'SEARCH'),
                handler: function() {
                    panel.searchPanel.hide();
                    panel.getSearchField().setValue('');
                    panel.getGridLogList().store.proxy.extraParams = panel.getSearchParams();
                    panel.getGridLogList().store.loadPage(1);
                },
                scope: panel
            }, {
                text: _S('FILE_EXPLORER', 'INFO_RESET'),
                handler: function(button) {
                    button.up('form').getForm().reset();
                }
            }],
            renderTo: panel.tabPanel.el.dom,
            listeners: {
                afterrender: function(me) {
                    var relocate = function() {
                        var top = panel.btnSearch.getHeight() + 45 + 10,
                            left = panel.tabPanel.getWidth() - me.width - 20;

                        me.setPosition(left, top);
                    };

                    relocate();
                    panel.tabPanel.on('resize', relocate);

                    me.el.on('click', function(event) {
                        event.stopPropagation();
                    });
                }
            }
        });
    },

    getSearchField: function() {
        var panel = this;
        if (!Ext.isEmpty(panel.searchField))
            return panel.searchField;

        panel.searchField = Ext.create('AS.ARC.apps.datasynccenter.searchField', {
            itemId: 'search',
            enableKeyEvents: true,
            checkChangeBuffer: AS.ARC.config.keyBuffer,
            listeners: {
                change: function(textField, value) {
                    Ext.apply(panel.getGridLogList().store.proxy.extraParams, { keyword: value });
                    panel.getGridLogList().store.loadPage(1);
                }
            }
        });

        return panel.searchField;
    },

    getSearchParams: function() {
        var panel = this,
            formValues = panel.searchPanel.getForm().getValues(),
            searchParams = {};
        
        if (formValues.search_key) {
            Ext.apply(searchParams, { "keyword": formValues.search_key });
        }

        if (formValues.status) {
            Ext.apply(searchParams, { "status": formValues.status });
        }

        if (formValues.event) {
            Ext.apply(searchParams, { "event_type": formValues.event });
        }

        if (formValues.file_type) {
            Ext.apply(searchParams, { "file_type": formValues.file_type });
        }

        if (formValues.time_before) {
            Ext.apply(searchParams, { "time_before": (new Date(formValues.time_before + ' 23:59:59')) / 1000 });
        }

        if (formValues.time_after) {
            Ext.apply(searchParams, { "time_after": (new Date(formValues.time_after)) / 1000 });
        }

        if (Object.keys(searchParams).length === 0) {
            return null;
        }

        return searchParams;
    },

    getBtnSearch: function() {
        var panel = this;
        if (!Ext.isEmpty(panel.btnSearch))
            return panel.btnSearch;

        panel.btnSearch = Ext.create('Ext.button.Button', {
            iconCls: 'image-btn',
            cls: 'advanced-search',
            scope: this,
            handler: function(button, event) {
                if (!Ext.isEmpty(panel.searchPanel)) {
                    var keyWord = panel.getSearchField().getValue();
                    if (!Ext.isEmpty(keyWord)) {
                        panel.searchPanel.down('#keyword').setValue(keyWord);
                    }

                    panel.searchPanel.setVisible(!panel.searchPanel.isVisible());
                    event.stopPropagation();
                }
            }
        });

        return panel.btnSearch;
    },

    getGridLogList: function() {
        var panel = this;
        if (!Ext.isEmpty(panel.gridLogList))
            return panel.gridLogList;

        panel.gridLogList = Ext.create('Ext.grid.GridPanel', {
            itemId: 'grid-log-list',
            margin: '0 0 10 0',
            flex: 1,
            viewConfig: {
                loadMask: false
            },
            defaults: {
                draggable: false
            },
            columns: [{
                text: _S('COMMON', 'STATUS'),
                dataIndex: 'status',
                flex: 1,
                renderer: function(value) {
                    var icon = '';
                    if (value === 'done') {
                        icon = '<div class="status-icon-16 ok"></div>';
                    } else if (value === 'failed') {
                        icon = '<div class="status-icon-16 error"></div>';
                    }
                    return icon;
                }
            }, {
                text: _S('COMMON', 'TIME'),
                dataIndex: 'time',
                flex: 3,
                renderer: function(value) {
                    var timeOffset = AS.ARC.config.dateFormat.LocalTimeGmtOffset,
                        dateFormat = AS.ARC.config.dateFormat.DateFormat,
                        timeFormat = AS.ARC.config.dateFormat.TimeFormat,
                        result = AS.ARC.util.toSystemTime(dateFormat, timeFormat, value + timeOffset * 60);

                    return result;
                }
            }, {
                text: _S('SYSTEMINFORMATION', 'EVENT'),
                dataIndex: 'event_type',
                flex: 1.5,
                renderer: function(value) {
                    var event = '',
                        EVENT = {
                            "upload": _S('COMMON', 'UPLOAD'),
                            "download": _S('COMMON', 'DOWNLOAD'),
                            "delete": _S('COMMON', 'DELETE'),
                            "move": _S('FILE_EXPLORER', 'MENU_MOVE'),
                            "rename": _S('COMMON', 'RENAME'),
                        };
                    
                    return EVENT[value] || '';
                }
            }, {
                text: _S('COMMON', 'TYPE'),
                dataIndex: 'file_type',
                flex: 1.5,
                renderer: function(value) {
                    var type = '';
                    if (value === 'file') {
                        type = _S('FILE_EXPLORER', 'FILE_PERMISSIONS');
                    } else if (value === 'folder') {
                        type = _S('COMMON', 'FOLDER');
                    }
                    return type;
                }
            }, {
                text: _S('COMMON', 'NAME'),
                dataIndex: 'info',
                flex: 4,
                renderer: function(value, metadata) {
                    var name = value.split('/').pop();
                    metadata.tdAttr = 'data-qtip="' + Ext.htmlEncode(Ext.htmlEncode(value)) + '"';
                    return Ext.htmlEncode(name);
                }
            }],
            bbar: Ext.create('AS.ARC.pagingToolbar')
        });

        return panel.gridLogList;
    },

    getGridStore: function(connectionId) {
        var panel = this,
            urlOpts = {
                act: 'list_log',
                connection_id: connectionId
            };

        var store = Ext.create('Ext.data.Store', {
            fields: [
                'log_id', 'status', 'time', 'event_type', 'file_type', 'info'
            ],
            proxy: {
                type: 'ajax',
                url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, urlOpts),
                timeout: 60000,
                startParam: 'offset',
                reader: {
                    type: 'json',
                    root: 'list'
                },
                listeners: {
                    exception: function(proxy, response, operation, opts) {
                        AS.ARC.util.showMsgWindow({
                            5006: _S('ERROR', 'UNABLE_APPLY_SETTINGS')
                        }, Ext.JSON.decode(response.responseText), panel.tabPanel.fn.win);
                    }
                }
            },
            listeners: {
                beforeload: function(store, operation) {},
                load: function(store) {
                    panel.getGridLogList().el.unmask();
                }
            }
        });

        return store;
    },

    getDummyData: function() {
        var data = [];

        var status = ['done', 'failed'];
        var event = ['upload', 'download'];
        var fileType = ['file', 'folder'];
        var date = new Date();
        date.setDate(date.getDate() - 7);

        for (var i = 0; i < 10 ; i++) {
            random = Math.floor(Math.random() * status.length);
            data.push({
                "log_id": 1,
                "status": status[random],
                "time": Math.floor(Math.random() * (Ext.Date.now() - date.getTime()) + date.getTime()) / 1000 ,
                "event_type": event[random],
                "file_type": fileType[random],
                "info": "Hello, world!"
            });
        }
        return data;
    },

    handleClear: function() {
        var panel = this,
            tabPanel = panel.tabPanel;
        var buttonOk = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'OK'),
            handler: function() {
                winConfirm.el.mask(_S('COMMON', 'APPLYING'));
                AS.ARC.ajax({
                    url: AS.ARC.util.getApiUrlWithSid(panel.tabPanel.fn.apiUrl, { act: 'clean_log' }),
                    params: {
                        connection_id: panel.tabPanel.fn.syncList.getConnectionId()
                    },
                    method: 'get',
                    success: function() {
                        panel.getGridLogList().el.mask(_S('COMMON', 'LOADING'));
                        panel.getGridLogList().getStore().load();
                        winConfirm.close();
                    },
                    failure: function(json) {
                        winConfirm.close();
                        AS.ARC.util.showMsgWindow({}, json, tabPanel.fn.win);
                    }
                });
            },
            scope: this
        });

        var buttonCancel = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'CANCEL'),
            handler: function() {
                winConfirm.close();
            },
            scope: this
        });

        var winConfirm = new AS.ARC.msgWindow({
            parentWin: tabPanel.fn.win,
            title: _S('COMMON', 'CONFIRMATION'),
            width: 400,
            height: 180,
            iconType: 'question',
            asItems: [{
                xtype: 'displayfield',
                value: _S('SYSTEMINFORMATION', 'NOTE_REMOVE_LOG')
            }],
            fbar: [
                buttonOk,
                buttonCancel
            ]
        });

        winConfirm.show();
    },

    loadData: function(type) {
        var panel = this,
            connectionId = panel.tabPanel.fn.syncList.getConnectionId();
        if (type === 'connection-update') return false;

        if (type === 'sync-list-change' || type === 'tab-change') {
            panel.getGridLogList().el.mask(_S('COMMON', 'LOADING'));
        }
        var store = panel.getGridStore(connectionId);
        panel.getGridLogList().bindStore(store);
        panel.getGridLogList().getStore().load();
        panel.getGridLogList().getDockedItems()[0].bindStore(store);
        panel.getGridLogList().getDockedItems()[0].store.load();
    },

    listeners: {
        afterrender: function(panel) {
            if (Ext.isEmpty(panel.searchPanel)) {
                panel.createSearchPanel();
            }

            panel.tabPanel.fn.win.el.on('click', function(e, t) {
                if (!panel.tabPanel.fn.win.el.isMasked()) {
                    panel.searchPanel.setVisible(false);
                }
            });
        }
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.syncTabPanel
 * @extends Ext.tab.Panel
 */
Ext.define('AS.ARC.apps.datasynccenter.syncTabPanel', {
    extend: 'Ext.TabPanel',
    alias: ['widget.syncTabPanel'],
    height: '100%',
    cls: 'as-tab-panel',
    bodyCls: 'no-footer',
    border: false,

    initComponent: function() {
        var tabPanel = this;
        tabPanel.items = [
            tabPanel.getStatusPanel(),
            tabPanel.getTaskListPanel(),
            tabPanel.getLogPanel()
        ];

        tabPanel.callParent(arguments);
    },

    getStatusPanel: function() {
        var tabPanel = this;
        if (!Ext.isEmpty(tabPanel.statusPanel))
            return tabPanel.statusPanel;

        tabPanel.statusPanel = Ext.create('AS.ARC.apps.datasynccenter.syncTabPanel.statusPanel', { tabPanel: tabPanel });
        return tabPanel.statusPanel;
    },

    getTaskListPanel: function() {
        var tabPanel = this;
        if (!Ext.isEmpty(tabPanel.taskListPanel))
            return tabPanel.taskListPanel;

        tabPanel.taskListPanel = Ext.create('AS.ARC.apps.datasynccenter.syncTabPanel.taskListPanel', { tabPanel: tabPanel });
        return tabPanel.taskListPanel;
    },

    getLogPanel: function() {
        var tabPanel = this;
        if (!Ext.isEmpty(tabPanel.logPanel))
            return tabPanel.logPanel;

        tabPanel.logPanel = Ext.create('AS.ARC.apps.datasynccenter.syncTabPanel.logPanel', { tabPanel: tabPanel });
        return tabPanel.logPanel;
    },

    listeners: {
        tabchange: function(panel, newCard, oldCard) {
            newCard.loadData('tab-change');
            Ext.callback(oldCard.stopPolling, oldCard);
        }
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.taskListLink
 * @extends AS.ARC.flexbox
 * @author ChengHsuanLai 2021-08-16
 * TaskList hyperlink component.
 */
Ext.define('AS.ARC.apps.datasynccenter.taskListLink', {
    extend: 'AS.ARC.flexbox',
    alias: ['widget.tasklistlink'],

    cls: 'common-layout-horizontal-3-0',
    hidden: true,

    initComponent: function() {
        var taskListLink = this;
        taskListLink.items = [{
            xtype: 'component',
            autoEl: {
                tag: 'div',
                cls: 'status-icon-16 ' + taskListLink.cls,
                style: 'display: inline-block; margin: 0px 5px 0px 0px;'
            }
        }, {
            xtype: 'displayfield',
            itemId: taskListLink.textItemId,
            value: taskListLink.textValue,
            listeners: {
                afterrender: function(me) {
                    me.el.on('click', function() {
                        taskListLink.click();
                    }, this, { delegate: '.hyperlink-string'});
                }
            }
        }];
        this.callParent(arguments);
    },
})

/**
 * @class AS.ARC.apps.datasynccenter.uploadingTaskListSubWindow
 * @extends AS.ARC.subWindow
 * @author ChengHsuanLai 2021-08-16
 * Uploading Task List Sub Window.
 */
Ext.define('AS.ARC.apps.datasynccenter.uploadingTaskListSubWindow', {
    extend: 'AS.ARC.subWindow',

    itemId: 'uploading-task-list-sub-window',
    width: 700,
    height: 450,
    
    initComponent: function() {
        var win = this;

        win.gridStatusInfoList = Ext.create('Ext.grid.GridPanel', {
            itemId: 'grid-uploading-task-list',
            cls: 'app-datasync-center-tasklist',
            height: 320,
            store: win.getStore([]),
            columns: [{
                text: _S('COMMON', 'NAME'),
                dataIndex: 'path',
                draggable: false,
                flex: 2.5,
                renderer: function(value, cell, record) {
                    var value = Ext.String.htmlEncode(value.split('/').pop());
                    cell.tdAttr = 'data-qtip="' + value + '"';
                    return '<span>' + value + '</span>';
                }
            }, {
                text: _S('UPLOAD', 'PROGRESS'),
                dataIndex: 'progress',
                width: 220,
                renderer: function(value, meta, record) {
                    return AS.ARC.util.renderProgress(Ext.Number.constrain(value * 100, 0, 100), meta, record);
                }
            }, {
                text: _S('COMMON', 'SPEED'),
                dataIndex: 'bandwidth',
                align: 'right',
                renderer: function(value, meta, record) {
                    return AS.ARC.util.convertSizeUnit(value) + '/s';
                }
            }],
            viewConfig: {
                deferEmptyText: true,
                emptyText: (
                    '<div class="devices-empty" style="display: flex; justify-content: center; align-items: center; flex-direction: column; height: 100%;">'+
                        '<div class="x-form-display-field">' +
                            Ext.String.format(_S('DATASYNC_CENTER', 'SYNCHRONIZATION_COMPLETE'), '<a class="hyperlink-string" href="#">', '</a>') +
                        '</div>' +
                    '</div>'
                )
            },
            listeners: {
                afterrender: function(me) {
                    me.el.on('click', function() {
                        win.clickHyperlink();
                    }, this, { delegate: '.hyperlink-string'});
                }
            }
        });

        Ext.apply(win, {
            title: _S('DATASYNC_CENTER', 'TASKS'),
            items: [{
                xtype: 'asPanelTop10',
                items: [win.gridStatusInfoList]
            }],
            fbar: [{
                xtype: 'button',
                text: _S('COMMON', 'CLOSE'),
                handler: function() {
                    win.close();
                }
            }],
        });

        this.callParent(arguments);
    },

    getStore: function(data) {
        return {
            store: 'array',
            fields: ['id', 'path', 'progress', 'bandwidth'],
            data: data
        };
    },

    loadData: function(data) {
        var win = this;
        win.gridStatusInfoList.getStore().loadData(Ext.isEmpty(data) ? [] : data);
    },
})

/**
 * @class AS.ARC.apps.datasynccenter.errorFileList
 * @extends AS.ARC.flexbox
 * @author JerryJiang 2018-11-28
 * If it found files cannot be synchronized, it will show and provide a link to list the files.
 */
Ext.define('AS.ARC.apps.datasynccenter.errorFileList', {
    extend: 'AS.ARC.flexbox',
    alias: ['widget.errorfilelist'],

    API_URL: AS.ARC.util.getUserAppsPath() + 'datasync-center/datasync_center.cgi',

    hidden: true,
    fetchedData: [],

    initComponent: function() {
        var fileList = this;
        fileList.fetchedData = [];
        this.items = [{
            xtype: 'component',
            autoEl: {
                tag: 'div',
                cls: 'status-icon-16 warning',
                style: 'display: inline-block; margin: 0px 5px 0px 0px;'
            }
        }, {
            xtype: 'displayfield',
            itemId: 'link',
            value: Ext.String.format(_S('DATASYNC_CENTER', 'ONE_FILE_CANNOT_BE_SYNCHRONIZED'), '<a class="hyperlink-string" href="#">', '</a>'),
            listeners: {
                afterrender: function(me) {
                    me.el.on('click', function() {
                        fileList.openFileList();
                    }, this, { delegate: '.hyperlink-string' });
                }
            }
        }];
        this.callParent(arguments);
    },

    openFileList: function() {
        var me = this;

        me.gridFileList = Ext.create('Ext.grid.GridPanel', {
            itemId: 'grid-file-list',
            height: 320,
            store: me.getStore([]),
            columns: [{
                text: _S('COMMON', 'NAME'),
                dataIndex: 'path',
                draggable: false,
                flex: 2.5,
                renderer: function(value, cell, record) {
                    cell.tdAttr = 'data-qtip="' + Ext.htmlEncode(Ext.htmlEncode(value)) + '"';
                    if (record.data.type === 'upload') {
                        return '<a href="#" class="hyperlink-string">' + Ext.String.htmlEncode(value.split('/').pop()) + '</a>';
                    } else {
                        return Ext.String.htmlEncode(value.split('/').pop());
                    }
                }
            }, {
                text: _S('COMMON', 'SIZE'),
                dataIndex: 'size',
                draggable: false,
                flex: 1,
                renderer: function(value) {
                    return AS.ARC.util.convertSizeUnit(value);
                }
            }, {
                text: _S('COMMON', 'STATUS'),
                dataIndex: 'type',
                draggable: false,
                flex: 3,
                renderer: function(value, cell, record) {
                    var s = value === 'upload' ? _S('DATASYNC_CENTER', 'UPLOAD_FAILED') : _S('DATASYNC_CENTER', 'DOWNLOAD_FAILED');
                    var error = record.data.error_msg ? Ext.htmlEncode(record.data.error_msg) : _S('DATASYNC_CENTER', 'SYNCHRONIZATION_FAILED');
                    cell.style = 'display: flex';
                    return s + '<div class="status-icon-16 warning" style="margin-left: 5px;" data-errorqtip="' + Ext.htmlEncode(error) + '"></div>';
                }
            }],
            listeners: {
                cellclick: function (view, cell, cellIndex, record, row, rowIndex, e) {
                    if (e && e.target.tagName == 'A') {
                        AS.ARC.core.openApp('app-fileExplorer', record.data.path);
                    }
                }
            }
        });

        var win = Ext.create('AS.ARC.subWindow', {
            title: _S('DATASYNC_CENTER', 'FILE_LIST'),
            width: 700,
            height: 450,
            parentWin: me.fn.win,
            items: [{
                xtype: 'asPanelTop10',
                items: [me.gridFileList]
            }],
            fbar: [{
                xtype: 'button',
                text: _S('COMMON', 'CLOSE'),
                handler: function() {
                    win.close();
                }
            }],
            listeners: {
                afterrender: function() {
                    me.gridFileList.el.mask(_S('COMMON', 'LOADING'));
                    Ext.defer(function() {
                        me.loadFailedList();
                        me.gridFileList.el.unmask();
                    }, 500);
                }
            }
        });

        win.show();
    },

    getStore: function(data) {
        return {
            store: 'array',
            fields: ['id', 'path', 'size', 'type', 'error_msg'],
            data: data
        };
    },

    loadFailedList: function() {
        var me = this;
        me.gridFileList.getStore().loadData(me.getData());
    },

    setData: function(data) {
        this.fetchedData = Ext.isEmpty(data) ? [] : data;

        var message = (this.fetchedData.length === 1) ? _S('DATASYNC_CENTER', 'ONE_FILE_CANNOT_BE_SYNCHRONIZED') : _S('DATASYNC_CENTER', 'N_FILES_CANNOT_BE_SYNCHRONIZED');
        this.down('#link').setValue(
            Ext.String.format(message, '<a class="hyperlink-string" href="#">', '</a>', this.fetchedData.length)
        );
    },

    getData: function() {
        return this.fetchedData;
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.remoteTreeSelector
 * @extends Object
 * @author JerryJiang 2019-09-03
 */
Ext.define('AS.ARC.apps.datasynccenter.remoteTreeSelector', {
    extend: 'Object',

    //define msg show error type
    MSG_TYPE_ERROR: 'error',
    //define msg show warning type
    MSG_TYPE_WARNING: 'warning',
    API_URL: AS.ARC.util.getUserAppsPath() + 'datasync-center/datasync_center.cgi',
    
    expandList: [],
    cancelAjax: true,

    constructor: function(config) {
        Ext.apply(this, config);
        this.callParent();

        this.init();
        this.createWindow();
    },

    init: function() {
        this.expandList = [];
    },

    abortRequest: function() {
        var fs = this;
        if (Ext.isEmpty(fs.request)) {
            return;
        }
        Ext.Ajax.abort(fs.request);
    },

    maskTreePanel: function() {
        var fs = this,
            mask;

        if (fs.cancelAjax) {
            mask = fs.locationTree.el.mask(Ext.String.format(_S('DATASYNC_CENTER', 'PLEASE_WAIT_OR_CANCEL'), '<a class="hyperlink-string" href="#">', '</a>'));
            mask.next().on('click', function() {
                fs.abortRequest();
                fs.resetExpandingNode(fs.selectedNode);
                fs.unmaskTreePanel();
            }, this, {
                    delegate: '.hyperlink-string'
                });
        }
    },

    unmaskTreePanel: function() {
        var fs = this;

        if (fs.cancelAjax) {
            fs.locationTree.el.unmask();
        }
        
        fs.btnRefresh.enable();
        fs.btnCreate.setDisabled(!fs.hasSelected());
    },

    renderTreeNodes: function(node, callback) {
        var fs = this,
            urlOpts = {
                act: 'list_folder',
                connection_id: fs.connectionId,
                remote_id: node.data.remote_id
            };
        
        node.removeAll();
        node.set('loading', true);
        fs.btnRefresh.disable();
        fs.btnCreate.disable();
        fs.selectedNode = node;
        fs.maskTreePanel();
        fs.request = AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fs.API_URL, urlOpts),
            method: 'post',
            success: function(json, opts) {
                var lists = json.list;
                Ext.each(lists, function(data) {
                    var nodeConfig = {
                        text: Ext.htmlEncode(data.path),
                        leaf: false,
                        qtip: Ext.htmlEncode(Ext.htmlEncode(data.path)),
                        
                        remote_id: data.remote_id,
                        path: node.data.path + '/' + data.path
                    }

                    node.appendChild(nodeConfig);
                });

                node.set('loading', false);
                setTimeout(function() {
                    Ps.update(fs.locationTree.body.dom);
                    fs.unmaskTreePanel();
                    Ext.callback(callback);
                }, 200);
            },
            failure: function(json, opts) {
                fs.resetExpandingNode(node);
                fs.unmaskTreePanel();
                AS.ARC.util.showMsgWindow({
                    5001: _S('DATASYNC_CENTER', 'ACCOUNT_REVOKED_EXPIRED')
                }, json, fs.win);
            }
        });
    },

    createLocationTree: function() {
        var fs = this;
        var treeContextMenu = Ext.create('Ext.menu.Menu', {
            items: [{
                xtype: 'menuitem',
                text: _S('COMMON', 'CREATE_FOLDER'),
                scope: this,
                handler: fs.openCreateFolderWin
            }]
        });

        fs.locationTree = Ext.create('Ext.tree.TreePanel', {
            cls: fs.cls ? fs.cls : '',
            rootVisible: false,
            useArrows: true,
            autoScroll: true,
            height: 250,
            flex: 1,
            resizable: false,
            scope: this,
            listeners: {
                afterrender: function(com, eOpts) {
                    //treePandel's root node
                    var rootNode = {
                        text: '',
                        expanded: true,

                        remote_id: '',
                        path: ''
                    };
                    fs.locationTree.setRootNode(rootNode);
                    fs.expandList.push(fs.locationTree.getRootNode());
                    fs.renderTreeNodes(fs.locationTree.getRootNode());
                },
                itemcontextmenu: function(view, record, item, index, event) {
                    event.stopEvent();
                    fs.selectedNode = record;

                    fs.btnCreate.setDisabled(false);
                    x = event.browserEvent.clientX;
                    y = event.browserEvent.clientY;
                    treeContextMenu.showAt([x, y]);
                    view.select(index);
                },
                itemexpand: function(node, eOpts) {
                    fs.locationTree.view.select(node);
                    if (fs.expandList.indexOf(node) === -1) {
                        fs.expandList.push(node);
                        fs.renderTreeNodes(node);
                    }
                },
                select: function(model, record) {
                    fs.selectedNode = record;
                    fs.btnCreate.setDisabled(false);
                }
            }
        });

        return fs.locationTree;
    },

    hasSelected: function() {
        return this.locationTree.getSelectionModel().getSelection().length > 0 ? true : false;
    },

    changeValue: function() {
        var fs = this;
        fs.objectPath.setValue(fs.selectedNode.data.path);
        fs.objectPath.remoteId = fs.selectedNode.data.remote_id;
    },

    createWindow: function() {
        var fs = this;
        var buildTree = fs.createLocationTree();
        fs.btnCreate = Ext.create(Ext.button.Button, {
            text: _S('COMMON', 'CREATE_FOLDER'),
            scale: AS.ARC.config.defaultActionButtonSize,
            handler: fs.openCreateFolderWin,
            scope: this,
            disabled: true
        });

        fs.btnRefresh = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'REFRESH'),
            scale: AS.ARC.config.defaultActionButtonSize,
            handler: function() {
                var rootNode = fs.locationTree.getRootNode();
                fs.locationTree.getSelectionModel().deselectAll();
                fs.expandList = [];
                fs.expandList.push(rootNode);

                rootNode.expand();
                fs.renderTreeNodes(rootNode);
            },
            disabled: true
        });

        fs.btnOk = Ext.create(Ext.button.Button, {
            text: _S('COMMON', 'OK'),
            handler: function() {
                if (fs.selectedNode.data.path !== '') {
                    fs.objectPath.remoteId = fs.selectedNode.data.remote_id;
                    fs.changeValue();
                    fs.win.close();
                } else {
                    AS.ARC.util.msgDialog(fs.win, fs.MSG_TYPE_ERROR, _S('COMMON', 'SELECT_FOLDER'));
                }
            }
        });

        fs.win = Ext.create('AS.ARC.subWindow', {
            title: _S('COMMON', 'DIRECTORY'),
            parentWin: fs.parentWin,
            width: 500,
            height: 420,
            scope: this,
            fbar: [
                fs.btnOk, {
                    text: _S('COMMON', 'CANCEL'),
                    handler: function() { fs.win.close(); }
                }],
            items: [{
                xtype: 'asPanelTop10',
                height: 320,
                items: [{
                    xtype: 'asPanelToolbarTop',
                    items: [fs.btnRefresh, fs.btnCreate]
                },
                    buildTree
                ]
            }]
        });

        fs.win.show();
    },

    openCreateFolderWin: function() {
        var fs = this,
            node = fs.locationTree.getSelectionModel().getLastSelected();

        if (Ext.isEmpty(fs.selectedNode) || fs.selectedNode.data.path === '') {
            AS.ARC.util.msgDialog(fs.win, fs.MSG_TYPE_ERROR, _S('COMMON', 'SELECT_FOLDER'));
            return false;
        }

        var runCreateFolder = function() {
            folderText.disable();
            var cb = function() {
                folderText.enable();
            }

            if (folderText.getValue().trim().length === 0) {
                AS.ARC.util.msgDialog(win, fs.MSG_TYPE_ERROR, _S('MESSAGE', 'HF00014'), cb);
                return false;
            }

            win.el.mask(_S('COMMON', 'APPLYING'));
            var urlOpts = {
                act: 'make_dir',
                connection_id: fs.connectionId,
                remote_id: node.data.remote_id,
                name: folderText.getValue().trim()
            };

            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(fs.API_URL, urlOpts),
                method: 'post',
                success: function(json, opts) {
                    win.close();
                    if (fs.expandList.indexOf(node) === -1) {
                        fs.expandList.push(node);
                        node.expand();
                        fs.renderTreeNodes(node, function() {
                            var createdNode = node.findChild('remote_id', json.folder.remote_id),
                                nodeDom = fs.locationTree.view.getNode(createdNode);
                            
                            fs.locationTree.view.select(createdNode);
                            fs.locationTree.body.dom.scrollTop = nodeDom.offsetTop;
                        });
                    } else {
                        node.expand();
                        var createdNode = node.appendChild({
                            text: Ext.htmlEncode(json.folder.path),
                            leaf: false,
                            qtip: Ext.htmlEncode(Ext.htmlEncode(json.folder.path)),
                            
                            remote_id: json.folder.remote_id,
                            path: node.data.path + '/' + json.folder.path
                        });

                        var nodeDom = fs.locationTree.view.getNode(createdNode);
                        fs.locationTree.view.select(createdNode);
                        Ext.defer(function() {
                            fs.locationTree.body.dom.scrollTop = nodeDom.offsetTop;
                        }, 500);
                        
                    }
                    
                },
                failure: function(json, opts) {
                    win.el.unmask();
                    AS.ARC.util.showMsgWindow({
                        5001: _S('DATASYNC_CENTER', 'ACCOUNT_REVOKED_EXPIRED')
                    }, json, win, 'warning', cb);
                }
            });
        };

        var buttonApply = Ext.create('Ext.button.Button', {
            disabled: true,
            text: _S('COMMON', 'OK'),
            handler: function() {
                runCreateFolder();
            }
        });

        var folderText = Ext.create(Ext.form.field.Text, {
            fieldLabel: _S('COMMON', 'NEW_NAME'),
            width: 350,
            itemId: 'name',
            blankText: _S('MESSAGE', 'HF00011'),
            allowBlank: false,
            msgTarget: AS.ARC.config.msgTarget,
            listeners: {
                validitychange: function(field, isValid, eOpts) {
                    if (isValid) {
                        buttonApply.setDisabled(false);
                    } else {
                        buttonApply.setDisabled(true);
                    }
                },
                specialkey: function(field, e) {
                    if (e.getKey() == e.ENTER && !field.disabled && field.isValid()) {
                        runCreateFolder();
                    }
                }
            }
        });

        var win = Ext.create('AS.ARC.subWindow', {
            title: _S('COMMON', 'CREATE_FOLDER'),
            width: 400,
            height: 150,
            parentWin: fs.win,
            fbar: [
                buttonApply, {
                    text: _S('COMMON', 'CANCEL'),
                    handler: function() { win.close(); }
                }],
            items: [{
                xtype: 'asPanelTop10',
                itemId: 'panel',
                buttonApply: buttonApply,
                defaults: {
                    labelWidth: AS.ARC.config.layout.defaultLabelWidth,
                    msgTarget: AS.ARC.config.msgTarget,
                    blankText: _S('MESSAGE', 'HF00011')
                },
                height: 50,
                labelWidth: 50,
                items: [folderText]
            }],
            listeners: {
                show: function(win) {
                    win.down('#name').focus(true, 10);
                }
            }
        });

        win.show();
    },

    resetExpandingNode: function(node) {
        var fs = this;
        node.collapse();
        node.set('loading', false);
        node.set('loaded', false);
        node.set('expanded', false);
        if (fs.expandList.indexOf(node) !== -1) {
            fs.expandList.splice(fs.expandList.indexOf(node), 1);
        }
    }
});

/**
 * Copy from AS.ARC.treeSelector and fix #31926
 * @class AS.ARC.apps.datasynccenter.localTreeSelector
 * @extends AS.ARC.treeSelector
 */
Ext.define('AS.ARC.apps.datasynccenter.localTreeSelector', {
    extend: 'AS.ARC.treeSelector',

	PATH_USER_HOME: '/User Homes',

    unselectableFolder: ['/User Homes'],

    unmaskTreePanel: function() {
        var fs = this;

        if (fs.cancelAjax) {
            fs.locationTree.el.unmask();
        }

        fs.btnRefresh.enable();
        fs.btnCreate.setDisabled(Ext.isEmpty(fs.locationTree.getSelectionModel().getLastSelected()));
    },

    createTreeNodes: function(rootNode) {
        var fs = this;
        rootNode.removeAll();
        //create all share_folders treeNodes use ajax
        var urlOpts = {
            act: 'file_list',
            path: 'share_folders'
        };

        if (fs.isShowHome)
            Ext.apply(urlOpts, { showhome: true });

        fs.locationTree.body.mask(_S('COMMON', 'LOADING'));
        AS.ARC.ajax({
            url: AS.ARC.util.getApiUrlWithSid(fs.FEUrl, urlOpts),
            method: 'post',
            success: function(json, opts) {
                fs.locationTree.body.unmask();
                var datas = json.data;
                if (datas.length === 0) {
                    var noFolderWin = Ext.create('AS.ARC.msgWindow', {
                        title: _S('COMMON', 'INFORMATION'),
                        parentWin: fs.win,
                        height: 200,
                        iconType: 'info', // info, question, warning
                        asItems: [{
                            xtype: 'displayfield',
                            value: _S('ERROR', 'NO_ACCESS_RIGHT_SHARE_FOLDERS')
                        }],
                        fbar: [{
                            text: _S('COMMON', 'OK'),
                            handler: function() { noFolderWin.close(); }
                        }]
                    }).show();
                }
                for (var i = 0; i < datas.length; i++) {
                    var myarchiveFormat = '/volume1/MyArchive',
                        // path = datas[i].phy_path;
                        path = datas[i].physical_path,
                        isUserHomes = datas[i].file_path === fs.PATH_USER_HOME;
                    var isMyArchive = function() {
                        array = path.split(myarchiveFormat);
                        if (array.length === 2) {
                            if ((array[1] !== '') && (array[1] <= AS.ARC.config.bay - 1)) {
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    };
                    if (fs.disableMyArchive && isMyArchive()) {
                        continue;
                    } else if (isUserHomes && AS.ARC.user.account !== 'admin') {
                        continue;
                    }

                    if (fs.onlyShowHome) {
                        if (path.indexOf('/home/') === -1) {
                            continue;
                        }
                    }

                    if (!fs.hideReadOnly || !fs.isReadOnly(datas[i].access_permission)) {
                        if (datas[i].share_folder_type != fs.SHARE_FOLDERS_TYPE_MOUNT) {
                            var nodeConfig = {
                                text: Ext.htmlEncode(datas[i].filename),
                                leaf: false,
                                id: datas[i].file_path,
                                type: datas[i].share_folder_type,
                                phy_path: datas[i].physical_path,
                                qtip: Ext.htmlEncode(Ext.htmlEncode(datas[i].filename))
                            }
                        } else {
                            var nodeConfig = {
                                text: Ext.htmlEncode(datas[i].filename),
                                leaf: false,
                                id: datas[i].file_path,
                                type: datas[i].share_folder_type,
                                phy_path: datas[i].physical_path,
                                qtip: Ext.htmlEncode(Ext.htmlEncode(datas[i].filename)),
                                icon: AS.ARC.util.fixDc('apps/fileExplorer/images/files_list/iso.png'),
                            }
                        }
                        rootNode.appendChild(nodeConfig);
                    }
                }
                setTimeout(function() {
                    var panelBody = fs.win.el.dom.querySelectorAll('.x-panel-body')[2];
                    Ps.update(panelBody);
                    fs.btnRefresh.enable();
                }, 200);
            },
            failure: function(json, opts) {
                fs.btnRefresh.enable();
                AS.ARC.util.msgDialog(fs.win, fs.MSG_TYPE_ERROR, json.error_msg);
            }
        });
    },

    createLocationTree: function() {
        var fs = this;

        //create tree mount context menu
        var treeContextMenu = Ext.create('Ext.menu.Menu', {
            items: [{
                xtype: 'menuitem',
                text: _S('COMMON', 'CREATE_FOLDER'),
                scope: this,
                handler: fs.openCreateFolderWin
            }]
        });

        // create the Tree
        fs.locationTree = Ext.create('Ext.tree.TreePanel', {
            cls: fs.cls ? fs.cls : '',
            rootVisible: false,
            useArrows: true,
            autoScroll: true,
            height: 250,
            flex: 1,
            resizable: false,
            scope: this,
            listeners: {
                afterrender: function(com, eOpts) {
                    //treePandel's root node
                    var rootNodes = {
                        text: 'root',
                        expanded: true,
                        id: ''
                    };
                    fs.locationTree.setRootNode(rootNodes);
                    fs.createTreeNodes(fs.locationTree.getRootNode());
                },
                beforeitemcontextmenu: function(view, record, item, index, event) {
                    return fs.isSelectable(record.data.id);
                },
                itemcontextmenu: function(view, record, item, index, event) {
                    event.stopEvent();
                    fs.selectedNode = record;
                    fs.selectPath = record.data.id;
                    fs.volumePath = record.data.phy_path;

                    fs.btnCreate.setDisabled(false);
                    x = event.browserEvent.clientX;
                    y = event.browserEvent.clientY;
                    treeContextMenu.showAt([x, y]);
                    view.select(index);
                },
                select: function(model, record) {
                    fs.selectedNode = record;
                    fs.selectPath = record.data.id;
                    fs.volumePath = record.data.phy_path;
                    fs.btnCreate.enable();
                },
                itemexpand: function(node, eOpts) {
                    fs.locationTree.view.select(node);
                    //get folder's node list
                    if (fs.expandList.indexOf(node) === -1) {
                        fs.expandList.push(node);
                        node.removeAll();
                        node.set('loading', true);
                        var urlOpts = {
                            act: 'file_list',
                            path: node.data.id,
                            onlydir: true
                        };

                        fs.maskTreePanel();
                        fs.request = AS.ARC.ajax({
                            url: AS.ARC.util.getApiUrlWithSid(fs.FEUrl, urlOpts),
                            method: 'post',
                            success: function(json, opts) {
                                var datas = json.data;
                                for (var i = 0; i < datas.length; i++) {
                                    var folderName = fs.getFolderName(datas[i].file_path);

                                    var nodeConfig = {
                                        text: Ext.htmlEncode(folderName).replace(/\u0020/ig, '&nbsp;'),
                                        leaf: false,
                                        id: datas[i].file_path,
                                        type: datas[i].share_folder_type,
                                        phy_path: node.data.phy_path + '/' + folderName,
                                        qtip: Ext.htmlEncode(Ext.htmlEncode(folderName))
                                    }
                                    node.appendChild(nodeConfig);
                                }

                                node.set('loading', false);

                                setTimeout(function() {
                                    var panelBody = fs.win.el.dom.querySelectorAll('.x-panel-body')[2];
                                    Ps.update(panelBody);
                                    fs.unmaskTreePanel();
                                }, 200);

                            },
                            failure: function(json, opts) {
                                node.set('loading', false);
                                fs.unmaskTreePanel();
                                AS.ARC.util.msgDialog(fs.win, fs.MSG_TYPE_ERROR, json.error_msg);
                            }
                        });
                    }
                },
                beforeselect: function(node, record, index, eOpts) {
                    return fs.isSelectable(record.data.id);
                }
            }
        });

        return fs.locationTree;
    },

    createWindow: function() {
        var fs = this;
        var buildTree = fs.createLocationTree();
        fs.selectPath = '';
        fs.volumePath = '';
        fs.btnCreate = Ext.create(Ext.button.Button, {
            text: _S('COMMON', 'CREATE_FOLDER'),
            scale: AS.ARC.config.defaultActionButtonSize,
            handler: fs.openCreateFolderWin,
            scope: this,
            disabled: true
        });

        fs.btnRefresh = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'REFRESH'),
            scale: AS.ARC.config.defaultActionButtonSize,
            handler: function() {
                fs.btnRefresh.disable();
                fs.btnCreate.disable();
                fs.selectPath = '';
                fs.volumePath = '';
                fs.createTreeNodes(fs.locationTree.getRootNode());
            },
            hidden: !fs.hasRefresh,
            disabled: true
        });

        fs.btnOk = Ext.create(Ext.button.Button, {
            text: _S('COMMON', 'OK'),
            handler: function() {
                if (fs.selectPath != '') {
                    fs.objectPath.volumePath = fs.volumePath;

                    if (fs.type == 'volume') {
                        fs.changeValue(fs.volumePath);
                    } else {
                        if (fs.selectPath.indexOf('/home/') === 0) {
                            var replacePath = fs.selectPath.replace('/' + AS.ARC.user.account, '');
                            if (fs.hideUserName === false) {
                                fs.changeValue(replacePath + ' [' + AS.ARC.user.account + ']');
                            } else {
                                replacePath = replacePath.replace('home', 'Home');
                                fs.changeValue(replacePath);
                            }
                        } else {
                            fs.changeValue(fs.selectPath);
                        }
                    }
                    fs.win.close();
                } else {
                    AS.ARC.util.msgDialog(fs.win, fs.MSG_TYPE_ERROR, _S('COMMON', 'SELECT_FOLDER'));
                }
            }
        });

        fs.win = Ext.create('AS.ARC.subWindow', {
            title: _S('COMMON', 'DIRECTORY'),
            parentWin: fs.parentWin,
            width: 500,
            height: 420,
            scope: this,
            fbar: [
                fs.btnOk, {
                    text: _S('COMMON', 'CANCEL'),
                    handler: function() { fs.win.close(); }
                }],
            items: [{
                xtype: 'asPanelTop10',
                height: 320,
                items: [{
                    xtype: 'asPanelToolbarTop',
                    items: [fs.btnRefresh, fs.btnCreate]
                },
                    buildTree
                ]
            }]
        });

        fs.win.show();
    },

    openCreateFolderWin: function() {
        var fs = this;

        var node = fs.locationTree.getSelectionModel().getSelection()[0];
        node.expand();

        if (fs.CREATEFOLDER_WIN_NUMBER < fs.MAX_CREATEFOLDER_WIN) {
            if (fs.selectPath != '') {
                var runCreateFolder = function() {
                    folderText.disable();

                    var cb = function() {
                        folderText.enable();
                    }

                    var checkFolder = folderText.getValue().trim();
                    if (checkFolder.length == 0) {

                        AS.ARC.util.msgDialog(win, fs.MSG_TYPE_ERROR, _S('MESSAGE', 'HF00014'), cb);
                    } else {

                        win.el.mask(_S('COMMON', 'APPLYING'));
                        var urlOpts = {
                            act: 'createdir',
                            dest_path: fs.selectPath.trim(),
                            dest_folder: folderText.getValue().trim()
                        };

                        AS.ARC.ajax({
                            url: AS.ARC.util.getApiUrlWithSid(fs.FEUrl, urlOpts),
                            method: 'post',
                            success: function(json, opts) {

                                var child = node.appendChild({
                                    text: folderText.getValue().replace(/\u0020/ig, '&nbsp;'),
                                    leaf: false,
                                    id: node.data.id + '/' + folderText.getValue().trim(),
                                    type: 2,
                                    phy_path: node.data.phy_path + "/" + folderText.getValue().trim(),
                                    qtip: folderText.getValue().trim()
                                });

                                var sm = fs.locationTree.getSelectionModel();
                                sm.select(child);
                                fs.selectPath = node.data.id + '/' + folderText.getValue().trim();


                                fs.saveParams();
                                win.close();

                                var nodeDom = fs.locationTree.view.getNode(child);
                                /**
                                 * Scroll to view the new appended child in the middle of the treepanel.
                                 * The half of height of the treepanel.body is 125.
                                 * The half of height of the nodeDom is 17.
                                 */
                                fs.locationTree.body.dom.scrollTop = nodeDom.offsetTop - 125 + 17;
                            },
                            failure: function(json, opts) {
                                win.el.unmask();
                                var errors = {
                                    5008: _S('ERROR', 'UNABLE_CREATE_ALREADY_FOLDER'),
                                    5011: _S('ERROR', 'UNABLE_MODIFY_FILE_PROPERTIES'),
                                    5019: _S('ERROR', 'CANOT_CREATE_FOLDER'),
                                    5025: _S('ERROR', 'NOT_ALLOWED_OPERATION'),
                                    5026: _S('ERROR', 'READ_ONLY_FILE'),
                                    5045: _S('ERROR', 'MAXIMUN_USAGE_QUOTA'),
                                    5605: _S('ERROR', 'NO_SPACE_DISK')
                                };

                                AS.ARC.util.showMsgWindow(errors, json, win, 'warning', cb);
                            }
                        });
                    }
                };

                var buttonApply = Ext.create('Ext.button.Button', {
                    disabled: true,
                    text: _S('COMMON', 'OK'),
                    handler: function() {
                        runCreateFolder();
                    }
                });

                var folderText = Ext.create(Ext.form.field.Text, {
                    fieldLabel: _S('COMMON', 'NEW_NAME'),
                    width: 350,
                    itemId: 'name',
                    vtype: 'createFolder',
                    blankText: _S('MESSAGE', 'HF00011'),
                    allowBlank: false,
                    msgTarget: AS.ARC.config.msgTarget,
                    listeners: {
                        validitychange: function(field, isValid, eOpts) {
                            if (isValid) {
                                buttonApply.setDisabled(false);
                            } else {
                                buttonApply.setDisabled(true);
                            }
                        },
                        specialkey: function(field, e) {
                            if (e.getKey() == e.ENTER && !field.disabled && field.isValid()) {
                                runCreateFolder();
                            }
                        }
                    }
                });

                var win = Ext.create('AS.ARC.subWindow', {
                    title: _S('COMMON', 'CREATE_FOLDER'),
                    width: 400,
                    height: 150,
                    parentWin: fs.win,
                    fbar: [
                        buttonApply, {
                            text: _S('COMMON', 'CANCEL'),
                            handler: function() {
                                win.close();
                            }
                        }],
                    items: [{
                        xtype: 'asPanelTop10',
                        itemId: 'panel',
                        buttonApply: buttonApply,
                        defaults: {
                            labelWidth: AS.ARC.config.layout.defaultLabelWidth,
                            msgTarget: AS.ARC.config.msgTarget,
                            blankText: _S('MESSAGE', 'HF00011')
                        },
                        height: 50,
                        labelWidth: 50,
                        items: [folderText]
                    }],
                    listeners: {
                        show: function(win) {
                            win.items.get('panel').items.get('name').focus(true, 10);
                        },
                        afterrender: function(com, eOpts) {
                            fs.CREATEFOLDER_WIN_NUMBER++;
                        },
                        beforeclose: function() {
                            fs.CREATEFOLDER_WIN_NUMBER--;
                        }
                    }
                });

                win.show();
            } else {
                AS.ARC.util.msgDialog(fs.win, fs.MSG_TYPE_ERROR, _S('COMMON', 'SELECT_FOLDER'));
            }
        }
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.ezsyncLoginWin
 * @extends AS.ARC.subWindow
 * @author JerryJiang 2019-11-13
 */
Ext.define('AS.ARC.apps.datasynccenter.ezsyncLoginWin', {
    extend: 'AS.ARC.subWindow',

    API_URL: AS.ARC.util.getUserAppsPath() + 'datasync-center/datasync_center.cgi',

    width: 690,
    height: 430,
    ignoreVerifySSL: false,
    isReauthorize: false,

    initComponent: function(config) {
        this.init();
        this.callParent(arguments);
    },

    init: function() {
        var win = this;
        win.title = _S('DATASYNC_CENTER', 'LOGIN') + ' - ' + _UN_TS('ASUSTOR NAS');
        win.buttonApply = Ext.create('Ext.button.Button', {
            text: _S('DATASYNC_CENTER', 'LOGIN'),
            handler: win.handleApply,
            scope: win
        });

        win.buttonCancel = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'CANCEL'),
            handler: function() {
                win.close();
            },
            scope: win
        });

        win.fbar = [
            win.buttonApply,
            win.buttonCancel
        ];
        win.items = [win.getItems()];
    },

    getItems: function() {
        var win = this;
        return [{
            xtype: 'asPanelTop20',
            buttonApply: this.buttonApply,
            ignoreCheckForm: true,
            defaults: {
                labelWidth: 230,
                blankText: _S('MESSAGE', 'HF00011'),
                maxLengthText: _S('MESSAGE', 'HF00016'),
                msgTarget: AS.ARC.config.msgTarget,
            },
            items: [{
                xtype: 'textfield',
                fieldLabel: _S('DATASYNC_CENTER', 'ADDRESS'),
                itemId: 'address',
                allowBlank: false,
                width: 500,
                listeners: {
                    change: function(me) {
                        var port = win.down('#port'),
                            https = win.down('#https');
                            addressType = win.classifyAddress(me.getValue());
                        if (win.isReauthorize) return false;
                        https.setDisabled(addressType.is_cloud_id);
                        port.setDisabled(addressType.is_cloud_id);
                    }
                },
                disabled: win.isReauthorize
            }, {
                xtype: 'textfield',
                fieldLabel: _S('COMMON', 'ACCOUNT'),
                itemId: 'account',
                allowBlank: false,
                width: 500,
                disabled: win.isReauthorize
            }, {
                xtype: 'textfield',
                fieldLabel: _S('COMMON', 'PWD'),
                itemId: 'password',
                inputType: 'password',
                allowBlank: false,
                width: 500,
            }, {
                xtype: 'checkbox',
                itemId: 'https',
                boxLabel: _S('DATASYNC_CENTER', 'HTTPS'),
                listeners: {
                    change: function(me) {
                        var port = win.down('#port');
                        if (win.isReauthorize) return false;
                        if (!port.isDisabled()) {
                            port.setValue(me.getValue() ? '8001' : '8000');
                        }
                    }
                },
                disabled: win.isReauthorize
            }, {
                xtype: 'textfield',
                fieldLabel: _S('DATASYNC_CENTER', 'PORT'),
                itemId: 'port',
                value: '8000',
                width: 500,
                disabled: win.isReauthorize
            }, {
                xtype: 'textfield',
                itemId: 'verify-code',
                emptyText: _S('TWOSTEP', 'STEP_THREE_CODE'),
                maskRe: /[0-9]/,
                fieldLabel: _S('DATASYNC_CENTER', 'VERIFICATION_CODE'),
                allowBlank: false,
                maxLength: 6,
                enforceMaxLength: true,
                width: 500,
                disabled: true,
                hidden: true
            }, {
                xtype: 'displayfield',
                itemId: 'error-message',
                cls: 'displayfield note-string',
                style: 'margin-top: 35px;',
            }]
        }];
    },

    classifyAddress: function(value) {
        var isIp = AS.ARC.re.ipv4Simple.test(value),
            isCloudId = (value.indexOf('.') === -1) && !Ext.isEmpty(value),
            isDomain = !isIp && !isCloudId;
        return {
            is_cloud_id: isCloudId,
            is_ip: isIp,
            is_domain: isDomain
        };
    },

    getConfirmVerifySSLWinodw: function() {
        var win = this;
        var buttonOk = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'OK'),
            handler: function() {
                winConfirm.close();
                win.ignoreVerifySSL = true;
                win.handleApply();
            },
            scope: this
        });

        var buttonCancel = Ext.create('Ext.button.Button', {
            text: _S('COMMON', 'CANCEL'),
            handler: function() { winConfirm.close(); },
            scope: this
        });

        var winConfirm = new AS.ARC.msgWindow({
            parentWin: win,
            title: _S('COMMON', 'WARNING'),
            width: 400,
            height: 180,
            iconType: 'question',
            asItems: [{
                xtype: 'displayfield',
                value: _S('DATASYNC_CENTER', 'NOT_PRIVATE_CONTINUE')
            }],
            fbar: [
                buttonOk,
                buttonCancel
            ]
        });

        return winConfirm;
    },

    handleApply: function() {
        var win = this,
            errorMessage = win.down('#error-message');

        errorMessage.hide().setValue('');
        if (win.down('asPanelTop20').getForm().isValid()) {
            var addressType = win.classifyAddress(win.down('#address').getValue());
            var params = {};
            if (win.isReauthorize) {
                Ext.apply(params, {
                    "address": win.fetchedData.address,
                    "port": win.fetchedData.port,
                    "is_https": win.fetchedData.is_https,
                    "is_cloud_id": win.fetchedData.is_cloud_id,
                    "account": win.fetchedData.account,
                    "password": win.down('#password').getValue(),
                    "verifycode": win.down('#verify-code').getValue(),
                    "verify_ssl": win.fetchedData.verify_ssl,
                });
            } else {
                var params = {
                    "address": win.down('#address').getValue(),
                    "port": win.down('#port').getValue(),
                    "is_https": win.down('#https').getValue(),
                    "is_cloud_id": addressType.is_cloud_id,
                    "account": win.down('#account').getValue(),
                    "password": win.down('#password').getValue(),
                    "verifycode": win.down('#verify-code').getValue(),
                    "verify_ssl": addressType.is_domain ? true : false,
                };
            }

            if (win.ignoreVerifySSL) {
                win.ignoreVerifySSL = false;
                Ext.apply(params, { verify_ssl: false });
            }

            win.el.mask(_S('COMMON', 'APPLYING'));
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(win.API_URL, { act: 'ezsync_login' }),
                params: params,
                method: 'post',
                success: function() {
                    win.close();
                },
                failure: function(json) {
                    var error = {
                        5002: _S('DATASYNC_CENTER', 'LOGIN_FAILED') + ' ' + _S('DATASYNC_CENTER', 'ACCOUNT_EXPIRED_DISABLED'),
                        5005: _S('DATASYNC_CENTER', 'LOGIN_FAILED') + ' ' + _S('DATASYNC_CENTER', 'UNABLE_NAS_VOLUME'),
                        5010: _S('DATASYNC_CENTER', 'CANNOT_CONNECT_TO_SERVER') + ' ' + _S('DATASYNC_CENTER', 'CLOUD_ID_NOT_EXIST'),
                        5014: _S('DATASYNC_CENTER', 'LOGIN_FAILED') + ' ' + _S('DATASYNC_CENTER', 'ACCOUNT_PASSWORD_INCORRECT'),
                        5034: 'Need verify code',
                        5601: _S('DATASYNC_CENTER', 'CANNOT_CONNECT_TO_SERVER') + ' ' + _S('DATASYNC_CENTER', 'PLEASE_CHECK_NAS_SETTINGS'),
                        5604: 'HTTPs certificate error.',
                        5701: _S('DATASYNC_CENTER', 'CANNOT_CONNECT_TO_SERVER') + ' ' + _S('DATASYNC_CENTER', 'BE_BLOCKED_FAILED_LOGIN'),
                        6217: _S('DATASYNC_CENTER', 'LOGIN_FAILED') + ' ' + _S('DATASYNC_CENTER', 'ADM_FIRMWARE_VERSION_NOT_COMPATIBLE')
                    };

                    win.el.unmask();
                    if (json.error_code === 5034) {
                        win.down('#address').hide().disable();
                        win.down('#account').hide().disable();
                        win.down('#password').hide().disable();
                        win.down('#https').hide().disable();
                        win.down('#port').hide().disable();
                        win.down('#verify-code').show().enable();
                    } else if (json.error_code === 5604) {
                        win.getConfirmVerifySSLWinodw().show();
                    } else if (!Ext.isEmpty(error[json.error_code])){
                        errorMessage.setValue(error[json.error_code]).show();
                    }
                }
            });
        }
    },

    listeners: {
        afterrender: function(win) {
            if (!win.isReauthorize) return false;
            win.el.mask(_S('COMMON', 'LOADING'));
            AS.ARC.ajax({
                url: AS.ARC.util.getApiUrlWithSid(win.API_URL, { act: 'ezsync_connection_info' }),
                params: {
                    connection_id: win.connectionId
                },
                method: 'post',
                success: function(json) {
                    win.el.unmask();
                    win.fetchedData = json;
                    win.down('#address').setValue(json.address);
                    win.down('#account').setValue(json.account);
                    win.down('#https').setValue(json.is_https);
                    win.down('#port').setValue(json.port);
                },
                failure: function(json) {
                    AS.ARC.util.showMsgWindow({}, json, win);
                }
            });
        }
    }
});

/**
 * @class AS.ARC.apps.datasynccenter.core
 * @extends Ext.util.Observable
 */
Ext.define('AS.ARC.apps.datasynccenter.core', {
    extend: 'Ext.util.Observable',

    apiUrl: AS.ARC.util.getUserAppsPath() + 'datasync-center/datasync_center.cgi',

    constructor: function(config) {
        Ext.apply(this, config);
        this.callParent();
        this.init();
    },

    init: function() {
        var fn = this;
        fn.win = fn.desktop.createWindow({
            app: fn.app,
            id: fn.id,
            itemId: fn.id,
            width: 880,
            height: 520,
            resizable: true,
            border: false,
            layout: 'border',
            bodyStyle: 'padding: 0; border: none;',
            items: [
                fn.getMainPanel(),
                fn.getWestPanel()
            ],
            listeners: {
                afterrender: function (win) {
                    win.header.addCls('as-window-header-no-tab');
                    win.header.items.items[1].hide();
                    win.el.dom.classList.add('as-fnunction-panel-has-tab');

                    fn.syncList.loadSyncList();
                    fn.mainPanel.loadData();
                    fn.mainPanel.doLayout();
                }
            }
        });
    },

    getWestPanel: function() {
        var fn = this;

        fn.syncList = Ext.create('AS.ARC.apps.datasynccenter.syncList', { fn: fn });
        fn.syncListToolbar = Ext.create('AS.ARC.apps.datasynccenter.syncListToolbar', { fn: fn });

        fn.westPanel = Ext.create('Ext.panel.Panel', {
            cls: 'as-selector-panel',
            page: fn,
            region: 'west',
            split: true,
            maxWidth: 350,
            minWidth: 250,
            layout: 'fit',
            items: [{
                xtype: 'panel',
                itemId: 'main',
                border: false,
                layout: {
                    type: 'vbox',
                    align: 'stretch'
                },
                items: [fn.syncList, fn.syncListToolbar]
            }],
            listeners: {
                resize: function(panel) {
                    Ext.defer(function() {
                        panel.ownerCt.layout.splitters.west.setPosition(panel.el.getWidth());
                    }, 100);
                }
            }
        });

        return fn.westPanel;
    },

    getMainPanel: function() {
        var fn = this;
        fn.syncTabPanel = Ext.create('AS.ARC.apps.datasynccenter.syncTabPanel', { fn: fn });

        var mainPanel = Ext.create('AS.ARC.fn.mainPanel', { items: [fn.syncTabPanel] });

        fn.mainPanel = Ext.create('Ext.panel.Panel', {
            cls: 'as-function-panel',
            region: 'center',
            border: false,
            layout: 'card',
            activeItem: 0,
            items: [mainPanel],
            loadData: function () {}
        });

        return fn.mainPanel;
    },

    openIntroWindow: function() {
        var fn = this;

        var introWin = Ext.create('AS.ARC.subWindow', {
            parentWin: fn.win,
            closable: true,
            id: 'datasync-center-intro-win',
            border: false,
            width: 800,
            height: 520,
            style: {
                "text-align": 'center'
            },
            items: [{
                xtype: 'component',
                autoEl: {
                    tag: 'div',
                    cls: 'intro-win-image',
                    style: {
                        "margin": '0px -40px',
                        "height": "250px"
                    }
                }
            }, {
                xtype: 'component',
                autoEl: {
                    tag: 'div',
                    style: {
                        "margin-top": '22px',
                        "font-size": '24px',
                        "font-weight": 'bold'
                    },
                    html: Ext.String.format(_S('DATASYNC_CENTER', 'WELCOME_TITLE'), _S('DATASYNC_CENTER', 'DATASYNC_CENTER'))
                }
            }, {
                xtype: 'component',
                autoEl: {
                    tag: 'div',
                    style: {
                        "margin-top": '15px',
                        "min-height": '50px'
                    },
                    html: _S('DATASYNC_CENTER', 'WELCOME_INFO')
                }
            }, {
                xtype: 'button',
                width: 150,
                height: 31,
                style: 'margin-top: 20px',
                text: _S('COMMON', 'START'),
                handler: function() {
                    introWin.close();
                }
            }],
            listeners: {
                destroy: function() {
                    fn.win.el.mask(_S('COMMON', 'LOADING'));
                    Ext.defer(function() {
                        if (Ext.isEmpty(fn.win) || fn.win.isDestroyed) return false;
                        fn.syncListToolbar.handleAdd();
                    }, 500);
                }
            }
        });

        introWin.show();
    },
});

/**
 * @class AS.ARC.apps.datasynccenter.main
 * @extends AS.ARC._appBase
 * <p>Main app</p>
 */
Ext.define('AS.ARC.apps.datasynccenter.main', {
    extend: 'AS.ARC._appBase',

    appTag: 'datasync-center',
    title: 'Datasync Center',
    appMaxNum: 1,
    appOpenNum: 0,
    appIsReady: true,
    appWins: [],

    createWindow: function () {
        var desktop = this.core.getDesktop(),
            app = this;

        if ((this.appOpenNum === this.appMaxNum) || !this.appIsReady) {
            this.appWins[0].show();
            return;
        }

        // Set appIsReady to false
        this.appIsReady = false;

        var newApp = new AS.ARC.apps.datasynccenter.core({
            app: this,
            desktop: desktop,
            id: this.id + '-' + Ext.id()
        });

        newApp.win.on('render', function () {
            app.appOpenNum++;
            app.appIsReady = true;
        });

        newApp.win.on('beforeclose', function () {
            app.appOpenNum--;
            app.appIsReady = true;
            app.appWins.pop();
        });
        newApp.win.show();

        // Get all wins for handling
        this.appWins.push(newApp.win);
    },
    onClose: function () {
        console.log('datasync center close');
    }
});
